import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.serial.*; 
import processing.net.*; 
import java.io.File; 
import java.io.FileInputStream; 
import java.io.FileOutputStream; 
import java.io.IOException; 
import java.util.zip.ZipEntry; 
import java.util.zip.ZipOutputStream; 
import java.util.zip.ZipInputStream; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class NLEDAuroraMatrix extends PApplet {

/*
 MIT License
 
 Copyright (c) 2018 Northern Lights Electronic Design, LLC
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 
 Original Author: Jeffrey Nygaard
 Company: Northern Lights Electronic Design, LLC
 Contact: JNygaard@NLEDShop.com
 Date Updated: December 7, 2018
 Software Version:  1a
 Webpage: www.NLEDShop.com/nledauroramatrix
 Written in Processing v3.4  - www.Processing.org
 
 ============================================================================================================
 
 Supported Devices:
 
 More to be added
 
 NLED Pixel Controller Ion
   Max Pixels: 512 for all synchronous chipsets(CLK & DAT), or for asynch(DAT) 469 or 312 pixels, depends on chipset - will try to update in the future
     WS2813 = 469 pixels,
     WS2812/B = 312 pixels or use WS2813 protocol for 469 pixels,
     APA106, PD9823, UCS2903,UCS2904,UCS1903 =  369 pixels
     WS2801, APA102, AP107, LPD8806 = 512(max) pixels
   No hardware color reordering, use software side color order
   
 NLED Pixel Controller Electron
   Max Pixels: 1024 pixels with all chipsets
   Hardware color ordering for most chipsets, otherwise use software side
   
 ============================================================================================================
 
Changes pre-release to v.1a
  - electron expansion header was missing a listing
  - changed accelerometer movement detection, idle sequence is always slot#1
  - Added software color reordering - RGB to GRB etc
  - optimized some variables
  - Fixed ability to have more pixels loaded in software than device supports. Uploads of larger channel sizes is cropped

============================================================================================================

 Hotkeys:
   Spacebar - Play/pause toggle sequence software preview
   ,(<) is Sequence software preview previous frame
   .(>) is Sequence software preview next frame
   = is aurora command, commands hardware to play the next sequence
   
============================================================================================================
     
  Instructions: Videos and additional documentation will be posted as some point
  1. Start software, either through Processing IDE or with the exported applicaiton for your operating system.
  2. Plug in your controller.
  3. Use the drop down in the upper left and select your controller's COM/serial port. Assigned by the operating system. 
  4. Press the connect button. If it successfully connects the status message will update and the button will change to "Disconnect"
  5. Adjust configurations if required - controller's require their settings such as pixel chipset to be set before use.
    5a. Click the "Configurations" button.
    5b. Use the configuration modules to select your required settings.
    5c. Once all the configuration modules are set, press the "Upload Configurations" button to save them to the hardware controller.
    5d. Close the configurations menu.
  6. Add color sequence files or load a aurora matrix save file that contains color sequences and index settings
  
    Loading a Aurora Matrix save file. Extension .auroramatrix - is a ZIP file, rename to ZIP to extract or edit
    a. Press the "Load File" button, navigate to your .auroramatrix save file and open
    b. The softare will fill in the patch file, index, and color sequences.
    c. Adjust the color sequence and index
    
    Adding Color Sequence Files. Extension .txt or .fileplay - created from NLED AllPixMatrix or other software
    a. Start with defining a patch file. Click the "Load Patch" button and select a patch file(.txt). Patch files are created in a separate program.
    b. On the left side is the index, click the "Load" button for slot#1 and select a FilePlay file
    c. Adjust the speed, in miliseconds. IE 33mS is 30FPS. Adjust playmode and transition if applicable.
    d. Continue clicking the "Add sequence" tile in the index and loading FilePlay files.
    
  7. Once all the patch and color sequence files are defined in the index. Press the "Save File" button to save your changes.
  8. The sequences are ready for upload to the controller. Uploading saves everything to the controllers memory. And are available without any computer or data connection.
  9. If applicable set the Global Intensity as a percentage. This pre-calculates new color data values in software before uploading. Otherwise leave at 100%
  10. Press the "Upload Configurations" button to upload and save the color sequences and index to the hardware controller.
  11. Wait several seconds while the upload is in progress. A progress bar will indicate its progress.
  12. If your controller's configurations are correct the color sequence in slot#1 will start playing as soon as the upload completes.
  13. If the pixels are flickering or white, or just random, your chipset may not be correcty selected.
  14. The color sequences are now loaded and ready to use, the controller can be powered off and removed, the software connection is no longer required.    

============================================================================================================

To Do: --------------------------------------------------------------------------------------------


File Notes: --------------------------------------------------------------------------------------------

  .aurm is the sequence definition files. They store the file names for the sequences and the sequence variables(speed, playmodes, transitions) - never see these they are contained in the .auroramatrix
  .auroramatrix files are standard ZIP files. They store all the individual sequence files, patch file, and sequence definition file
  .txt files are used for both patches(which define the pixel physical layout and order) and the Aurora FilePlay sequence files(which store the actual color data) - should have used custom file extensions but a bit late now
  
  File Saving: 
  The application collects the patch file, the Aurora sequence files, creates a sequence definition file and saves it all into a ZIP folder with the extension .auroramatrix
  Once the save file is created the original patch and sequence files can be moved or deleted. All of that is stored together.
  
  File Loading: 
  The app unzips the files to the 'temp' folder in the sketch folder, patch file, sequence definition file, the sequence data files. The temp folder can not
   
 Future Upgrades: --------------------------------------------------------------------------------------------
 
 - live control transmission of matrix previews
 - live painting, draw and fill using the mouse directly to the LED matrix
 - doesn't save intensity value
 
 Done: --------------------------------------------------------------------------------------------

 
 Notes: --------------------------------------------------------------------------------------------
 
 
 Known Issues: --------------------------------------------------------------------------------------------

    java.lang.IllegalStateException: Buffers have not been created - random, when not focused?
    
    software is locking up communication, think it is awaiting a state that it should timeout or over ride if it doesn't finish
      - happens a lot when devices are being swapped around
      - hasn't happened in a while

 */

// IMPORT Libraries

 









//============ Constants ================

//Configuration Input Module constant strings
final String[] cPixelChipsetStrings = {"DEFAULT", "WS2801", "WS2811", "WS2812", "WS2812B", "WS2813", "APA102", "APA107", "APA104", "APA106", "SK6805", "PD9823", "TM1803", "TM1804", "UCS290", "UCS190"};
final String[] cPixelChipsetColorID = {"unknown", "R>G>B", "G>R>B", "G>R>B", "G>R>B", "G>R>B", "B>G>R", "B>G>R", "R>G>B", "R>G>B", "G>R>B", "R>G>B", "G>R>B", "G>R>B", "G>R>B", "G>R>B"};

final String cGlobalColSwapStrV2[] = {"R>G>B", "B>R>G", "G>B>R", "R>B>G", "B>G>R", "G>R>B"};
final String cGlobalColSwapStr[] = {"R>G>B(>W)", "B>R>G", "G>B>R", "R>B>G", "B>G>R", "G>R>B", "G>R>B>W"};
final String cDDBaudstring[] = {"9600", "19200", "38400", "57600", "115200", "230400", "250000", "460800", "500000", "1Mbit"}; //fix these, there is no 250k without custom FTDI driver, but maybe make one for 500k and 1Mb and 250k
final String cDDLEDMstring[] = {"Both", "Stand-Alone", "RX", "Disabled"};
final String cDDPOVModeString[] = {"Normal Mode", "Mode 1mS", "Mode 2mS", "Mode 3mS", "Mode 4mS", "Mode 5mS", "Mode 6mS", "Mode 8mS", "Mode 16mS"};
final String cDDAutoDetectModesT1[] = {"None/Disabled", "Auto-Detect DMX", "Auto-Detect Serial"};
final String cDDAutoDetectModesT2[] = {"None/Disabled", "Auto-Detect DMX"};
final String cDDAutoDetectModesT3[] = {"None/Disabled", "Auto-Detect"};
final String cDDMasterModes8Bit[] = {"None/Slave", "Master Full Pkts", "Master Partial Pkts"};
final String cDDMasterModesMultiBit[] = {"None/Slave", "Master Full Pkts 8-Bit", "Master Full Pkts 16-Bit", "Master Partial Pkts 8-Bit", "Master Partial Pkts 16-Bit"};
final String cDDDisplayModes[] = {"Always On", "Countdown", "Dim", "Countdown+Dim"};
final String cDDAccMeterShutOff[] = {"Disabled", "5 Sec.", "10 Sec.", "20 Sec.", "30 Sec", "40 Sec.", "60 Sec.", "80 Sec."};
final String cDDAccMeterShutOffMode[] = {"Blank", "Play Slot#1"};
final String cDDPowerTimeOut[] = {"Disabled", "5 Min.", "10 Min.", "15 Min."};
final String cDDAccMeterTapMode[] = {"Disabled", "Sensitive", "Normal", "Strong"};
final String cDDPWMFreqA[] = {"31 KHz", "7.8 KHz", "1.95 KHz", "488 Hz"};
final String cDDPWMProfileA[] = {"16-Bit, 244Hz", "12-Bit, 3.9KHz", "12-Bit, 478Hz", "8-Bit, 62.5Khz", "8-Bit, 976Hz"};
final String cDDIRRemote[] = {"Disabled", "Enabled, 19200 baud", "Enabled 250k baud"};
final String cDDStandAloneA[] = {"Hold", "Blank", "Fade Out", "Stand-Alone"};
final String cDDDMXRelstring[] = {"Disabled", "Timeout & Continue", "Timeout & Play Idle"};
final String cDDExpansionHeaderStrings[] = {"No Usage", "NLED Serial Reception", "Aurora Command Protocol", "Receive DMX", "IR Remote", "Sync Master", "Sync Master+IR Remote", "Sync Slave"};
final String cHardwareColorReOrderStrings[] = {"Disabled(RGB)", "Enable Hardware"};
final String cGlediatorOverRideStrs[] = {"Disabled", "Enable Glediator Protocol"};
final int cAccMeterShutOffVals[] = {0, 16, 32, 64, 96, 128, 192, 250};

//Software String constants
final String cNoneStr[] = {"None"};
final String cRevisionIDstr[] = {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"};
final String cDDPortBaudstring[] = {"USB/None", "9600", "19200", "38400", "57600", "115200", "230400", "250000", "460800", "500000", "1000000"};

final String cDDSeqPlayModes[] = {"Loop", "One-Shot Next", "One-Shot Start", "One-Shot End", "One-Shot Blank", "One-Shot Idle"};
final String cDDSeqTransistions[] = {"Instant", "Fast Fade In", "Slow Fade In"};

//User notification string constants
final String cConfirmBoxText[] = {"null", "Uploading Configurations will overwrite the current settings. Continue?", "Are you sure you want to remove the sequence?", "Are you sure you want to upload the index & color sequences?"};

final String cNotificationText[] = {
  "none", 
  "FilePlay did not load correctly, but may still work", 
  "Connection Success", 
  "Port Failed to Open", 
  "Define Previous Sequence" //4
};    

//============ Software Constants ================

final int cMaxSequences = 64;
final int cSoftwareMaxFrames = 0xFFFF;  
final int cSoftwareMaxChans = 768;  
final int cSoftwareMaxSequences = 256;

final int cHeaderByteLen = 16;

final int cBufferByteSize = 1024; //think it is just the filename

final int cDefaultSeqSpeed = 33; //for 30FPS

//============ GUI Constants ================

final int cInputModuleOffsetX = 100;
final int cInputModuleOffsetY = 180;
final int cInputModuleAreaWidth = 800;
//final int cModuleAreaHeight = 150;
final int cInputModuleHeight = 100;

final int cSeqModuleHeight = 150;
final int cSeqModuleWidth = 244;
final int cSeqModuleYOffset = 240;
final int cSeqModuleXOffset = 35;

final int cPreviewContentWidth = 750-20; //minus 20 is the margin
final int cPreviewContentHeight = 648-20; 

final int cPreviewXPos = 294;  //+20 for margin
final int cPreviewYPos = 120;  //+20 for margin 

final int cMaxIndexViewed = 3; //sequence index

//=============== Aurora Command ID Constants ==================

final int cmdOpen = 4;
final int cmdHWPV = 99;
final int cmdFullSeqUpload = 100;
final int cmdUploadConfigurations = 101;
final int cmdGammaTableUpload = 102;
final int cmdEnterBootloader = 140;
final int cmdRequestConfigurations = 120;

//============ Global GUI Variables ================

int mouseXS, mouseYS; //scaled mouse values for resizing
float SF = 1; //scale factor for GUI resizing

int OverlayMenuID = 0;
int ConfirmBoxIDNum = 0; //0 is no message, greater than 0 is the ID number
int ShowNotification = 0; //0 is off, 1 and up is used for string pointer

boolean GlobalDDOpen = false;
boolean GlobalDragging = false;
boolean TextFieldActive = false;
boolean NumberInputFieldActive = false;
boolean sEnableMouseOver = true;

String GlobalLabelStore = "";  //used for text fields

String DisplayMessageStr = "Starting"; 

String SelectedFilePath = "";
int FileBrowserAction = 0;

float UploadProgressVar = 0;

//============ Global Configuration & Module Variables ==============

String[] DevStrArray = new String[16];
int ConfigFlags = 0;  
int holdHardwareID = 0;

int RecievedDeviceConfigsMSB = 0;
int RecievedDeviceConfigsLSB = 0;

int modulePosX = 90;
int modulePosY = 380;
int moduleHoldX = 0;
int moduleHoldY = 0;
int moduleHoldSz = 0;

int[] PixelChipsetID  = new int[2];

//============ Global Communication Variables ==============

String portName = "None";

int RXByte1 = -1;    // Incoming serial data
int RXByte2 = 0;
int RXByte3 = 0;
int RXByte4 = 0;
int RXByte5 = 0;

int SendCounter = 0;
int SendCounterB = 0;
int USBpacketCount = 0;

byte[] USBPacket = new byte[64];
int PacketPointer = 0;

int ExpectedRecieved = 0;
int ReceiveCounter = 0;
int cmdFlags = 0;
int cmdByte = 4; //open connection default
int cmdData[] = {0, 0, 0, 0};

int CMDTimeOutVal = 3000; //time in miliseconds default

boolean CmdIssued = false;
boolean devConnected = false;
boolean RecievedOpenAck = false;
boolean SentCmdRequest = false;
boolean SentOpenRequest = false;
boolean SentConfigRequest = false;
boolean LiveControlEnabled = false;
boolean UploadInProgress = false;
boolean WaitForAckFlag = false;  
boolean TerminateUpload = false;
boolean ConfigUploadSent = false;

int CommunicationMode = 0; //0 = serial port, 1 = TCP Client
int ProgramBaudRate = 19200; 

int ColorOrderCounter = 0;

//============ Global Application Variables ==============

int displayPixSize = 10; //start at default, recalculated to new size when patch file is loaded
short[] PatchCoordX = new short[1];
short[] PatchCoordY = new short[1];

int SelectedSlot = 0;
int IndexedSequences = 1;
int IndexedMemoryUsage = 0;
String IndexedMemoryUsageStr = "";

int PlayBackFrameNum = 0; //used for matrix preview

int UtilizedChannels = 0; //number of channels to upload, either device.Channels or matrix.patchedChannels whichever is smaller

//====================================== General Objects ==============================================

//Data Objects
GUIObj gui; //gui object, holds colors and such
SoftwareObj software;
DeviceObj device;
MatrixObj matrix;
SequenceObj[] sequence;

zipper FileZipper;

//hardware objects
Serial serialPort;
Client TCPClient; 

//GUI element objects
guiDropDown DropDownPointer;
guiSliderBar SliderPointer;
guiTextField textFieldPtr;
guiNumberInputField numberInputFieldPtr;

guiOverlayMenu[] OverlayMenus;

SpecHWModules[] InputModule;  //Configuration Modules

//============ GUI Objects ================

guiButton menuCloseButton, menuUploadConfigurations, menuRequestConfigurations;
guiButton ConfirmButtonYes, ConfirmButtonNo, UploadProgressCancel;

guiButton comConnectButton, comOpenConfigurationsMenu, comDefaultConfigurations, comUploadSequences, comControlCommands;
guiButton comPlayPauseButton, comLoadPatchFile;

guiDropDown comSerialPortDD, comSerialBaudRateDD, comHardwareColorReOrderDD;
guiButton comSaveSequenceFile, comLoadSequenceFile;

guiNumberInputField comGlobalIntensityField;

guiNumberInputField[] SequenceSpeedField;

guiSliderBar indexScrollBar;

guiButton[] SequenceLoadButtons, SequenceRemoveButtons;
guiDropDown[] SequencePlayModeDD, SequenceTransitionDD;

//============ Module GUI Objects ================

guiDropDown DMXModesDD, PixelChipsetDD, AutoDetectDD, MasterModeDD, HardwareColSwapDD;
guiDropDown BaudRateDD, LEDIndicatorModeADD, LEDIndicatorModeBDD;
guiDropDown StandAloneDD, DisplayModeDD, DisplayModeBDD, ShapeOptionsDD;
guiDropDown AccMeterAutoDetectDD, AccMeterAutoDetectModeDD, AccMeterTapDD, PowerDownTimeOutDD;
guiDropDown PWMProfileADD, IRRemoteModeDD, StandAloneADD, PWMFreqADD, DMXAutoReleaseDD;
guiDropDown HardwareColorReOrderDD, GlediatorOverrideDD;

//added
guiDropDown StandardExpansionHeaderDD, AutoReleaseLegacyDD, ReceptionModeLegacyDD;

guiTextField[] TextFields;

guiButton[] HardwareButtons;
guiCheckBox[] HardwareCheckBox;

//================================== ArrayList for GUI MouseOver Elements ====================================================

ArrayList<guiButton> ButtonList; //makes list to make it easier to mouseover
guiButton PointerButton;

ArrayList<guiTextField> TextFieldList;
guiTextField TextFieldListPointer;

ArrayList<guiCheckBox> CheckBoxList;
guiCheckBox CheckBoxListPointer;

ArrayList<guiDropDown> DropDownList; //makes list to make it easier to mouseover
int DropDownMouseOverID = 0;

ArrayList<guiNumberInputField> NumberInputList; //makes list to make it easier to mouseover

//================================== Graphic Objects ====================================================

PFont font;

PImage imgSliderBarHoriz, imgMixerHandleHoriz, imgMixerHandleVert, imgSliderBarVert, imgColorSelector;

//========================= END OBJECT DECLARTION ==============================

public void setup() 
{
  // Initial window size
   
  surface.setResizable(true);   // Needed for resizing the window to the sender size
  surface.setLocation(100, 100);
  colorMode(RGB);
  frameRate(30);

  font = createFont("Arial", 48);
  textFont(font);

  PImage titlebaricon = loadImage("favicon.gif");
  surface.setIcon(titlebaricon);

  //====================== Init Images ======================

  imgMixerHandleHoriz = loadImage("mixerhandle-horiz.png");  
  imgSliderBarHoriz = loadImage("sliderbg-horiz.png");
  imgSliderBarVert = loadImage("sliderbg-vert.png");
  imgMixerHandleVert = loadImage("mixerhandle-vert.png");  

  //==================== Init ArrayList for objects ======================

  ButtonList = new ArrayList<guiButton>();  
  DropDownList = new ArrayList<guiDropDown>();  
  TextFieldList = new ArrayList<guiTextField>();  
  CheckBoxList = new ArrayList<guiCheckBox>();  
  NumberInputList = new ArrayList<guiNumberInputField>();  

  //====================== Init General Objects ==========================

  gui = new GUIObj(); //init graphic user interface from XML file
  software = new SoftwareObj();
  device = new DeviceObj("None");
  matrix = new MatrixObj();
  sequence = new SequenceObj[64];

  FileZipper = new zipper(); 
  FileZipper.localTempFolder = sketchPath()+File.separator+"temp"+File.separator; //static for now, update from function or is null
  
  BuildCOMDropDown();
  comSerialBaudRateDD = new guiDropDown(cDDPortBaudstring, 0, 10, 45, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");

  comHardwareColorReOrderDD = new guiDropDown(cGlobalColSwapStrV2, 0, 165, 165, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");

  comSaveSequenceFile = new guiButton("Save File", 920, 30, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  comLoadSequenceFile = new guiButton("Load File", 920, 65, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  ConfirmButtonYes = new guiButton("Yes", 435, 410, 60, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  ConfirmButtonNo = new guiButton("No", 525, 410, 60, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  UploadProgressCancel = new guiButton("Cancel", 340, 340, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  menuUploadConfigurations = new guiButton("Upload Configurations", 340, 340, 160, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  menuRequestConfigurations = new guiButton("Request Configurations", 340, 340, 160, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  menuCloseButton = new guiButton("Close", 325, 325, 60, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  comConnectButton = new guiButton("Connect", 140, 10, 90, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  comOpenConfigurationsMenu  = new guiButton("Configurations", 580, 5, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  comUploadSequences = new guiButton("Upload Sequences", 5, 130, 140, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  comControlCommands = new guiButton("Control Commands", 740, 5, 150, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
  comPlayPauseButton = new guiButton("Play", 850, 40, 40, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);
  comLoadPatchFile = new guiButton("Load Patch", 140, 45, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

  indexScrollBar = new guiSliderBar(0, cSeqModuleYOffset, 24, height-cSeqModuleYOffset, 0, 0, 63, color(255), gui.buttonHighlightColor, color(255), color(0), false, true, true, false, "sliderHandlerIndex");

  comGlobalIntensityField = new guiNumberInputField(155, 130 , 25, 50, 1, 100, 2, "genericNumberInputField");
  comGlobalIntensityField.setValue(100);

  sequence = new SequenceObj[cMaxSequences];
  SequenceLoadButtons = new guiButton[5];
  SequenceRemoveButtons = new guiButton[5];
  SequenceSpeedField = new guiNumberInputField[5];
  SequencePlayModeDD = new guiDropDown[5];
  SequenceTransitionDD = new guiDropDown[5];

  //sequence[1] = new SequenceObj(1);
  for (int i = 0; i < cMaxSequences; i++)
  {
    sequence[i] = new SequenceObj(i);
  }

  //use the same buttons regardless of index scroll value
  for (int i = 0; i < cMaxIndexViewed; i++) //5 is how many indexed sequences can be viewed
  {
    SequenceLoadButtons[i] = new guiButton("Load", 90, cSeqModuleYOffset+(i*cSeqModuleHeight)+10, 60, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);
    SequenceRemoveButtons[i] = new guiButton("Remove", 200, cSeqModuleYOffset+(i*cSeqModuleHeight)+10, 60, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, false, true);

    SequenceSpeedField[i] = new guiNumberInputField(130, cSeqModuleYOffset+(i*cSeqModuleHeight)+85, 25, 50, 1, 65535, 1, "SequenceSpeedField");
    SequenceSpeedField[i].setValue(cDefaultSeqSpeed); //set to default FPS in mS

    SequencePlayModeDD[i]  = new guiDropDown(cDDSeqPlayModes, 0, 35, cSeqModuleYOffset+(i*cSeqModuleHeight)+115, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "SequencePlayModeDDFunc");
    SequenceTransitionDD[i]  = new guiDropDown(cDDSeqTransistions, 0, 165, cSeqModuleYOffset+(i*cSeqModuleHeight)+115, 105, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "SequenceTransitionDDFunc");
  }

  //=====================================================================================================================================

  OverlayMenus = new guiOverlayMenu[3];
  OverlayMenus[0] = new guiOverlayMenu(0, 0, 0, 0, 0); //null Menu
  OverlayMenus[1] = new guiOverlayMenu(1, 10, 40, 900, 550); //Configurations Menu
  OverlayMenus[2] = new guiOverlayMenu(1, 10, 40, 780, 550); //Stand-alone commands menu

  //====================== Start GUI objects Definition ==========================

  AccMeterAutoDetectDD = new guiDropDown(cDDAccMeterShutOff, 0, 0, 0, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  AccMeterAutoDetectModeDD = new guiDropDown(cDDAccMeterShutOffMode, 0, 0, 0, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  
  AccMeterTapDD  = new guiDropDown(cDDAccMeterTapMode, 0, 0, 0, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  LEDIndicatorModeADD = new guiDropDown(cDDLEDMstring, 0, 420, 180, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  LEDIndicatorModeBDD = new guiDropDown(cDDLEDMstring, 0, 420, 180, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  DMXModesDD = new guiDropDown(cNoneStr, 0, 0, 0, 180, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  BaudRateDD = new guiDropDown(cDDBaudstring, 0, 0, 0, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  DisplayModeDD = new guiDropDown(cDDDisplayModes, 0, 0, 0, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");    
  DisplayModeBDD = new guiDropDown(cDDDisplayModes, 0, 0, 0, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");    
  MasterModeDD = new guiDropDown(cDDMasterModes8Bit, 0,0, 0, 140, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  AutoDetectDD = new guiDropDown(cDDAutoDetectModesT1, 0, 0, 0, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  HardwareColSwapDD = new guiDropDown(cGlobalColSwapStr, 0, 0, 0, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  PixelChipsetDD = new guiDropDown(cNoneStr, 0, 0, 0, 220, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "ChipsetDDFunc"); 
  PowerDownTimeOutDD = new guiDropDown(cDDPowerTimeOut, 0, 405, 300, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  PWMProfileADD = new guiDropDown(cDDPWMProfileA, 0, 405, 300, 140, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  IRRemoteModeDD = new guiDropDown(cDDIRRemote, 0, 405, 300, 140, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  PWMFreqADD = new guiDropDown(cDDPWMFreqA, 0, 5, 160, 100, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");   
  StandAloneADD = new guiDropDown(cDDStandAloneA, 0, 5, 160, 110, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");  
  //StandAloneDD = new guiDropDown(cColorSwapStr, cColorSwapStr.length, 0, 215, 60, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  DMXAutoReleaseDD = new guiDropDown(cDDDMXRelstring, 0, 0, 0, 150, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");

  //New Aurora Matrix Modules
  StandardExpansionHeaderDD =  new guiDropDown(cDDExpansionHeaderStrings, 0, 0, 0, 190, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  HardwareColorReOrderDD =  new guiDropDown(cHardwareColorReOrderStrings, 1, 0, 0, 150, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  
  GlediatorOverrideDD =  new guiDropDown(cGlediatorOverRideStrs, 0, 0, 0, 180, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");
  
  //=====================================================================================================================================

  TextFields = new guiTextField[15];

  //TextFields[0] = new TextField("Enter#", 35, 510, 50, 25, color(255), color(200, 40, 40), 1, 0, 255, true, false, "txtHandleSlideValue"); //Slide Value
  //don't have to do this TextFields[0].Status = 1; //start greyed out
  TextFields[1] = new guiTextField("Enter#", 225, 470, 50, 20, color(255), color(200, 40, 40), 1, 0, 100, true, false, "txtHandleGenericNumber"); //whats this for now? 
  TextFields[2] = new guiTextField("1", 120, 195, 80, 20, color(255), color(200, 40, 40), 1, 1, 512, true, false, "txtHandleGenericNumber"); //DMX Adr Config
  TextFields[3] = new guiTextField("5", 120, 195, 80, 20, color(255), color(200, 40, 40), 1, 5, 32, true, false, "txtHandleGenericNumber");  // CONVERTED - End-Of-Frame Timer
  TextFields[4] = new guiTextField("170", 120, 195, 80, 20, color(255), color(200, 40, 40), 1, 1, 170, true, false, "txtHandleGenericNumber");  // CONVERTED - Pixel Amount
  TextFields[13] = new guiTextField("1", 20, 265, 25, 25, color(255), color(200, 40, 40), 1, 1, 32, true, false, "txtHandleGenericNumber");  //Stand-Alone Module ID field

  textFieldPtr =  TextFields[1]; //default it so it doesn't NullPointerException

  //=====================================================================================================================================

  HardwareButtons = new guiButton[6];
  HardwareButtons[0] = new guiButton("Enabled", 125, 470, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);
  HardwareButtons[1] = new guiButton("Enabled", 300, 470, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);
  HardwareButtons[2] = new guiButton("Enabled", 300, 470, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);
  HardwareButtons[3] = new guiButton("Enabled", 255, 330, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);
  HardwareButtons[4] = new guiButton("Enabled", 300, 470, 80, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, true, false, true);  

  HardwareCheckBox = new guiCheckBox[4];
  HardwareCheckBox[0] = new guiCheckBox(255, 285, 20, color(255), color(0), color(0), false, "");
  HardwareCheckBox[1] = new guiCheckBox(255, 285, 20, color(255), color(0), color(0), false, "");

  //added
  HardwareCheckBox[2] = new guiCheckBox(255, 285, 20, color(255), color(0), color(0), true, ""); //enable activity LED
  HardwareCheckBox[3] = new guiCheckBox(255, 285, 20, color(255), color(0), color(0), false, ""); //enable serial color swap - pixmicro

  //====================== Start Module Definition ==========================

  InputModule = new SpecHWModules[37];
  InputModule[0] = new SpecHWModules(0, "Serial RS-485 Enable:\n(Disable for TTL Serial)", 0, 0, 140, "fRS485Enable", "BYTE", 0);
  InputModule[1] = new SpecHWModules(1, "Second Activity\nLED Mode:", 0, 0, 130,  "fLEDModeMSBb,fLEDModeLSBb", "BYTE", 0);
  InputModule[2] = new SpecHWModules(2, "Select Correct Color Order For Chipset:", 0, 0, 120,  "CONFIG", "bGlobalColorShift", 240);
  InputModule[3] = new SpecHWModules(3, "Select The Pixel Chipset:\n\n\n\nRecommended Color: varies / R>G>B", 0, 0, 240,  "CONFIG", "bPixelChipset", 250);
  InputModule[4] = new SpecHWModules(4, "Enable Serial Reception Instead of DMX reception. Auto-Detect is disabled during serial reception.", 0, 0, 200,  "CONFIG", "BYTE", 0);
  InputModule[5] = new SpecHWModules(5, "DMX Address:\n(Enter Number)\n\n\n(1-512)", 0, 0, 120,  "CONFIG", "bDMXAdrMSB,bDMXAdrLSB", 0);
  InputModule[6] = new SpecHWModules(6, "DMX Reception\nMode - ID Number: "+DMXModesDD.selStr, 0, 0, 190,  "CONFIG", "bDMXModeID", 190); 
  InputModule[7] = new SpecHWModules(7, "DMX Master Mode:\n(Not all listed options are valid)", 0, 0, 160, "fMasterEnable,fMasterFullPkts,fMaster16Bit", "BYTE", 190); 
  InputModule[8] = new SpecHWModules(8, "Serial Baud Rate:\nID Number: "+BaudRateDD.selStr, 0, 0, 120, "f8or16Bit", "bBaudRateID", 200); 
  InputModule[9] = new SpecHWModules(9, "Activity\nLED Mode:", 0, 0, 120,  "fActivtyLEDMSBa,fActivtyLEDLSBa", "BYTE", 0); 
  InputModule[10] = new SpecHWModules(10, "External\nLED Display Mode:", 0, 0, 140,  "fDisModeMSBa,fDisModeLSBa", "BYTE", 190); 
  InputModule[11] = new SpecHWModules(11, "Auto-Detect Data Control Reception:", 0, 0, 140,  "fAutoDetectID,fAutoDetect", "BYTE", 180); 
  InputModule[12] = new SpecHWModules(12, "Enable Gamma\nCorrection:\n(See Connection tab)", 0, 0, 140,  "fGammaCorrectionEnabled", "CONFIG", 0); 
  InputModule[13] = new SpecHWModules(13, "Accelerometer Movement Detection:", 0, 0, 140,  "fAccMeterAutoShutOff,fAccMeterAutoShutOffMode", "bAccmeterTimeoutVal", 210); 
  InputModule[14] = new SpecHWModules(14, "Enable Accelerometer Speed Adjustment", 0, 0, 140,  "fAccMeterMovement", "BYTE", 0); 
  InputModule[15] = new SpecHWModules(15, "Enable Accelerometer Double Tap Button Press", 0, 0, 140,  "fAccMeterDoubleTap", "bAccmeterDoubleTapVal", 200); 
  InputModule[16] = new SpecHWModules(16, "Power Down\nTime Out", 0, 0, 140,  "fTimeOutEnabledMSB,fTimeOutEnabledLSB", "BYTE", 0); 
  InputModule[17] = new SpecHWModules(17, "Pixel Data Packet Cloning:\n(0 is Disabled)", 0, 0, 120,  "DELETE", "DELETE", 0); 
  InputModule[18] = new SpecHWModules(18, "Sequence Fade Transition:\n(0 is Disabled)", 0, 0, 120,  "DELETE", "DELETE", 0); 
  InputModule[19] = new SpecHWModules(19, "Dual Communication Mode (Serial\n/WiFi/Bluetooth)", 0, 0, 120,  "fDualModeCom", "BYTE", 0); 
  InputModule[20] = new SpecHWModules(20, "PWM Frequency:", 0, 0, 120,  "BIT", "bPWMModeLegacyA", 180); 
  InputModule[21] = new SpecHWModules(21, "Stand-Alone Mode\n& Signal Loss Mode:", 0, 0, 160,  "CONFIG", "bStandaloneLegacyA", 160);
  InputModule[22] = new SpecHWModules(22, "PWM Profile\nResolution & Frequency:", 0, 0, 160,  "CONFIG", "bPWMMode", 160);
  InputModule[23] = new SpecHWModules(23, "Enable I.R. Remote Control:\n(Addon Card)", 0, 0, 160,  "fIRCardEnable,fIRCardEnableSpeed", "BYTE", 160);

  //added for configuration software  
  InputModule[24] = new SpecHWModules(24, "DMX Pixel Amount:\n(Enter Number)\n\n\n(1-170)", 0, 0, 120,  "CONFIG", "bUserPixelAmountMSB,bUserPixelAmountLSB", 0); //textfield
  InputModule[25] = new SpecHWModules(25, "Enable Activity LED:", 0, 0, 120,  "fActivityLEDEnable", "BYTE", 0); //checkbox
  InputModule[26] = new SpecHWModules(26, "DMX Decoder Mode:", 0, 0, 160,  "fDMXDecoderModeFull,fDMXDecoderModeEnable", "BYTE", 0); //Drop down
  InputModule[27] = new SpecHWModules(27, "Enable Serial Color Swap:", 0, 0, 120,  "fEnableSerialColorSwap", "BYTE", 0); //checkbox
  InputModule[28] = new SpecHWModules(28, "Select Color Order For Chipset:", 0, 0, 120,  "fColorOrderGRB,fColorOrderGRBW,fColorOrderBGR,fColorOrderBRG", "BYTE", 210); //drop down
  InputModule[29] = new SpecHWModules(29, "End-Of-Frame Timer:\n(in miliseconds):", 0, 0, 120,  "CONFIG", "bUserTimerValue", 0); //textfield
  InputModule[30] = new SpecHWModules(30, "Serial Baud Rate:\nID Number: "+BaudRateDD.selStr, 0, 0, 120,  "CONFIG", "bBaudRateIDMiniV4", 200); //Drop down

  //out of order vs NLED Aurora
  InputModule[31] = new SpecHWModules(31, "DMX Timeout\n Release:", 0, 0, 170,  "fDMXAutoRelease,fDMXReleaseIdle", "BYTE", 100);

  //new legacy modules
  InputModule[32] = new SpecHWModules(32, "Signal Loss Detection\n(Runs Signal Loss Action):", 0, 0, 140,  "AutoDetectFlag", "BYTE", 100);
  InputModule[33] = new SpecHWModules(33, "Reception Mode:", 0, 0, 120,  "SerialOrDMX", "BYTE", 100);  

  //New Aurora Matrix Modules
  InputModule[34] = new SpecHWModules(34, "Enable Hardware Color Order:\n(Disabled uses RGB)", 0, 0, 170,  "fEnableColorReOrder", "BYTE", 240);  
  InputModule[35] = new SpecHWModules(35, "Expansion Header Mode:\n(May require additional hardware)\n(See Device Datasheet For Details)", 0, 0, 210,  "BIT", "bExpansionHeaderMode", 230);  
  InputModule[36] = new SpecHWModules(36, "Glediator Protocol Override:\n(TTL serial or USB Live Control)\n(Requires power cycle)", 0, 0, 200,  "fGlediatorOverRide", "BYTE", 220);  
  

//------------------------- Final Initializations -------------------------

  SelectedSlot = 0; //reset
  sequence[SelectedSlot].selected = true;

  //Set Default Message
  DisplayMessageStr = "Software Ready"; //rather than call show notificaiton...
  
  device.MaxFramesGlobal = 0xFFFF; //set to max for aurora matrix - that is 36.4 minutes at 30FPS per sequence
  
  //------------------------- Debug setup -------------------------

  //LoadSequencesFile(sketchPath()+File.separator+"mytest.auroramatrix");

} //end setup()

//======================================================================================================

public void BuildCOMDropDown()
{
  //store and reselect value, say COM 6, and COM4 is added teh selStr is wrong, find the proper value again

  if (devConnected == false) ShowNotification(0);

  comSerialPortDD = new guiDropDown(serialPort.list(), 0, 10, 10, 120, 25, gui.buttonColor, gui.buttonHighlightColor, gui.textColor, false, "genericDDCallBack");

  //shorten serial name string if too long
  for (int i=0; i != Serial.list().length; i++)
  {
    if (comSerialPortDD.labels[i].length() > 10) 
    {
      comSerialPortDD.labels[i] = comSerialPortDD.labels[i].substring(comSerialPortDD.labels[i].length()-10, comSerialPortDD.labels[i].length());
    }
  } //end for()
} //end func

//======================================================================================================
//================================= START CONFIGURATION MODULES GUI ELEMENTS=================================================================

public boolean CheckModuleElements()
{
  //--------------------------------------------------------------------------------------------------
  if (InputModule[2].Enabled == true && HardwareColSwapDD.over()) return true; 
  //--------------------------------------------------------------------------------------------------
  if (InputModule[3].Enabled == true && PixelChipsetDD.over())     return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[9].Enabled == true && LEDIndicatorModeADD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[1].Enabled == true && LEDIndicatorModeBDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[10].Enabled == true && DisplayModeDD.over()) return true;  
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[7].Enabled == true && MasterModeDD.over()) return true;  
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[11].Enabled == true && AutoDetectDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[22].Enabled == true && PWMProfileADD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[23].Enabled == true && IRRemoteModeDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[8].Enabled == true && BaudRateDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[6].Enabled == true && DMXModesDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[14].Enabled == true && AccMeterAutoDetectDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[14].Enabled == true && AccMeterAutoDetectModeDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if ( InputModule[15].Enabled == true && AccMeterTapDD.over()) return true;
  //--------------------------------------------------------------------------------------------------  
  if (InputModule[16].Enabled == true && PowerDownTimeOutDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[20].Enabled == true && PWMFreqADD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[21].Enabled == true && StandAloneADD.over()) return true;   
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[31].Enabled == true && DMXAutoReleaseDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[32].Enabled == true && AutoReleaseLegacyDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[33].Enabled == true && ReceptionModeLegacyDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[34].Enabled == true && HardwareColorReOrderDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[35].Enabled == true && StandardExpansionHeaderDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (InputModule[13].Enabled == true && AccMeterAutoDetectDD.over()) return true;
  if (InputModule[13].Enabled == true && AccMeterAutoDetectModeDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    
  if (HardwareCheckBox[0].over() && InputModule[8].Enabled == true) return true;//16-bit serial checkbox, not always enabled via Status
  if (HardwareCheckBox[1].over() && InputModule[19].Enabled == true) return true; //Dual Communication Mode
  //--------------------------------------------------------------------------------------------------
  if (InputModule[36].Enabled == true && GlediatorOverrideDD.over()) return true;
  //--------------------------------------------------------------------------------------------------    

  if (HardwareButtons[0].over() && InputModule[0].Enabled == true) return true;//Enable/Disable RS485

  if (HardwareButtons[1].over() && InputModule[1].Enabled == true) return true;  //16 or 8 bit DMX Transmission

  if (HardwareButtons[2].over() && InputModule[4].Enabled == true) return true;

  if (HardwareButtons[3].over() && InputModule[12].Enabled == true) return true;  //gamma   

  if (HardwareButtons[4].over() && InputModule[14].Enabled == true) return true;    

  //--------------------------------------------------------------------------------------------------  

  if (TextFields[2].over() && InputModule[5].Enabled == true) { 
    TextFields[2].selectField(); 
    return true;
  }//DMX Address
  if (TextFields[13].over() && InputModule[21].Enabled == true) { 
    TextFields[13].selectField(); 
    return true;
  }//StandAlone Field A    

  //--------------------------------------------------------------------------------------------------  

  //added for configuration software  

  if (HardwareCheckBox[2].over() && InputModule[25].Enabled == true) return true;  //enable activity LED

  if (HardwareCheckBox[3].over() && InputModule[27].Enabled == true) return true;  //Enable Serial Color Swap:

  if (TextFields[4].over() && InputModule[24].Enabled == true) { 
    TextFields[4].selectField(); 
    return true;
  }//DMX pixel amount

  if (TextFields[3].over() && InputModule[29].Enabled == true) { 
    TextFields[3].selectField(); 
    return true;
  }//End of Frame Timer

  //--------------------------------------------------------------------------------------------------  
  return false;
} //end func

//===================================================================================================

public void FuncRecommendColorShift()
{  
  println("FuncRecommendColorShift()");
  InputModule[3].Label = "Select The Pixel Chipset:\n\n\n\nHardware Color Order:\n";
  try
  {
    for (int i=0; i <= (PixelChipsetDD.numStrs+1); i++)
    {
      if (PixelChipsetDD.labels[PixelChipsetDD.selStr].indexOf(cPixelChipsetStrings[i]) >= 0)
      {
         InputModule[3].Label += cPixelChipsetColorID[i]; //add to label string
        return; //return if something is found
      }
    } //end for()
  }
  catch(Exception e)
  {
    //nothing
  }
  //otherwise uses default
  InputModule[3].Label += "Not Available, uses R>G>B"; //add to label string
} //end FuncRecommendColorShift

//=================================================================================================================================================  

public boolean IsOnConfigFlags(String passedStr)
{
  for (int i = 0; i != DevStrArray.length; i++)
  {  
    if (DevStrArray[i].equals(passedStr) == true) //if string matches
    {
      println("IsOnConfigFlags() found "+passedStr);
      return true;
    }
  } //end i for()
  return false;
}//end func  

//==================================================================================================================================================

public void bitSet(int pointer)
{
  ConfigFlags |= (1 << pointer); 
  //println("BitSet "+pointer+"    "+ConfigFlags);
} //end func

//==================================================================================================================================================

class SpecHWModules
{
  int IDnumber;
  String Label;
  int xpos;
  int ypos;
  int bWidth;
  int bHeight;

  int Priority;

  String ModuleFlagStr;
  String ModuleByteStr;

  boolean Enabled;

  SpecHWModules(int iIDnumber, String iLabel, int ixpos, int iypos, int ibWidth,  String iModuleFlagStr, String iModuleByteStr, int iPriority)
  {
    IDnumber = iIDnumber;
    Label = iLabel;
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = cInputModuleHeight; //static
    ModuleFlagStr = iModuleFlagStr;
    ModuleByteStr = iModuleByteStr;
    Priority = iPriority;
    Enabled = false;
  }

  public void display()
  {
    if (Enabled == false) return;

    stroke(0);
    strokeWeight(2);
    fill(gui.windowStroke, 200);
    rect(xpos, ypos, bWidth, bHeight);
    noStroke();

    switch(IDnumber)
    {
    case 0:  //device.RS485Capable
      HardwareButtons[0].display();    
      break;    
    case 1:    //LED indicator 2
      LEDIndicatorModeBDD.display();
      //if (LEDIndicatorModeBDD.selected == true)  DropDownPointer = LEDIndicatorModeBDD;    
      break;      
    case 2:  //device.GlobalColorShift    
      HardwareColSwapDD.display();  
      //if (HardwareColSwapDD.selected == true)  DropDownPointer = HardwareColSwapDD;  
      break;      
    case 3:  //Pixel Type
      PixelChipsetDD.display();  
      //if (PixelChipsetDD.selected == true)  DropDownPointer = PixelChipsetDD;
      break;        
    case 4:  //device.HardwareID
      HardwareButtons[2].display();  
      break;      
    case 5: //DMX Adress
      TextFields[2].display(); //DMX Address
      break;      
    case 6:
      Label = "DMX Reception\nMode - ID Number: "+DMXModesDD.selStr;
      DMXModesDD.display(); //has to be last so it overlays everything else
      break;  
    case 7:
      MasterModeDD.display();
      break;
    case 8:
      HardwareCheckBox[0].display();
      fill(0);
      textAlign(LEFT);
      text("16-Bit Serial", HardwareCheckBox[0].xpos+25, HardwareCheckBox[0].ypos+15);    
      Label = "Serial Baud Rate:\nID Number: "+BaudRateDD.selStr;
      BaudRateDD.display();  
      break;
    case 9:
      LEDIndicatorModeADD.display();
      break;  
    case 10:
      DisplayModeDD.display();
      break;  
    case 11:
      AutoDetectDD.display();  
      break;      
    case 12: //fGammaCorrectionEnabled
      HardwareButtons[3].display();
      break;      
    case 13: //Automatic Shut Off/Resume Time Out Value
      AccMeterAutoDetectDD.display();  
      AccMeterAutoDetectModeDD.display();  
      break;        
    case 14: //Enable Accelerometer Speed Adjustment
      HardwareButtons[4].display();
      break;
    case 15: //Enable Double Tap Button Press
      AccMeterTapDD.display();  
      break;
    case 16: //Power Down Time Out
      PowerDownTimeOutDD.display();  
      break;    
    case 17: //Packet Clone

      break;  
    case 18: //Fade Transition

      break;
    case 19: //Dual Communication Mode
      HardwareCheckBox[1].display();  
      fill(0);
      textAlign(LEFT);
      text("Enable", HardwareCheckBox[1].xpos-45, HardwareCheckBox[1].ypos+15); 
      break;  
    case 20: //PWM Freq A
      PWMFreqADD.display();
      break;
    case 21: //Stand-Alone A
      StandAloneADD.display();
      TextFields[13].display();
      fill(0);
      textAlign(LEFT);
      textSize(12);
      text("Enter Sequence ID#:", TextFields[13].xpos-90, TextFields[13].ypos-5);
      break;
    case 22: //PWM Profile
      PWMProfileADD.display();
      break;  
    case 23: //I.R. Remote addon card
      IRRemoteModeDD.display();
      break;      
      //start added modules
    case 24: //DMX Pixel Amount:
      TextFields[4].display();
      break;
    case 25: //"Enable Activity LED:
      HardwareCheckBox[2].display();
      break;
    case 26: //DMX Decoder Mode:

      break;
    case 27: //Enable Serial Color Swap:
      HardwareCheckBox[3].display();
      break;
    case 28: //Select Color Order For Chipset:
    
      break;
    case 29: //End-Of-Frame Timer
      TextFields[3].display();
      break;
    case 30: //mini Serial Baud Rate

      break;
    case 31: //DMX Timeout auto release
      DMXAutoReleaseDD.display();
      break;   
    case 32: //Legancy Signal Loss Auto Detection

      break;   
    case 33: //Legacy Reception mode. DMX or serial

      break;  
    case 34: //Hardware color order
      HardwareColorReOrderDD.display();
      break;   
    case 35: //Expansion Header Mode
      StandardExpansionHeaderDD.display();
      break;
      case 36: //glediator protocol override
      GlediatorOverrideDD.display();
      break;
    }

    textAlign(CENTER);
    textSize(12);  
    textLeading(14);
    fill(0);
    text(Label, xpos+5, ypos+10, bWidth-10, bHeight-10);
  } //end display

  //-------------------------------------------------------------------------------------------------------

  public void InitPositions()
  {

    switch(IDnumber)
    {
    case 0:  //device.RS485Capable
      HardwareButtons[0].xpos = xpos+30;
      HardwareButtons[0].ypos = ypos+55;
      break;    
    case 1:  //second LED indicator
      LEDIndicatorModeBDD.xpos = xpos+10;
      LEDIndicatorModeBDD.ypos = (ypos+(bHeight))-40;
      break;      
    case 2:  //device.GlobalColorShift  
      HardwareColSwapDD.xpos = xpos+10;            
      HardwareColSwapDD.ypos = ypos+60;      
      break;      
    case 3:  //Pixel Chipset
      PixelChipsetDD.xpos = xpos+10;    
      PixelChipsetDD.ypos = ypos+35;    
      break;        
    case 4:  //device.HardwareID
      HardwareButtons[2].xpos = xpos+60;
      HardwareButtons[2].ypos = ypos+55;  
      break;  
    case 5:  
      TextFields[2].xpos = xpos+20;
      TextFields[2].ypos = ypos+45;        
      break;        
    case 6: //DMX Reception Mode  
      DMXModesDD.xpos = xpos+5;
      DMXModesDD.ypos = ypos+45;      
      break;      
    case 7: //DMX Master Mode
      if (device.Bit16Mode == true)
      {
        MasterModeDD.labels = cDDMasterModesMultiBit;
        MasterModeDD.numStrs = cDDMasterModesMultiBit.length;
      } else
      {
        MasterModeDD.labels = cDDMasterModes8Bit;
        MasterModeDD.numStrs = cDDMasterModes8Bit.length;
      }
      MasterModeDD.xpos = xpos+10;
      MasterModeDD.ypos = (ypos+(bHeight))-35;  
      break;      
    case 8:
      BaudRateDD.xpos = xpos+10;
      BaudRateDD.ypos = ypos+45;  

      HardwareCheckBox[0].xpos = xpos+10;
      HardwareCheckBox[0].ypos = (ypos+(bHeight))-25;  

      //set checkbox grey out or not if enabled
      if (IsOnConfigFlags("f8or16Bit") == true) HardwareCheckBox[0].status = 0; //0=show, 1=grey out, 2=hide
      else HardwareCheckBox[0].status = 1; //0=show, 1=grey out, 2=hide    
      break;  
    case 9:
      LEDIndicatorModeADD.xpos = xpos+10;
      LEDIndicatorModeADD.ypos = (ypos+(bHeight))-40;
      break;  
    case 10:
      DisplayModeDD.xpos = xpos+10;
      DisplayModeDD.ypos = (ypos+(bHeight))-40;
      break;  
    case 11:
      AutoDetectDD.xpos = xpos+10;
      AutoDetectDD.ypos = (ypos+(bHeight))-40;      
      break;      
    case 12://fGammaCorrectionEnabled
      HardwareButtons[3].xpos = xpos+30;
      HardwareButtons[3].ypos = ypos+55;  
      break;  
    case 13: //Automatic Shut Off/Resume Time Out Value
      AccMeterAutoDetectModeDD.xpos = xpos+10;    
      AccMeterAutoDetectModeDD.ypos = ypos+40;
      AccMeterAutoDetectDD.xpos = xpos+10;    
      AccMeterAutoDetectDD.ypos = ypos+70;    
      break;        
    case 14: //Enable Accelerometer Speed Adjustment
      HardwareButtons[4].xpos = xpos+30;
      HardwareButtons[4].ypos = ypos+45;  
      break;
    case 15: //Enable Double Tap Button Press
      AccMeterTapDD.xpos = xpos+20;    
      AccMeterTapDD.ypos = ypos+55;  
      break;    
    case 16: //Power Down Time Out  
      PowerDownTimeOutDD.xpos = xpos+10;
      PowerDownTimeOutDD.ypos = (ypos+(bHeight))-40;  
      break;
    case 17: //empty

      break;  
    case 18: //empty

      break;  
    case 19: //Dual Communication Mode
      HardwareCheckBox[1].xpos=xpos+70;  
      HardwareCheckBox[1].ypos=ypos+65;  
      break;  
    case 20: //PWM Freq A
      PWMFreqADD.xpos=xpos+10;
      PWMFreqADD.ypos=(ypos+(bHeight))-40;  
      break;
    case 21: //Stand-Alone A
      StandAloneADD.xpos=xpos+5;  
      StandAloneADD.ypos=(ypos+(bHeight))-35;      
      TextFields[13].xpos=xpos+125;  
      TextFields[13].ypos=(ypos+(bHeight))-35;  
      break;
    case 22: //PWM Profile
      PWMProfileADD.xpos = xpos+10;
      PWMProfileADD.ypos = (ypos+(bHeight))-35;      
      break;      
    case 23: //I.R. Remote Add on
      IRRemoteModeDD.xpos = xpos+10;
      IRRemoteModeDD.ypos = (ypos+(bHeight))-35;      
      break;
    case 24: //DMX Pixel Amount:
      TextFields[4].xpos = xpos+20;
      TextFields[4].ypos = (ypos+(bHeight))-50;   
      break;
    case 25: //"Enable Activity LED:
      HardwareCheckBox[2].xpos=xpos+50;  
      HardwareCheckBox[2].ypos=ypos+40;  
      break;
    case 26: //DMX Decoder Mode:
     
      break;
    case 27: //Enable Serial Color Swap:
      HardwareCheckBox[3].xpos=xpos+50;  
      HardwareCheckBox[3].ypos=ypos+50;  
      break;
    case 28: //Select Color Order For Chipset:
   
      break;
    case 29: //End-Of-Frame Timer
      TextFields[3].xpos = xpos+20;
      TextFields[3].ypos = (ypos+(bHeight))-35;   
      break;
    case 30: //mini Serial Baud Rate
     
      break;
    case 31: //DMX Timeout auto release
      DMXAutoReleaseDD.xpos = xpos+10;
      DMXAutoReleaseDD.ypos = ypos+50;        
      break;     
    case 32: //Legancy Signal Loss Auto Detection
      AutoReleaseLegacyDD.xpos = xpos+10;
      AutoReleaseLegacyDD.ypos = (ypos+(bHeight))-35;        
      break;   
    case 33: //Legacy Reception mode. DMX or serial
      ReceptionModeLegacyDD.xpos = xpos+10;
      ReceptionModeLegacyDD.ypos = (ypos+(bHeight))-35;        
      break;
    case 34: //Native color order
      HardwareColorReOrderDD.xpos = xpos+10;
      HardwareColorReOrderDD.ypos = (ypos+(bHeight))-35;   
      break;   
    case 35: //Expansion Header Mode
      StandardExpansionHeaderDD.xpos = xpos+10;
      StandardExpansionHeaderDD.ypos = (ypos+(bHeight))-35;   
      break;
    case 36: //glediator protocol override
      GlediatorOverrideDD.xpos = xpos+10;
      GlediatorOverrideDD.ypos= (ypos+60);   
      break;     
    }
  }//end InitPositions

  //-------------------------------------------------------------------------------------------------------

  public int GetBitValues(int passedID)
  {

    switch(IDnumber)
    {
    case 0:  //device.RS485Capable
      //"fRS485Enable,f8or16Bit"
      if (HardwareButtons[0].selected && passedID == 0)  return 1;//fRS485Enable
      if (HardwareCheckBox[0].selected && passedID == 1) return 1;//f8or16FBit
      break;    
    case 1: //LED Indicator Mode 2

      break;      
    case 2:  //bGlobalColorShift

      break;      
    case 3:  //Pixel Chipset

      break;        
    case 4: //Enable Serial Reception Instead of DMX

      break;  
    case 5:  //DMX Adress

      break;        
    case 6: //DMX Reception

      break;      
    case 7: //DMX Master Mode

      if (IsOnConfigFlags("fMaster16Bit") == true) 
      {
        //"fMasterEnable,fMasterFullPkts,fMaster16Bit"
        switch(MasterModeDD.selStr)
        {
        case 0: //None
          //both ID flag and Autodetect stay 0

          return 0;
        case 1: //Master Full Pkts 8-Bit
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable
          //temp2Flags = temp2Flags | 0x02; //fMasterFullPkts(1 is Full, 0 is partial)
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 1;//fMasterFullPkts(1 is Full, 0 is partial)
          if (passedID == 2)       return 0;//fMaster16Bit  stays 0            
          //fMaster16Bit stays 0  
          break;
        case 2: //Master Full Pkts 16-Bit
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable
          //temp2Flags = temp2Flags | 0x02; //fMasterFullPkts(1 is Full, 0 is partial)  
          //temp2Flags = temp2Flags | 0x04; //fMaster16Bit
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 1;//fMasterFullPkts(1 is Full, 0 is partial)
          if (passedID == 2)       return 1;//fMaster16Bit  stays 0            

          break;
        case 3:  //Master Partial Pkts 8-Bit
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable  
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 0;//fMasterFullPkts(1 is Full, 0 is partial) 
          if (passedID == 2)       return 0;//fMaster16Bit  stays 0      
          break;  
        case 4:  //Master Partial Pkts 16-Bit
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable  
          //temp2Flags = temp2Flags | 0x04; //    
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 0;//fMasterFullPkts(1 is Full, 0 is partial)
          if (passedID == 2)       return 1;//fMaster16Bit  
          break;
        }//end switch
      } else
      {
        //"fMasterEnable,fMasterFullPkts"
        switch(MasterModeDD.selStr)
        {
        case 0: //None
          //both ID flags  
          return 0;
        case 1: //Master Full Packets
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable
          //temp2Flags = temp2Flags | 0x02; //fMasterFullPkts(1 is Full, 0 is partial)
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 1; //fMasterFullPkts

          break;
        case 2: //Master Partial Packets
          //temp2Flags = temp2Flags | 0x01; //fMasterEnable        
          if (passedID == 0)       return 1;//fMasterEnable
          if (passedID == 1)       return 0;//fMasterFullPkts(1 is Full, 0 is partial) - leave it 0
          break;
        }//end switch
      }

      break;      
    case 8: //Serial Baud Rate
      if (HardwareCheckBox[0].selected == true) return 1;  
      return 0;
      //break;  
    case 9: //LED Inidicator A
      //11 "Both", 01"Stand-Alone", 10"RX", 00"Disabled"
      //"fLEDModeMSBa,fLEDModeLSBa"
      switch(LEDIndicatorModeADD.selStr) //LEDMode
      {
      case 0: //Both
        //tempFlags = tempFlags | 0x0C;
        if (passedID == 0)       return 1;
        if (passedID == 1)       return 1;
        break;
      case 1: //Stand-Alone
        //tempFlags = tempFlags | 0x04;
        if (passedID == 0)       return 0;
        if (passedID == 1)       return 1;      
        break;
      case 2: //RX only
        //tempFlags = tempFlags | 0x08;
        if (passedID == 0)       return 1;
        if (passedID == 1)       return 0;        
        break;
      case 3: //disabled
        //no change both 00
        return 0;
      }
      break;  
    case 10: //External LED Display Mode
      //"fDisModeMSBa,fDisModeLSBa"
      switch(DisplayModeDD.selStr)
      {
      case 0: //Always On
        //00 is always on
        return 0;
      case 1: //Countdown
        if (passedID == 0)       return 1;
        if (passedID == 1)       return 0;
        break;
      case 2: //Dim
        if (passedID == 0)       return 0;
        if (passedID == 1)       return 1;
        break;
      case 3: //both
        if (passedID == 0)       return 1;
        if (passedID == 1)       return 1;
        break;
      }
      break;  
    case 11: //Standard Auto-Detect
      switch(AutoDetectDD.selStr)
      {
      case 0: //None
        //both ID flag and Autodetect stay 0
        return 0;
      case 1: //Autodetect DMX
        //tempFlags = tempFlags | 0x10; //Set AutoDetectID Flag to 1
        //tempFlags = tempFlags | 0x20; //Set AutoDetectEnable Flag to 1
        if (passedID == 0)       return 1;
        if (passedID == 1)       return 1;
        break;
      case 2: //Autodetect Serial
        //AutoDetectID Flag stays 0 for serial
        //tempFlags = tempFlags | 0x20; //Set AutoDetectEnable Flag to 1
        if (passedID == 0)       return 0;
        if (passedID == 1)       return 1;
        break;
      }//end switch
      break;      
    case 12: //Gamma Correction Enabled
      if (HardwareButtons[3].selected == true) return 1;
      return 0;
    case 13: //Automatic Shut Off/Resume Time Out Value
      if (passedID == 0) if (AccMeterAutoDetectDD.selStr > 0) return 1;
      if (passedID == 1) if (AccMeterAutoDetectModeDD.selStr == 1) return 1;
      //if(AccMeterAutoDetectDD.selStr > 0)  return 1; //fAccMeterAutoShutOff
      //if(AccMeterAutoDetectModeDD.selStr > 0)  return 1; //fAccMeterAutoShutOffMode
      return 0; //break;        
    case 14: //Enable Accelerometer Speed Adjustment
      if (HardwareButtons[4].selected)  return 1;//fAccMeterMovement
      return 0; //break;
    case 15: //Enable Double Tap Button Press
      if (AccMeterTapDD.selStr > 0) return 1;//fAccMeterDoubleTap
      return 0; //break;    
    case 16: //Power Down Time Out  

      break;
    case 17:

      break;  
    case 18:

      break;  
    case 19: //Dual Communication Mode
      if (HardwareCheckBox[1].selected == true) return 1; //fDualModeCom
      return 0; //break;  
    case 20: //PWM Freq A

      break;
    case 21: //Stand-Alone A

      break;
    case 22: //PWM Profile

      break;    
    case 23: //IR Remote Addon
      switch(IRRemoteModeDD.selStr)
      {
      case 0: //None
        //both ID flags  0
        return 0;
      case 1: //Enabled, 19200, 1-0
        if (passedID == 0)   return 1;
        if (passedID == 1)   return 0;  
        break;
      case 2: //Enabled, 250k, 1-1  
        if (passedID == 0)   return 1;
        if (passedID == 1)   return 1;
        break;
      }//end switch
      break;

      //start added modules
    case 24: //DMX Pixel Amount:

      break;
    case 25: //"Enable Activity LED:
      if (HardwareCheckBox[2].selected == true) return 1; //fActivityLEDEnable
      else return 0;
      //break;
    case 26: //DMX Decoder Mode:

      break;
    case 27: //Enable Serial Color Swap:
      if (HardwareCheckBox[3].selected == true) return 1; //fEnableSerialColorSwap
      break;
    case 28: //Select Color Order For Chipset:
    
      break;
    case 29: //End-Of-Frame Timer

      break;
    case 30: //mini Serial Baud Rate

      break;
    case 31: //DMX Timeout auto release
      switch(DMXAutoReleaseDD.selStr)
      {
      case 0: //None
        //both ID flags  0
        return 0;
      case 1: //Enabled, Restore Sequence
        if (passedID == 0)   return 1;
        if (passedID == 1)   return 0;  
        break;
      case 2: //Enabled, Play Idle  
        if (passedID == 0)   return 1;
        if (passedID == 1)   return 1;
        break;
      }//end switch
      break;
    case 32: //Legancy Signal Loss Auto Detection
      if (AutoReleaseLegacyDD.selStr == 0) return 0;
      else return 1;
      //break;   
    case 33: //Legacy Reception mode. DMX or serial
      if (ReceptionModeLegacyDD.selStr == 0) return 0;
      else return 1;    
      //break;
    case 34: //Hardware color order
      if(HardwareColorReOrderDD.selStr == 0) return 0;
      else return 1;
      //break;   
    case 35: //Expansion Header Mode
      
     break;
      case 36: //glediator protocol override
      if(GlediatorOverrideDD.selStr == 0) return 0;
      else return 1;
     //break;           
    }  //end switch 

    println("Did not find InputModule[] to GetBitValues() from with ID "+IDnumber);
    return 0;
  } //end GetBitValues()

  //-------------------------------------------------------------------------------------------------------

  public int GetByteValues(int passedID)
  {
    switch(IDnumber)
    {
    case 0:  //device.RS485Capable

      break;    
    case 1:

      break;      
    case 2:  //bGlobalColorShift
      return (HardwareColSwapDD.selStr); //Global Color Swap    
    case 3:  //Pixel Chipset
      return (PixelChipsetID[PixelChipsetDD.selStr]); //Pixel Type    
    case 4: //Enable Serial Reception Instead of DMX

      break;  
    case 5:  //DMX Adress
      if (passedID == 0)   return ((PApplet.parseInt(TextFields[2].label) >> 8) & 0xFF); //DMXAdr MSB
      if (passedID == 1)   return (PApplet.parseInt(TextFields[2].label) & 0xFF); //DMXAdr LSB
      break;        
    case 6: //DMX Reception Mode
      return DMXModesDD.selStr;  //DMX Mode ID    
    case 7: //DMX Master Mode

      break;      
    case 8: //Serial Baud Rate
      return BaudRateDD.selStr; //BaudRateID
    case 9: //LED Inidicator A

      break;  
    case 10: //External LED Display Mode

      break;  
    case 11: //Standard Auto-Detect

      break;      
    case 12: //Gamma Correction Enabled

      break;  
    case 13: //Automatic Shut Off/Resume Time Out Value
      return (cAccMeterShutOffVals[AccMeterAutoDetectDD.selStr]);   //Timeoutval      
    case 14: //Enable Accelerometer Speed Adjustment

      break;
    case 15: //Enable Double Tap Button Press

      return (AccMeterTapDD.selStr);  //Double Tap Mode  
    case 16: //Power Down Time Out  

      break;
    case 17:

      break;  
    case 18:

      break;  
    case 19: //Dual Communication Mode

      break; 
    case 20: //PWM Freq LegacyA
      return (PWMFreqADD.selStr+4); //4,5,6,7 are loaded into T6CON
    case 21: //Stand-Alone Legacy A
      switch(StandAloneADD.selStr) //either 10-42 for sequences, 1 for hold, 2 for blank, 3 blank with fade out
      {
      case 0: //hold
      case 1: //blank
      case 2: //fade out
        return (StandAloneADD.selStr+1); //0 is null so add 1
      case 3: //stand alone
        return ((PApplet.parseInt(TextFields[13].label)+9)); //10+ is sequences, base 0, textfield should be at least 1
      }  //end switch
      break;
    case 22: //PWM Profile
      return (PWMProfileADD.selStr);  //16-bit lamp PWM Profile  
    case 23: //IR Remote Add On

      break;
      //start added modules
    case 24: //DMX Pixel Amount:
      if (passedID == 0)   return ((PApplet.parseInt(TextFields[4].label) >> 8) & 0xFF); //DMXAdr MSB
      if (passedID == 1)   return (PApplet.parseInt(TextFields[4].label) & 0xFF); //DMXAdr LSB
      break;
    case 25: //Enable Activity LED:

      break;
    case 26: //DMX Decoder Mode:

      break;
    case 27: //Enable Serial Color Swap:

      break;
    case 28: //Select Color Order For Chipset:

      break;
    case 29: //End-Of-Frame Timer
      return PApplet.parseInt((PApplet.parseInt(TextFields[3].label) * 1000)  / 128);
      //break;
    case 30: //mini Serial Baud Rate
      //return BaudRateMiniV4DD.selStr; //BaudRateID
    case 31: //DMX Timeout auto release

      break;
    case 32: //Legancy Signal Loss Auto Detection

      break;   
    case 33: //Legacy Reception mode. DMX or serial

      break;
    case 34: //Hardware color order

        break;   
    case 35: //Expansion Header Mode
      return StandardExpansionHeaderDD.selStr;
      //break;
    case 36: //glediator protocol override

     break;         
    }  //end switch  

    println("Did not find InputModule[] to GetByteValues() from with ID "+IDnumber);
    return 0;
  } //end GetByteValues()

  //-------------------------------------------------------------------------------------------------------

  public boolean over() 
  {
    if (Enabled == false) return false;  

    if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight) 
    {      
      return true;
    } 
    else {
      return false;
    }
  }

  //-------------------------------------------------------------------------------------------------------

  public void PlaceTile()
  {
    if (InputModule[IDnumber].bWidth < moduleHoldSz)
    {
      //println("Module "+IDnumber+" with width of: "+InputModule[IDnumber].bWidth+" can fit at Y: "+moduleHoldY+", X: "+moduleHoldX+" with available: "+moduleHoldSz);  

      InputModule[IDnumber].xpos = moduleHoldX;    
      InputModule[IDnumber].ypos = moduleHoldY;

      moduleHoldX = 0;
      moduleHoldY = 0;
      moduleHoldSz = 0;
    } 
    else if ((modulePosX + InputModule[IDnumber].bWidth) < (cInputModuleOffsetX+cInputModuleAreaWidth))  
    {
      InputModule[IDnumber].xpos = modulePosX;    
      modulePosX += (InputModule[IDnumber].bWidth+10);
      InputModule[IDnumber].ypos = modulePosY;
    } 
    else 
    {
      //saves previous row extra space and will try to fit something in
      if (((cInputModuleOffsetX+cInputModuleAreaWidth) - modulePosX) > moduleHoldSz)
      {
        //if new row size is larger than last, use it, otherwise utilize largest space  
        moduleHoldX = modulePosX;
        moduleHoldY = modulePosY;
        moduleHoldSz = (cInputModuleOffsetX+cInputModuleAreaWidth) - modulePosX;    
        //println("remaining Module Size: "+moduleHoldSz);
      }

      modulePosY += cInputModuleHeight+10;
      InputModule[IDnumber].xpos = cInputModuleOffsetX;
      modulePosX = cInputModuleOffsetX + (InputModule[IDnumber].bWidth+10);
      InputModule[IDnumber].ypos = modulePosY;
    }

    InputModule[IDnumber].Enabled = true;
    InputModule[IDnumber].InitPositions();
  } //end func
}  //end class

//==================================================================================================================================================

//===================================================================================================

public void OpenCOMPort(String NameID)
{
  println("OpenCOMPort() with "+NameID+" at baud: "+ProgramBaudRate);

  if (NameID.equals("TCP/IP"))
  {
    thread("OpenTCP");
    return;
  }

  try {
    serialPort.stop();
    serialPort.dispose();
  }
  catch(Exception e) {
  }

  int x = 0; //so can be used in catch

  try 
  {
    
    
    for (x =0; x != Serial.list().length; x++) 
    {
      if (NameID.equals(Serial.list()[x]))
      {  
        portName = Serial.list()[x];      
        serialPort = new Serial(this, portName, ProgramBaudRate);

        ShowNotification(2);
        println("Connected to "+portName);

        OpenConnection(); //Aurora device selected, attempt connection command
        break;
      } else
      {
        //println(Serial.list()[x]+" is not a Match for COM port in software.ini");
      }
    }
  }
  catch(Exception e)
  {
    println("Could Not Open COM Port "+Serial.list()[x]);
    devConnected = false;    
    portName = "None";
    ShowNotification(3);
  }
} //end func()

//===================================================================================================

public void CloseCOMPort()
{
  println("CloseCOMPort() - Closing Port");    
  devConnected = false;    
  UploadInProgress = false; //incase it was uploading and connection was lost

  try   
  {  
    //close COM port
    serialPort.stop();
    serialPort.dispose();
    devConnected = false;
  }
  catch(Exception e)
  {
    println("No Port To Close");
  }  

  portName = "None";
  println("COM Port Closed");
}

//===================================================================================================

public void CloseCommunicationPort()
{
  if (CommunicationMode == 0) CloseCOMPort();  
  //  else if(CommunicationMode == 1) CloseTCP();
} //end CloseCommunicationPort()


//===================================================================================================

//separate so one can pass characters, for exchange, other for data/numbers
public void SendSingleDataChar(char passedValue)
{
  try {
    if (CommunicationMode == 0) serialPort.write(passedValue);
    //  else if(CommunicationMode == 1) TCPClient.write(passedValue);
  }
  catch(Exception e) {
  }
}

public void SendSingleDataByte(int passedValue)
{
  println("Sent 1 byte: "+passedValue);

  try {
    if (CommunicationMode == 0) serialPort.write(passedValue);
    // else if(CommunicationMode == 1) TCPClient.write(passedValue);
  }
  catch(Exception e) {
  }
}

public void SendAckedCommand() //threaded
{
  println("Cmd Request Acknowledged, Sending - CmdByte: "+cmdByte+" : "+cmdData[0]+" : "+cmdData[1]+" : "+cmdData[2]+" : "+cmdData[3]);
  RecievedOpenAck = false;
  SendSingleDataByte(cmdByte);
  SendSingleDataByte(cmdData[0]);  
  SendSingleDataByte(cmdData[1]);   
  SendSingleDataByte(cmdData[2]);  
  SendSingleDataByte(cmdData[3]); 

  CmdIssued = true; //set flag to wait for authentication
}

//===================================================================================================  

public void serialEvent(Serial serialPort) 
{
  RXByte5 = RXByte4;
  RXByte4 = RXByte3;
  RXByte3 = RXByte2;
  RXByte2 = RXByte1;
  RXByte1 = serialPort.read();
  DataRecieved(); //run common function
}  //end serialEvent()

//===================================================================================================  

public void DataRecieved()  
{
 // println("Received: "+RXByte1+"  Char: "+char(RXByte1)); //DISABLED TO PREVENT PRINTLN FROM RUNNING OUT OF SPACE
  
  //START TIMER METHOD
  /*
   int tempTime = (millis() - HoldMiliSec);
   println("Received: "+RXByte1+"/"+char(RXByte1)+" Time: "+tempTime);
   HoldMiliSec=millis();  
   */
  //END TIMER METHOD

  if (SentCmdRequest == true && RXByte2 == 'a' && RXByte1 == '9')
  {
    RecievedOpenAck = true;
    SentCmdRequest = false;
    EnableTransmissionTimeOut();
    //myPort.write("nled99"); 
    SendSingleDataChar('n'); 
    SendSingleDataChar('l');  
    SendSingleDataChar('e');
    SendSingleDataChar('d');  
    SendSingleDataChar('9');   
    SendSingleDataChar('9');

    println("Sent Acknowledge");
  } 
  else if (RecievedOpenAck == true  && RXByte2 == 'f' && RXByte1 == '0')
  {
    thread("SendAckedCommand");
  } 
  else if (SentConfigRequest == true)
  {
    switch(ReceiveCounter)
    {
    case 0: //MSB
      RecievedDeviceConfigsMSB = RXByte1;
      break;      
    case 1: //LSB
      RecievedDeviceConfigsLSB = RXByte1;    
      SentConfigRequest = false;
      println("Requested and Recieved Configurations "+binary(RecievedDeviceConfigsMSB, 8)+" "+binary(RecievedDeviceConfigsLSB, 8));
      break;
    } //end switch
    ReceiveCounter++;
  } 
  else if (SentOpenRequest == true)
  {
    println("Counter: "+ReceiveCounter);
    switch(ReceiveCounter)
    {
    case 0: //hardware ID
      holdHardwareID = device.HardwareID; //holds current or old HardwareID
      device.HardwareID = RXByte1;
      break;      
    case 1: //hardware version ID, not really used
      device.HardwareVersion = RXByte1;
      break;
    case 2: //Firmware Version
      device.FirmwareVersion = RXByte1;
      break;
    case 3: //Firmware Revision
      device.FirmwareRevision = RXByte1;
      break;
    case 4:  //Bootloader Target HardwareID

      break;      
    case 5:  //Bootloader Version Number
      device.BootloaderVersion = RXByte1;
      break;      
    case 6:  //UserID Number Configuration
      device.UserConfiguredIDNum = RXByte1;

      //All Device Connection bytes received, now load device file and continue
      SentOpenRequest = false;
      devConnected = true;

      try {
        LoadDeviceFile();
        //String temp = cRevisionIDstr[device.FirmwareRevision]; //to test and catch if valid firmware revision number
      }
      catch(Exception e) { 
        ShowNotification(0);
      }
      break;
    }  
    ReceiveCounter++;
  } 
  else if (UploadInProgress == true || ConfigUploadSent == true) //added the cmdUploadConfigurations check so it would ack config upload
  {
    if (RXByte4 == 'z' && RXByte3 == 'A' && RXByte2 == 'c' && RXByte1 == 'k')  
    {
      WaitForAckFlag = true;
      println("ACK RECEIVED FOR PACKET# "+USBpacketCount);
      //also send CRC byte and packet# to maintain continuity
      if (ConfigUploadSent == true) ShowNotification(12);
      ConfigUploadSent = false;
    } 
  } 
  else if (CmdIssued == true)
  {
    if (RXByte5 == 'c' && RXByte4 == 'm' && RXByte3 == 'd' && RXByte2 == 0)  
    {
      CmdIssued = false;  
      println("COMMAND #"+RXByte1+" AUTHENTICATED");
      //run common init
      SendCounter = 0;
      SendCounterB = 0;
      USBpacketCount = 0; //reset counter  
      PacketPointer = 0; //would also get set to 0, since it always sends 64 bytes then clears it.

      //this executes the actual command software side once ACKed
      switch(cmdByte)
      {     
      case 4: //Request Connection
        SentOpenRequest = true;
        ReceiveCounter = 0;
        break;
       case 100: //Full Upload
        UploadInProgress = true;        
        thread("FullIndexUpload");
        break;       
      case 101:  //Upload Configurations
        UploadInProgress = true;  
        ConfigUploadSent = true;
        SendConfigurations();  
        UploadInProgress = false; 
        break;
      case 120: //Request Configurations
        SentConfigRequest = true;
        ReceiveCounter = 0;
        break;
      }  //end switch
    } //end byte check
    //else if(devConnected == false && cmdByte == 4)
    else if (RXByte4 > 0 && RXByte3 == 1 && RXByte2 == 1 && RXByte1 < 26)  
    {
      //Detects if version 1, rather than acking the cmd
      //hardwareID,hardwarev, firmwareV, firmwareRev
      println(RXByte1+"   "+RXByte2+"   "+RXByte3+"   "+RXByte4+"   "+RXByte5);

      //RXByte4 = ID numnber
      //RXByte3 = hardware v
      //RXByte2 = firmware v
      //RXByte1 = firmware rev

      device.HardwareID = RXByte4;
      device.HardwareVersion = RXByte3;
      device.FirmwareVersion = RXByte2;
      device.FirmwareRevision = RXByte1;    

      SentOpenRequest = false;
      devConnected = true;

      try {
        LoadDeviceFile();
        //String temp = cRevisionIDstr[device.FirmwareRevision]; //to test and catch if valid firmware revision number
      }
      catch(Exception e) { 
        ShowNotification(0);
      }
    }
  } //end CmdIssued if()
} //end func

//=====================================================================================================

public void RequestCommand()
{
  //reset all command authentication flags
  TerminateUpload = false; //reset flag
  WaitForAckFlag = false; 
  UploadInProgress = false;
  RecievedOpenAck = false;  
  SentCmdRequest = false;    

  try {
    // SendSingleDataByte("NLED11"); 
    SendSingleDataChar('N'); 
    SendSingleDataChar('L');  
    SendSingleDataChar('E');   
    SendSingleDataChar('D');  
    SendSingleDataChar('1');   
    SendSingleDataChar('1');
    SentCmdRequest = true; 
    EnableTransmissionTimeOut();
  }
  catch(Exception e)
  {
    println("Port Not Open, Or Unable to Send"); 
    SentCmdRequest = false;

    CloseCommunicationPort();
    ShowNotification(0);
  }
} //end func()

//===================================================================================================  

public void OpenConnection()
{
  println("Attempting Connection");

  cmdByte = cmdOpen; //connect to device
  cmdData[0] = 0;
  cmdData[1] = 0; 
  cmdData[2] = 0;  
  cmdData[3] = 0;  
  devConnected = false;

  RequestCommand();
}//end func()

//===================================================================================================  

public void RequestDeviceConfigurations()
{
  println("RequestDeviceConfigurations()");

  cmdByte = cmdRequestConfigurations; 
  cmdData[0] = 0;
  cmdData[1] = 0; 
  cmdData[2] = 0;  
  cmdData[3] = 0;  

  RequestCommand();
}//end func()

//===================================================================================================  

public void RequestConfigurationUpload()
{
  println("RequestConfigurationUpload()");

  cmdByte = cmdUploadConfigurations; //Upload Configurations
  cmdData[0] = 0; //MSB
  cmdData[1] = 0; //LSB
  cmdData[2] = 0;  
  cmdData[3] = 0;  
  RequestCommand();
} //end RequestConfigurationUpload

//===================================================================================================  

public void RequestNextSequenceCommand()
{
    try {
    serialPort.write("NLED11");
    delay(20);
    serialPort.write("nled99");
    delay(20);
    serialPort.write(92);
    serialPort.write(0);
    serialPort.write(0);
    serialPort.write(0);
    serialPort.write(0);
    }
    catch(Exception e)
    {
      println("Can not send command");
    }
}
  
 //=================================================================================================== 
 
public void TransmissionTimerThread()
{
  int myStart = millis();

  while ((millis() - myStart) < CMDTimeOutVal)
  {
    delay(1);  //Required or would only work a few times, then stops working
    if (SentCmdRequest == false && RecievedOpenAck == false) { 
      return;
    } //force out by mouse click after min time
  }

  CloseCommunicationPort();

  ShowNotification(6); //display notification

  WaitForAckFlag = false;
  UploadInProgress = false;
  RecievedOpenAck = false;  
  SentCmdRequest = false;    

  println("Command Timed Out "+(millis() - myStart));
  redraw();
} //end NotificationTimerThread()

public void EnableTransmissionTimeOut()
{
  println("EnableTransmissionTimeOut()");
  thread("TransmissionTimerThread");
} //end func

//============================================================================================

//===================================================================================================  

public void draw() {
  //update in case of slider or other gui features
  mouseXS = round((float)mouseX / SF);
  mouseYS = round((float)mouseY / SF);

  background(gui.windowBackground);

  fill(100);
  noStroke();
  rect(274, 100, 750, 680);

  strokeWeight(2);
  stroke(gui.windowStroke);
  line(0, 100, width, 100);
  line(274, 100, 274, height);
  line(0, 210, 274, 210);
  line(0, cSeqModuleYOffset-5, 274, cSeqModuleYOffset-5);
  line(570, 0, 570, 100);
  line(905, 0, 905, 100);
  line(725, 0, 725, 100);

  fill(gui.textColor);
  textSize(14);
  textAlign(LEFT);
  //Device Status: Not Connected\n
  text("Device: "+device.Name+"\nVersion: "+(device.FirmwareVersion)+(PApplet.parseChar(device.FirmwareRevision+97))+"\nPatch: "+matrix.patchFileShort, 255, 20);
  text("Slot#: "+(SelectedSlot+1)+"\nFrame: "+(PlayBackFrameNum+1)+"/"+sequence[SelectedSlot].totalFrames, 735, 60);
  
  text("Patch Info:\nW: "+matrix.width+"   H: "+matrix.height+"\nPixels: "+matrix.patchedPixels, 580, 50);
  
  text("Matrix Save Files:", 910, 15);
  
  text("Software Color Order:\n(Reordering applied by software at upload)", 5, 185);
  text("Selected Slot#: "+(SelectedSlot+1), 10, 230);
  text("Status: "+DisplayMessageStr, 10, 90);

  if (IndexedMemoryUsage > device.DataSpace) fill(255, 0, 0); //show red text if indexed sequences are too large
  if (devConnected == true)  text("Memory Usage: "+IndexedMemoryUsageStr+"% of "+device.MemorySpaceString, 10, 120);
  else text("Memory Usage: Connect Device", 10, 120);
  //text("Memory Usage: "+IndexedMemoryUsage+" of "+device.MemorySpaceString, 10, 120); //byte size text, for debugging
  fill(gui.textColor); // go back to white text

  //---------------------------------------------------------------------------------------------------------------------

  //could be done in a better way but this is quicker and less debugging
  if (devConnected == false) 
  {
    comOpenConfigurationsMenu.status = 1;
    comUploadSequences.status = 1;
    comControlCommands.status = 1;
    comSerialBaudRateDD.status = 0;
    comConnectButton.Label = "Connect";
  } else
  {
    comOpenConfigurationsMenu.status = 0;
    if (IndexedMemoryUsage > device.DataSpace)  comUploadSequences.status = 1;
    else comUploadSequences.status = 0;

    comControlCommands.status = 0;
    comSerialBaudRateDD.status = 1;
    comConnectButton.Label = "Disconnect";
  }
  
  //NOT YET ENABLED, maybe next revision
  comControlCommands.status = 1;
  
  //---------------------------------------------------------------------------------------------------------------------

  indexScrollBar.display();
  comSaveSequenceFile.display();
  comLoadSequenceFile.display();
  comGlobalIntensityField.display();
  comConnectButton.display();   
  comPlayPauseButton.display();  
  comLoadPatchFile.display(); 
  comControlCommands.display();
  comUploadSequences.display();
  comOpenConfigurationsMenu.display();
  comSerialBaudRateDD.display();
  comSerialPortDD.display();
  comHardwareColorReOrderDD.display();

  //---------------------------------------------------------------------------------------------------------------------

  for (int i = indexScrollBar.getValue(); i < IndexedSequences+1; i++)
  {
    sequence[i].display(i-indexScrollBar.getValue());
  }

  for (int i = 0; i < cMaxIndexViewed; i++)
  {
    if (i >= (IndexedSequences-indexScrollBar.getValue())) break;
    SequenceRemoveButtons[i].display();
    SequenceLoadButtons[i].display();
    SequenceSpeedField[i].display();
    SequencePlayModeDD[i].display();
    SequenceTransitionDD[i].display();
  }

  //-------------------------------------------- Matrix Preview -------------------------------------------------------------------------

  if (comPlayPauseButton.selected == true) 
  {
    if (sequence[SelectedSlot].speed > 0) frameRate(1000/sequence[SelectedSlot].speed); //convert from miliseconds to FPS
    else frameRate(30);

    PlayBackFrameNum++;
    if (PlayBackFrameNum >= sequence[SelectedSlot].totalFrames) PlayBackFrameNum = 0;
  }

  int i = 0;

  try {
    //default style
    stroke(255); //black outline - could also do white(255)
    strokeWeight(1);

    //output preview
    for (i = 0; i < matrix.patchedPixels; i++)
    {
      int pix = color(sequence[SelectedSlot].dataFrames[PlayBackFrameNum][(i*3)], sequence[SelectedSlot].dataFrames[PlayBackFrameNum][(i*3)+1], sequence[SelectedSlot].dataFrames[PlayBackFrameNum][(i*3)+2]);
      fill(pix);
      rect(cPreviewXPos+(PatchCoordX[i]*displayPixSize), cPreviewYPos+(PatchCoordY[i]*displayPixSize), displayPixSize, displayPixSize);
    }
  }
  catch(Exception e) {
    //colors from dataframes couldn't be loaded, display a blank matrix. Not a great solution but is quick
    //continue displaying from where it errored
    for (i = i; i < matrix.patchedPixels; i++)
    {
      int pix = color(0, 0, 0);
      fill(pix);
      rect(cPreviewXPos+(PatchCoordX[i]*displayPixSize), cPreviewYPos+(PatchCoordY[i]*displayPixSize), displayPixSize, displayPixSize);
    }
  }

  if ((device.Channels/3) < matrix.patchedPixels)
  {
    stroke(255, 0, 0); 
    fill(0, 0);

    for (i = (device.Channels/3); i < matrix.patchedPixels; i++)
    {
      rect(cPreviewXPos+(PatchCoordX[i]*displayPixSize), cPreviewYPos+(PatchCoordY[i]*displayPixSize), displayPixSize, displayPixSize);
    }

    fill(255);
    text("Hardware does not support that many pixels, pixels will be cropped(red box). Or connect a device.", 295, 115);
  } //end pixel amount if

  //---------------------------------------------------------------------------------------------------------------------

  //Writes the pixel# as text over the preview grid
  if (comPlayPauseButton.selected == false) 
  {
    try {
      noStroke();
      fill(255);
      textAlign(CENTER);
      textSize(12);      

      //output preview
      for (i = 0; i < matrix.patchedPixels; i++)
      {
        text(""+(i+1), cPreviewXPos+(PatchCoordX[i]*displayPixSize), cPreviewYPos+(PatchCoordY[i]*displayPixSize)+(displayPixSize/2), displayPixSize, displayPixSize);
      }
    }
    catch(Exception e) {
    }
  } //end if()

  //------------------------------------------- Overlay Menus --------------------------------------------------------------

  if (OverlayMenuID > 0)
  {  
    fill(gui.windowBackground, 128);
    rect(0, 0, width, height);  
    OverlayMenus[OverlayMenuID].display();
  }

  //---------------------------------------------------------------------------------------------------------------------

  if (serialPort.list().length != comSerialPortDD.numStrs) BuildCOMDropDown(); //updates drop down if available serial port list has changed

  if (GlobalDDOpen == true) DropDownPointer.display(); //if a drop down is open, display() it again to ensure it overlays all other drawing

  if (GlobalDragging == true) SliderPointer.runCallBack(); //slider drags with GUI refreshes, but uses mouse location so values are un-affected

  //---------------------------------------- Upload Progress Bar ----------------------------------------------------------------

  if (UploadInProgress == true)   DisplayProgressBar(width/2, height/2);

  //------------------------------------- User Notifications ----------------------------------------------------------------

  if (ConfirmBoxIDNum > 0)
  {
    fill(gui.windowBackground, 128);
    rect(0, 0, width, height);

    fill(gui.windowStroke);
    stroke(gui.buttonHighlightColor);
    strokeWeight(3);
    rect((1024/2)-120, (768/2)-60, 240, 120, 20);
    fill(0);
    textSize(14);
    textAlign(CENTER);
    text(cConfirmBoxText[ConfirmBoxIDNum], (1024/2)-110, (768/2)-50, 220, 100);
    textAlign(LEFT);

    ConfirmButtonYes.display();
    ConfirmButtonNo.display();
  }   

  //---------------------------------------------------------------------------------------------------------------------
} //end draw()

//=============================================================================================================================

public void DisplayProgressBar(int xpos, int ypos)
{
  //only call from draw, absolute positions
  //xpos and ypos are on center
  //300 wide, 120 tall

  fill(gui.windowBackground, 128);
  rect(0, 0, width, height); //grey out
  stroke(gui.buttonHighlightColor);
  strokeWeight(4);
  fill(gui.windowBackground);
  rect(xpos-150, ypos-80, 300, 120);

  stroke(255);  
  fill(0);  
  strokeWeight(2);
  rect(xpos-130, ypos, 260, 25);

  textAlign(CENTER);
  textSize(24);
  fill(255);
  text("Upload In Progress:", xpos, ypos-50);
  textAlign(LEFT);
  noStroke();

  UploadProgressCancel.xpos = xpos - (UploadProgressCancel.bWidth/2);
  UploadProgressCancel.ypos = ypos - 35;

  UploadProgressCancel.display();

  //rounds up since only full packets are expected
  UploadProgressVar = (float)SendCounterB / (float)(ceil((float)IndexedMemoryUsage/(float)device.CurrentBlockSize) * device.CurrentBlockSize);

  if (SendCounterB > 10) //fixes problem with software freeze, maybe was only on basic, but either way
  {  

    for (int v = 0; v != PApplet.parseInt(UploadProgressVar*21); v++)
    {
      colorMode(HSB, 25);
      fill(v, 25, 25);
      rect((xpos+5-130)+(v*12), ypos+5, 8, 16, 6);
    }  
    colorMode(RGB, 255);
  }
}

//=============================================================================================================================
/*
//Prevents application from exiting successfully, disabled as it is not required
void exit()
 {
 println("running exit code");
 FileZipper.clearTempFolder(); //empties the temp folder at application close
 }
 */
//===================================================================================================

//===================================================================================================  

public void fileSelected(File selection) 
{
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    println("User selected " + selection.getAbsolutePath());
    SelectedFilePath = selection.getAbsolutePath();  

    switch(FileBrowserAction)  
    {
    case 0: //null

      break;
    case 1: //load sequence file
      if (SelectedFilePath.indexOf(".auroramatrix") > 0)
      {
        LoadSequencesFile(SelectedFilePath);
      } else
      {
        ShowNotification(13);
        println("Not a correct aurora matrix sequences file");
      }
      break;
    case 2: //save sequence file
      //If overwriting a save file, need to strip off the extension so it can be added back on without duplication
      if (SelectedFilePath.contains("."))
      {
        //println("RAN "+SelectedFilePath.substring(0, SelectedFilePath.lastIndexOf('.')));
        SelectedFilePath = SelectedFilePath.substring(0, SelectedFilePath.lastIndexOf('.'));
      }
      SaveSequencesFile(SelectedFilePath); //now save with the edited file path
      break;
    case 3: //load File Play
      //reset GUI and preview variables
      comPlayPauseButton.selected = false;
      PlayBackFrameNum = 0;
      //now load sequence into object
      sequence[SelectedSlot].loadSequence(SelectedFilePath);
      DisplayMessageStr = "Loaded FilePlay file";
      break; 
    case 4: //load patch file
      //reset a variables
      comPlayPauseButton.selected = false;
      PlayBackFrameNum = 0;

      DisplayMessageStr = "Loaded Patch File";
      matrix.setPatchFile(SelectedFilePath);
      break;
    }//end switch
  }//end else
}//end func

//===================================================================================================  

public void LoadFilePlayModeFile(String passedFileName, int passedSlot)
{
  println("LoadFilePlayModeFile() with "+passedFileName);  

  int i = 0;
  int q = 0;
  //Loads text file with lines as frames, comma delimiated entries
  try
  {  
    String[] lines = loadStrings(passedFileName); 
    String[] WorkString = new String[cSoftwareMaxChans];  
    //println(lines.length+"    "+lines[0]);
    WorkString = split(lines[0], ',');  //updates WorkString.length to move to DataSize

    if (lines.length > device.MaxFramesGlobal) println("FilePlay file too large");
    else {  
      //This latches the new File Play channel amount to the sequence

      //Update Sequence Frame Amount
      sequence[passedSlot].totalFrames = (lines.length);
      sequence[passedSlot].dataFrames = new int[sequence[passedSlot].totalFrames][matrix.patchedChannels];

      println("Loading file play, length: "+lines.length+" using Datasize: "+matrix.patchedChannels);  

      for (q = 0; q < lines.length; q++)  
      {    
        WorkString = split(lines[q], ',');

        for (i = 0; i < matrix.patchedChannels; i++)  //already calced Datasize, prevents over runs
        {  
          WorkString[i] = trim(WorkString[i]);//removes any whitespace

          if (WorkString[i].equals("") == false)
          {
            sequence[passedSlot].dataFrames[q][i] = PApplet.parseInt(WorkString[i]);
          }
        } //end i for()
      } //end q for()
    } //end else
  }
  catch(Exception e)
  {
    ShowNotification(1); 
    println(i+"   "+q);
    println("File Play file failed to load");
    return;
  }   

  println("File Play file loaded correctly");
} //end func

//===================================================================================================

public void LoadDeviceFile()
{
  //Called when device responds to a Connection Request
  //Loads a TXT file with the device specific defines and variables, loads into the device object

  if (device.HardwareID == 0)
  {
    //Null
    device.Name = "No Device";
    return;
  }

  String BuildString = "devices"+File.separator+""+device.HardwareID+"-nled-contr-revs.txt";   //loads file with current firmware rev info

  String[] lines = loadStrings(BuildString); 
  String[] WorkString = new String[30]; //may need to up this  

  BuildString = "";  

  for (int i = 0; i != lines.length; i++)
  {
    WorkString = split(lines[i], '\t');
    println(PApplet.parseInt(WorkString[0])+" : "+PApplet.parseInt(WorkString[1]));    

    if (PApplet.parseInt(WorkString[0]) == device.FirmwareVersion)//version
    {

      if (PApplet.parseInt(WorkString[1]) < device.FirmwareRevision && i == 0)//version
      {
        println("Revison higher than local "+device.FirmwareRevision);
        BuildString = "devices"+File.separator+""+device.HardwareID+"-"+PApplet.parseInt(WorkString[0])+"-"+PApplet.parseInt(WorkString[1])+"-nled-contr.txt";
        break;
      } //end [1] if

      if (PApplet.parseInt(WorkString[1]) == device.FirmwareRevision)//version
      {
        println("REVISION FOUND with "+i);
        BuildString = "devices"+File.separator+""+device.HardwareID+"-"+PApplet.parseInt(WorkString[0])+"-"+PApplet.parseInt(WorkString[1])+"-nled-contr.txt";
        break;
      } //end [1] if
    } //end [0] if
  }//end for()

  //detects if a revID is lower than what is found, use the highest
  if (BuildString.equals(""))
  {
    println("Device File For V. & Rev. Not Found - Using Most Recent Version");  
    lines = loadStrings("devices"+File.separator+""+device.HardwareID+"-nled-contr-revs.txt"); 
    WorkString = split(lines[0], '\t');
    BuildString = "devices"+File.separator+""+device.HardwareID+"-"+PApplet.parseInt(WorkString[0])+"-"+PApplet.parseInt(WorkString[1])+"-nled-contr.txt";
  }

  println("LoadDeviceFile() - "+BuildString);  

  lines = loadStrings(BuildString); 
  int LinePointer = 1;

  //WorkString[0] is HardwareID, already checked....
  WorkString = split(lines[LinePointer++], '\t');
  device.Name = WorkString[1]; //String Name

  WorkString = split(lines[LinePointer++], '\t');
  device.WebpageURL = WorkString[1];    //String URL


  WorkString = split(lines[LinePointer++], '\t');
  if (device.FirmwareVersion < PApplet.parseInt(WorkString[1]))
  {
    println("FIRMWARE OUTDATED "+PApplet.parseInt(WorkString[1])+" vs "+device.FirmwareVersion);
    if (devConnected == true) //if no device is connected don't show the message
    {  
      //  ConfirmBoxIDNum = 8;
    }
  }

  WorkString = split(lines[LinePointer++], '\t');
  if (device.FirmwareRevision < PApplet.parseInt(WorkString[1]))
  {
    println("FIRMWARE REVISION OUTDATED "+PApplet.parseInt(WorkString[1])+" vs "+device.FirmwareRevision);
    if (devConnected == true) //if no device is connected don't show the message
    {
      // ConfirmBoxIDNum = 8;
    }
  }  

  //device.HardwareVersion = int(WorkString[1]);
  LinePointer++;  //for hardwareversion which is not read

  WorkString = split(lines[LinePointer++], '\t');
  device.Channels = PApplet.parseInt(WorkString[1]);   

  WorkString = split(lines[LinePointer++], '\t');
  device.LockChannelAmt = PApplet.parseBoolean(WorkString[1]);   

  WorkString = split(lines[LinePointer++], '\t');
  device.DataSpace = PApplet.parseInt(WorkString[1]);   

  WorkString = split(lines[LinePointer++], '\t');
  device.IndexBlockSize = PApplet.parseInt(WorkString[1]);
  try {   
    device.HPVBlockSize = PApplet.parseInt(WorkString[2]);   
    device.SeqMemoryBlockSize = PApplet.parseInt(WorkString[3]);
  }
  catch(Exception e) 
  {
    println("Error loading block sizes, using single value");
    device.CurrentBlockSize  = device.IndexBlockSize;
    device.HPVBlockSize = device.CurrentBlockSize;   
    device.SeqMemoryBlockSize = device.CurrentBlockSize;
  }

  WorkString = split(lines[LinePointer++], '\t');
  device.EraseBlockSize = PApplet.parseInt(WorkString[1]);  

  WorkString = split(lines[LinePointer++], '\t');
  device.HWPVSpace = PApplet.parseInt(WorkString[1]);    

  WorkString = split(lines[LinePointer++], '\t');
  device.MaxIndexedSequences = PApplet.parseInt(WorkString[1]);  

  WorkString = split(lines[LinePointer++], '\t');
  device.IndexMemoryModel = PApplet.parseInt(WorkString[1]);      

  WorkString = split(lines[LinePointer++], '\t');
  device.FillUpperData = PApplet.parseBoolean(WorkString[1]); 

  WorkString = split(lines[LinePointer++], '\t');
  device.Bit16Processor = PApplet.parseBoolean(WorkString[1]);    

  WorkString = split(lines[LinePointer++], '\t');
  device.Bit16Mode = PApplet.parseBoolean(WorkString[1]);    

  WorkString = split(lines[LinePointer++], '\t');
  device.BasicVersion = PApplet.parseInt(WorkString[1]);      

  WorkString = split(lines[LinePointer++], '\t');
  device.LinkedSupport = PApplet.parseBoolean(WorkString[1]);   

  WorkString = split(lines[LinePointer++], '\t');
  device.GammaCorrection = PApplet.parseInt(WorkString[1]);  //0=none, 1=8-bit gamma table, 2=16-bit calculation

  WorkString = split(lines[LinePointer++], '\t');
  device.AccelerometerMode = PApplet.parseInt(WorkString[1]);

  WorkString = split(lines[LinePointer++], '\t');
  device.MaxSpeed = PApplet.parseInt(WorkString[1]);    

  WorkString = split(lines[LinePointer++], '\t');
  device.MaxFrames = PApplet.parseInt(WorkString[1]);     //Max Frames for sequences that use DataFrameVar[] on device
  device.MaxFramesGlobal = PApplet.parseInt(WorkString[2]);  //max frames for POVs currently and anything that uses GlobalFrameVar

  WorkString = split(lines[LinePointer++], '\t');
  device.ConfigFlagsStr = WorkString[1];    

  WorkString = split(lines[LinePointer++], '\t');
  device.ConfigBytesStr = WorkString[1];    

  WorkString = split(lines[LinePointer++], '\t');
  device.DMXModeLabels = new String[16];
  device.DMXModeLabels = split(WorkString[1], ',');

  device.DMXModeAmt = device.DMXModeLabels.length;     

  //Init GUI Elements
  DMXModesDD.labels = device.DMXModeLabels;   //drop down
  DMXModesDD.numStrs = device.DMXModeAmt;   //drop down

  //Pixel Type Modes if applicable
  WorkString = split(lines[LinePointer++], '\t');
  device.ListedChipsets = PApplet.parseInt( WorkString[1]);   

  if (device.ListedChipsets > 0)
  {  
    String[] tempPixLables = new String[16];

    tempPixLables = new String[device.ListedChipsets]; //use same array as DMX  
    PixelChipsetID = new int[device.ListedChipsets];

    //pixel Type Compatibility
    for (int i = LinePointer; i != (LinePointer+device.ListedChipsets); i++)
    {
      WorkString = split(lines[i], '\t');
      tempPixLables[i-LinePointer] = WorkString[1];
      PixelChipsetID[i-LinePointer] = PApplet.parseInt(WorkString[2]);
    }

    PixelChipsetDD.labels = tempPixLables;
    PixelChipsetDD.numStrs = device.ListedChipsets;
  }  //end if()

  // =================   END DATA LOAD  =========================================================

  //set Memory/DataSpace label
  if (device.DataSpace > 1000000) device.MemorySpaceString = nf((((float)device.DataSpace/1048576)), 1, 2)+"MB";
  else device.MemorySpaceString = nf((((float)device.DataSpace/1024)), 1, 2)+"KB";  

  println("Device File Successfully Loaded - Starting Configuration Modules");

  //================  init Hardware Usage Modules  ============================================

  modulePosX = cInputModuleOffsetX; //constants
  modulePosY = cInputModuleOffsetY;

  moduleHoldX = 0;
  moduleHoldY = 0;
  moduleHoldSz = 0;

  for (int i = 0; i != InputModule.length; i++) InputModule[i].Enabled = false; //reset all modules to disabled

  ConfigFlags = 0; //reset flags here.....
  DevStrArray = split(device.ConfigFlagsStr, ',');
  String[] ModuleString = new String[16];  //reset
  String[] ByteStrArray = new String[16];
  ByteStrArray = split(device.ConfigBytesStr, ',');  
  //Now single delimenated string is an array, the pointer is the bit number in ConfigFlags  

  //First handle prioritized tiles ====================================================================================

  //z counts from high to low, and checks for tiles that are not yet placed and are above that number
  for (int z = 260; z > 0; z-=10)
  {  
    for (int y = 0; y != InputModule.length; y++)
    {  
      if (InputModule[y].Enabled == false && InputModule[y].Priority >= z)
      {
        ModuleString = split(InputModule[y].ModuleFlagStr, ','); //fill with bit string

        for (int q = 0; q != ModuleString.length; q++)
        {  
          for (int i = 0; i != DevStrArray.length; i++)
          {  
            if (DevStrArray[i].equals(ModuleString[q]) == true) //if string matches
            {
              if (InputModule[y].Enabled == false) 
              {
                //println("Bit tile placed priority "+z);
                //println("Module Priority: "+InputModule[y].Priority);              
                InputModule[y].PlaceTile();
              }
            }
          } //end i for()
        } //end q for()  

        //==============================================================

        ModuleString = split(InputModule[y].ModuleByteStr, ','); //refill with byte string

        for (int q = 0; q != ModuleString.length; q++)
        {  
          for (int i = 0; i != ByteStrArray.length; i++)
          {  
            if (ByteStrArray[i].equals(ModuleString[q]) == true) //if string matches
            {
              if (InputModule[y].Enabled == false) 
              {
                //println("Byte tile placed priority "+z);
                //println(InputModule[y].Priority);              
                InputModule[y].PlaceTile();
              }
            }
          } //end i for()
        } //end q for()
      }
    } //end y for()
  }  //end z for()


  ///Now place all unprioritized not yet enabled tiles
  for (int y = 0; y != InputModule.length; y++)
  {  
    if (InputModule[y].Enabled == false)
    {
      ModuleString = split(InputModule[y].ModuleFlagStr, ',');

      for (int q = 0; q != ModuleString.length; q++)
      {  
        for (int i = 0; i != DevStrArray.length; i++)
        {  
          if (DevStrArray[i].equals(ModuleString[q]) == true) //if string matches
          {
            if (InputModule[y].Enabled == false) 
            {            
              InputModule[y].PlaceTile();
            }
          }
        } //end i for()
      } //end q for()  

      //==============================================================

      ModuleString = split(InputModule[y].ModuleByteStr, ',');

      for (int q = 0; q != ModuleString.length; q++)
      {  
        for (int i = 0; i != ByteStrArray.length; i++)
        {  
          if (ByteStrArray[i].equals(ModuleString[q]) == true) //if string matches
          {
            if (InputModule[y].Enabled == false) 
            {          
              InputModule[y].PlaceTile();
            }
          }
        } //end i for()
      } //end q for()
    }
  } //end y for()
  //Modules placed and prioritized

  // ================   Set additional define flags that require configurations to be loaded ==============

  //if 16-bit data mode or has 16-bit pack function for serial, live control would also be 16-bit capable
  if (device.Bit16Mode == true || IsOnConfigFlags("f8or16Bit"))   device.Bit16LiveMode = true;

  // ================   grey out unavailable GUI elements ==============

  UpdateIndexMemoryUsage(); //update memory usage

  ShowNotification(1);
  println("Configuration Modules Successfully Loaded - end LoadDeviceFile()");
}//end func()

//===================================================================================================  

public void SaveSequencesFile(String passedFileLoc)
{
  println("SaveSequencesFile() to "+passedFileLoc);

  //Use user selected file name and directory
  //create string array containing file names and directories
  //Build aurora matrix seuquence file
  //ZIP aurm file and all the sequences into a folder

  passedFileLoc = passedFileLoc+".auroramatrix"; //add file extension, it is still a regular ZIP

  //checks to see if there is an undefined sequence on the index, that is one with no defined file
  for (int i = 0; i < IndexedSequences; i++) 
  {
    if (sequence[i].fileName.equals(""))
    {
      //sequence fileName is not defined
      //println(i+"   vs   "+IndexedSequences);
      IndexedSequences = i;
      break; //leave for()
    }
  } //end for()


  String[] filesToZipList = new String[IndexedSequences+2];

  for (int i = 0; i < IndexedSequences; i++) 
  {
    filesToZipList[i] = sequence[i].fileName;
  } //end for()

  //-------------------- Save Aurora Matrix Sequenc File ------------------------------

  String[] WorkString = new String[IndexedSequences+2]; //may need to up this

  WorkString[0] = "//NLED Aurora Matrix sequence save file - see readme for details";
  if (matrix.patchFileName.length() > 0) //incase patch is not defined, would error otherwise
  {
    String[] qpath = splitTokens(matrix.patchFileName, System.getProperty("file.separator"));
    WorkString[1] = qpath[qpath.length-1];
  }


  for (int i = 0; i < IndexedSequences; i++) 
  {
    try {
      WorkString[2+i] = sequence[i].getSaveString();
    }
    catch(Exception e)
    {
      println("getSaveString() with sequence#: "+i+" failed");
    }
  }

  //saves it to the temp folder
  saveStrings(sketchPath()+File.separator+"temp"+File.separator+"sequenceFile.aurm", WorkString);

  filesToZipList[IndexedSequences] = sketchPath()+File.separator+"temp"+File.separator+"sequenceFile.aurm";

  filesToZipList[IndexedSequences+1] = matrix.patchFileName;

  //------------------------------------ ZIP Up The Files -----------------------------------------

  FileZipper.zipFiles(filesToZipList, passedFileLoc);

  println("SaveSequencesFile() completed");
} //end func

//===================================================================================================  

public void LoadSequencesFile(String passedFileLoc)
{
  println("LoadSequencesFile() with "+passedFileLoc);
  //unzip to temp, find the .aurm file, define sequence object

  FileZipper.clearTempFolder(); //clear temp folder first to prevent issues
  FileZipper.unzipToTemp(passedFileLoc); //unzip files to local temp folder

  String[] lines = loadStrings(FileZipper.localTempFolder+"sequenceFile.aurm"); //always same name
  printArray(lines);

  matrix.setPatchFile(FileZipper.localTempFolder+lines[1]); //load patch

  IndexedSequences = lines.length-2;

  //loads the sequence parameters from the .aurm file
  for (int i = 0; i < IndexedSequences; i++) 
  {
    sequence[i].selected = false;
    sequence[i].setLoadString(lines[i+2]);
  }
  
  //update index and scrollbar
  SelectedSlot = 0;
  sequence[SelectedSlot].selected = true;
  indexScrollBar.setValue(0);
}

//===================================================================================================

//===================================================================================================

public void UpdateIndexGUIElements()
{
  for (int i = 0; i < cMaxIndexViewed; i++) //5 is how many indexed sequences can be viewed
  {
    SequenceSpeedField[i].setValue(sequence[i+indexScrollBar.getValue()].speed);
    SequencePlayModeDD[i].selStr = (sequence[i+indexScrollBar.getValue()].playMode);
    SequenceTransitionDD[i].selStr = (sequence[i+indexScrollBar.getValue()].transition);
  }
}

//===================================================================================================

public void SetOverlayMenuID(int passedVal)
{
  OverlayMenuID = passedVal;
  OverlayMenus[OverlayMenuID].initMenu();
}

//===================================================================================================

public void UpdateIndexMemoryUsage()
{
    IndexedMemoryUsage = 0; //clear it
  
    //Check if the amount of patched pixels is higher than the hardware device supports
    if (matrix.patchedPixels > (device.Channels/3)) UtilizedChannels = device.Channels; //more channels are defined in the patch than hardware supports
    else UtilizedChannels = matrix.patchedPixels * 3; //same as patchedChannels, but should not change that. Use the global.

    for (int i = 0; i < IndexedSequences; i++) 
    {
    sequence[i].memorySize = (sequence[i].totalFrames*UtilizedChannels)+cHeaderByteLen;  
    IndexedMemoryUsage += sequence[i].memorySize;
    }
    
    //println(IndexedMemoryUsage);  
    IndexedMemoryUsageStr =  ""+nf((((float)IndexedMemoryUsage / device.DataSpace)*100),1,2); //update string for GUI display
    
    if(UtilizedChannels != matrix.patchedPixels)   println("UpdateIndexMemoryUsage() result: "+IndexedMemoryUsage+"  Channel Size Was Cropped");
    else println("UpdateIndexMemoryUsage() result: "+IndexedMemoryUsage);
}

//===================================================================================================  

public void SetConfirmBox(int passedVal)
{
  ConfirmBoxIDNum = passedVal;
}

//============================================================================================

public void ShowNotification(int passedVal)
{
  println("ShowNotification() with "+passedVal);

//ShowNotification = passedVal;

  switch(passedVal)
  {
  case 0: //null

    break;
  case 1:
    DisplayMessageStr = "Connection and Definition Loading Successful.";
    break;
  case 2:
    DisplayMessageStr = "Available Serial Ports Have Updated.";
    break;    
  case 3:
    DisplayMessageStr = "Port Opened Successfully. Attempting Connection, please wait.";
    break; 
  case 4:
    DisplayMessageStr = "Configurations uploaded, waiting for acknowledge.";
    break;    
  case 5:
    DisplayMessageStr = "Configurations Successfully Uploaded";
    break;    
  case 6:
    DisplayMessageStr = "Command timed out. Device did not respond. Disconnecting from port.";
    break; 
  case 7:
    DisplayMessageStr = "This device does not respond, and may not actually be connected. See activity LED and datasheet for details.";
    break; 
   case 8:
    DisplayMessageStr = "Define Previous Sequence Before Adding More.";
    break; 
    case 9:
    DisplayMessageStr = "Sequence Removed From Index.";
    break;  
    case 10:
    DisplayMessageStr = "Sequence Added To Index.";
    break; 
    case 11:
    DisplayMessageStr = "Attempting to Upload Indexed Color Sequences.";
    break;    
      case 12:
    DisplayMessageStr = "Configurations Upload Completion Success.";
    break;   
   case 13:
    DisplayMessageStr = "Not the correct file type.";
    break;     
   case 16:
    DisplayMessageStr = "Could not load device file, firmware may be outdated or not compatible. See www.NLEDshop.com/deviceupdates";
    break;   
  case 22:
    DisplayMessageStr = "Port Can Not Be Opened.";
    break;
    
  }//end switch
}

//============================================================================================

//======================================================================================================

public void ChipsetDDFunc()
{
 FuncRecommendColorShift(); 
}

//======================================================================================================

public void SequencePlayModeDDFunc()
{
    sequence[SelectedSlot].playMode = DropDownPointer.selStr;
}

//======================================================================================================

public void SequenceTransitionDDFunc()
{
  sequence[SelectedSlot].transition = DropDownPointer.selStr;
}

//======================================================================================================

public void SequenceSpeedField()
{
  genericNumberInputField(); //common to all
  sequence[SelectedSlot].speed = (int)numberInputFieldPtr.value; //update sequence object with GUI elements value
}

//======================================================================================================

public void sliderHandlerIndex()
{
  //updates the scrollbar and the index gui elements with the relavent sequence data. Runs whenever scrollbar is moved.
  indexScrollBar.max = IndexedSequences-2;
  
  UpdateIndexGUIElements();
}

//======================================================================================================

public void txtHandleGenericNumber()
{
  if (textFieldPtr.inputMethod > 0)
  {
    if (PApplet.parseInt(textFieldPtr.label) > textFieldPtr.maxValue) textFieldPtr.label = ""+textFieldPtr.maxValue;
    else if (PApplet.parseInt(textFieldPtr.label) < textFieldPtr.minValue)  textFieldPtr.label = ""+textFieldPtr.minValue;
  }
  textFieldPtr.selected = false;
}

//======================================================================================================

public void genericDDCallBack()
{
  //do nothing
}

//======================================================================================================

public void genericHandlerTextField()
{
  textFieldPtr.selected = false;
}

//======================================================================================================

public void genericNumberInputFieldFloats()
{
  //println("generatedTextSize() with "+numberInputFieldPtr.selected);

  switch(numberInputFieldPtr.selected)
  {
  case 0:
    break;
  case 1: //decrement
    if (numberInputFieldPtr.value > numberInputFieldPtr.minValue)   numberInputFieldPtr.value-=0.1f;
    break;
  case 2: //increment
    if (numberInputFieldPtr.value < numberInputFieldPtr.maxValue)   numberInputFieldPtr.value+=0.1f;
    break; 
  case 3: //typed in selection
    numberInputFieldPtr.selected = 0; //clear selection

    numberInputFieldPtr.value = PApplet.parseFloat(numberInputFieldPtr.label); //covert string to float
    if (numberInputFieldPtr.value < numberInputFieldPtr.minValue)  numberInputFieldPtr.value = numberInputFieldPtr.minValue;
    else if (numberInputFieldPtr.value > numberInputFieldPtr.maxValue)   numberInputFieldPtr.value = numberInputFieldPtr.maxValue;
    break;
  } //end switch

  numberInputFieldPtr.label = str(numberInputFieldPtr.value);
  println("value: "+numberInputFieldPtr.value);
} //end func()

//======================================================================================================

public void genericNumberInputField()
{
 // println("generatedTextSize() with "+numberInputFieldPtr.selected);

  switch(numberInputFieldPtr.selected)
  {
  case 0:
    break;
  case 1: //decrement
    if (numberInputFieldPtr.value > numberInputFieldPtr.minValue)   numberInputFieldPtr.value--;
    break;
  case 2: //increment
    if (numberInputFieldPtr.value < numberInputFieldPtr.maxValue)   numberInputFieldPtr.value++;
    break; 
  case 3:
    numberInputFieldPtr.selected = 0; //clear selection

    numberInputFieldPtr.value = PApplet.parseInt(numberInputFieldPtr.label); //covert string to int
    if (numberInputFieldPtr.value < numberInputFieldPtr.minValue)  numberInputFieldPtr.value = numberInputFieldPtr.minValue;
    else if (numberInputFieldPtr.value > numberInputFieldPtr.maxValue)   numberInputFieldPtr.value = numberInputFieldPtr.maxValue;
    break;
  } //end switch

  numberInputFieldPtr.label = str(numberInputFieldPtr.value);
  println("value: "+numberInputFieldPtr.value);
} //end func()

//======================================================================================================
/*
class guiColorSelector
{
  //relies on being opened by a guiColorSquare object click. Not great but works for now.
  int xpos; 
  int ypos; 
  int mWidth;
  int mHeight;

  int red, green, blue, white, alpha;

  color selColor;

  boolean menuOpen;

  //--------------------------------------------------------------------------

  guiColorSelector(int iWidth, int iHeight)
  {
    mWidth = iWidth;
    mHeight = iHeight;
    menuOpen = false;
  }

  //--------------------------------------------------------------------------

  void display()
  {
    fill(gui.menuBackground);
    stroke(0); //black
    strokeWeight(4);

    rect(xpos, ypos, mWidth, mHeight, 20); //draw menu background

    fill(0);
    textSize(18);
    text("Choose a Color:", xpos+20, ypos+30);
    textSize(20);
    textAlign(RIGHT);
    textLeading(40);
    text("Red:\nGreen:\nBlue:\nWhite:\nAlpha:", xpos+70, ypos+70);
    textAlign(LEFT);
    image(imgColorSelector, xpos+10, ypos+250);

    menuColorSelClose.display();  
    menuColorSelNIFRed.display();
    menuColorSelNIFGreen.display();
    menuColorSelNIFBlue.display();
    menuColorSelNIFWhite.display();
    menuColorSelNIFAlpha.display();

    menuColorSelSliderRed.display();
    menuColorSelSliderGreen.display();
    menuColorSelSliderBlue.display();
    menuColorSelSliderWhite.display();
    menuColorSelSliderAlpha.display();
  } //end display()

  //--------------------------------------------------------------------------

  void closeMenu()
  {
    menuOpen = false;
    colorSquarePtr.selColor = color(red,green,blue,alpha);
    method(colorSquarePtr.callBack);
  }

  //--------------------------------------------------------------------------

  void initMenu(guiColorSquare passedSq, color passedColor)
  {
    menuOpen = true;
    colorSquarePtr = passedSq;
    
    xpos = mouseXS; 
    ypos = mouseYS;

    //make sure window does not open off-screen
    if((xpos+mWidth) > width) xpos = width-mWidth;
    if((ypos+mHeight) > height) ypos = height-mHeight;

    menuColorSelClose.xpos = (xpos+mWidth) - (menuColorSelClose.bWidth+20);
    menuColorSelClose.ypos = ypos+15; 

    menuColorSelNIFRed.xpos = xpos+250;
    menuColorSelNIFRed.ypos = ypos+50;
    menuColorSelNIFGreen.xpos = xpos+250;
    menuColorSelNIFGreen.ypos = ypos+90;
    menuColorSelNIFBlue.xpos = xpos+250;
    menuColorSelNIFBlue.ypos = ypos+130;
    menuColorSelNIFWhite.xpos = xpos+250;
    menuColorSelNIFWhite.ypos = ypos+170;
    menuColorSelNIFAlpha.xpos = xpos+250;
    menuColorSelNIFAlpha.ypos = ypos+210;

    menuColorSelSliderRed.xpos = xpos+80;
    menuColorSelSliderRed.ypos = ypos+55;
    menuColorSelSliderGreen.xpos = xpos+80; 
    menuColorSelSliderGreen.ypos = ypos+95;
    menuColorSelSliderBlue.xpos = xpos+80;
    menuColorSelSliderBlue.ypos = ypos+135;
    menuColorSelSliderWhite.xpos = xpos+80;
    menuColorSelSliderWhite.ypos = ypos+175;
    menuColorSelSliderAlpha.xpos = xpos+80;  
    menuColorSelSliderAlpha.ypos = ypos+215;

    //fill with passed cvolor
    red = (int)red(passedColor);
    green = (int)green(passedColor);
    blue = (int)blue(passedColor);
    //white is not used yet - just in case for future
    alpha = (int)alpha(passedColor);
    updateColorGUIElements();

    println("Color Selector Intialized");
  }
  
  //--------------------------------------------------------------------------

  void updateColorGUIElements()
  {
    menuColorSelSliderRed.setValue(guiColorSelectorMenu.red);
    menuColorSelNIFRed.setValue(guiColorSelectorMenu.red);
    menuColorSelSliderGreen.setValue(guiColorSelectorMenu.green);
    menuColorSelNIFGreen.setValue(guiColorSelectorMenu.green);
    menuColorSelSliderBlue.setValue(guiColorSelectorMenu.blue);
    menuColorSelNIFBlue.setValue(guiColorSelectorMenu.blue);
    menuColorSelSliderWhite.setValue(guiColorSelectorMenu.white);
    menuColorSelNIFWhite.setValue(guiColorSelectorMenu.white);
    menuColorSelSliderAlpha.setValue(guiColorSelectorMenu.alpha);
    menuColorSelNIFAlpha.setValue(guiColorSelectorMenu.alpha);
  }

  //--------------------------------------------------------------------------
  
  boolean over()
  {
    if (mouseXS >= xpos && mouseXS <= xpos+mWidth && mouseYS >= ypos && mouseYS <= ypos+mHeight) 
    {      
      return true;
    } else {
      return false;
    }
  }

  //--------------------------------------------------------------------------
  
  boolean overPicker()
  {
    if (mouseXS >= (xpos+10) && mouseXS <= (xpos+10)+imgColorSelector.width && mouseYS >= (ypos+250) && mouseYS <= (ypos+250)+imgColorSelector.height) 
    {      
      return true;
    } else {
      return false;
    }
  }
} //end class

//=====================================================================================================

class guiColorSquare
{
  int xpos; 
  int ypos; 
  int size;
  color selColor;
  int status; 
  boolean selected;

  String callBack;

  //--------------------------------------------------------------------------

  guiColorSquare(int ixpos, int iypos, int isize, String iCallBack)
  {
    xpos = ixpos;
    ypos = iypos;   
    size = isize;
    
    callBack = iCallBack;
    
    selColor = color(0, 0, 0, 0);
    status = 0;
    selected = false;
  }

  //--------------------------------------------------------------------------

  void display()
  {
    stroke(0); //black
    strokeWeight(5);
    fill(255); //fill white as transparency will see through the selected color
    rect(xpos, ypos, size, size);
    stroke(255); //white
    strokeWeight(2);      
    fill(selColor); //selected color
    rect(xpos, ypos, size, size);

    if (selected == true)
    {
      stroke(255, 0, 0); //red
      strokeWeight(3);
      fill(0, 0, 0, 0); //transparent fill
      rect(xpos-(size/8), ypos-(size/8), size+(size/4), size+(size/4));
      selected = false; //remove selected indicator for next display()
    }
  } //end display

  //--------------------------------------------------------------------------

  boolean over()
  {
    if (mouseXS >= xpos && mouseXS <= xpos+size && mouseYS >= ypos && mouseYS <= ypos+size && status == 0) 
    {      
      selected =! selected;  

      if (selected == true) 
      {
      guiColorSelectorMenu.initMenu(this, selColor);
      }
      
      return true;
    } else {
      return false;
    }
  }
} //end class
*/
//=====================================================================================================

class guiNumberInputField
{
  int xpos; 
  int ypos; 
  int size;
  int fieldWidth;

  int minValue;
  int maxValue;

  int inputMethod; //0 = characters only, 1 = numbers only , 2 = numbers only with trailing %, 3 = floats

  String callBack;

  float value;
  String label;
  int status; //0=show, 1=grey out, 2=hide

  int action; //0 = enter, 1 = decrement, 2 = increment

  int selected; //0 = no selection, 1 = decrement, 2 = textfield, 3 = increment
  
  
  boolean MouseOverFlag;

  int colTxtBG;
  int colTxtHL;
  int colTxtStroke;
  int colButtonBG;
  int colButtonStroke;

  guiNumberInputField(int ixpos, int iypos, int isize, int ifieldWidth, int iminValue, int imaxValue, int iinputMethod, String icallBack)
  {
    xpos = ixpos;
    ypos = iypos;   
    size = isize;
    fieldWidth = ifieldWidth;
    minValue = iminValue;
    maxValue = imaxValue;
    inputMethod = iinputMethod;
    callBack = icallBack;

    selected = 0;
    
    value = minValue;
    label = ""+value;

    colTxtBG = gui.textFieldBG;
    colTxtHL = gui.textFieldHighlight;
    colTxtStroke = gui.buttonColor;

    colButtonBG = gui.buttonColor;
    colButtonStroke = gui.textColor;
    
   //Setup mouse over list
  if (software.mouseOverEnabled  == true) NumberInputList.add(this);
  MouseOverFlag = false;   
    
  } //end constructor

  //--------------------------------------------------------------------------

  public void setValue(float passedVal)
  {
    value = passedVal;

    if (inputMethod == 1) label = ""+PApplet.parseInt(value); 
    else label = ""+value;
  }

  //--------------------------------------------------------------------------

  public void handleStatus()
  {
    noStroke();

    if (status == 1) //grey out
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      //    rect(xpos, ypos, fieldWidth+(size/2)+(size*2), size);  

      rect(xpos+size+(size/4), ypos, fieldWidth, size);
      rect(xpos, ypos, size, size, size/4);  //rounded corners
      rect(xpos+(size*1.5f)+fieldWidth, ypos, size, size, size/4);  

      noStroke();
    }
  }

  //--------------------------------------------------------------------------

  public void selectField()
  {
    try {
      deselectTextField(); //handle deselecting previous field if applicable
    }
    catch(Exception e) {
    }

    numberInputFieldPtr = this;
    GlobalLabelStore = label;
    label = "";
    NumberInputFieldActive = true;
  }

  //--------------------------------------------------------------------------

  public void display()
  {
    if (status == 2) return; //element is hidden, do not draw

    strokeWeight(1);
    stroke(colTxtStroke);
    strokeWeight(2);
    fill(colTxtBG);

    if (selected == 3) stroke(colTxtHL);
    else if(mouseOver() == 3) stroke(colTxtHL,128);
    else       stroke(gui.buttonColor);
    rect(xpos+size+(size/4), ypos, fieldWidth, size); //textfield

    //adjust stroke to on higligh and invert text color
    noStroke();
    fill(0);
    textSize((size/1.5f));
    textAlign(CENTER);

    if (inputMethod == 0 || inputMethod == 3) text(label, xpos+(size*1.25f), ypos+((size-(textAscent()+textDescent()))/1.5f), fieldWidth, size);
    else if (inputMethod == 1 && selected == 3) text(label, xpos+(size*1.25f), ypos+((size-(textAscent()+textDescent()))/1.5f), fieldWidth, size);
    else if (inputMethod == 1) text(""+PApplet.parseInt(label), xpos+(size*1.25f), ypos+((size-(textAscent()+textDescent()))/1.5f), fieldWidth, size);  
    else  if (inputMethod == 2)  text(PApplet.parseInt(label)+"%", xpos+(size*1.25f), ypos+((size-(textAscent()+textDescent()))/1.5f), fieldWidth, size); //percentage

    strokeWeight(1);
    stroke(colButtonStroke);

    if (selected == 1) fill(colTxtHL);
    else if(mouseOver() == 1)  fill(colButtonBG, 64);    
    else fill(colButtonBG);
    rect(xpos, ypos, size, size, size/4);  //rounded corners


    if (selected == 2) fill(colTxtHL);
    else if(mouseOver() == 2)  fill(colButtonBG, 64);
    else fill(colButtonBG);
    rect(xpos+(size*1.5f)+fieldWidth, ypos, size, size, size/4);  


    //adjust stroke to on higlight and invert text color
    fill(colButtonStroke);

    textSize((size/1.5f));
    text("-", xpos+(size/2), ypos+(size/1.5f));
    text("+", xpos+(size*2)+fieldWidth, ypos+(size/1.5f));

    handleStatus();

    if (selected !=3) selected = 0; //buttons do not stay highlighted but text field does
  }

//--------------------------------------------------------------------------

  public boolean over() 
  {
    if (status != 0) return false;

    //check decrement button    
    if (mouseXS >= xpos && mouseXS <= xpos+size && mouseYS >= ypos && mouseYS <= ypos+size)
    {     

      selectField();
      selected = 1; //decrement
      method(callBack);
      return true;
    } 

    //check increment button    
    if (mouseXS >= xpos+(size*1.5f)+fieldWidth && mouseXS <= xpos+(size*2.5f)+fieldWidth && mouseYS >= ypos && mouseYS <= ypos+size)
    {     

      selectField();
      selected = 2; //increment
      method(callBack);
      return true;
    } 

    //check text field    
    if (mouseXS >= xpos+(size*1.25f) && mouseXS <= xpos+(size*1.25f)+fieldWidth && mouseYS >= ypos && mouseYS <= ypos+size)
    {     
      selectField();
      selected = 3; //text field
      return true;
    } 

    return false;
  } //end 0ver()
  
  //--------------------------------------------------------------------------
  
   public int mouseOver() 
  {

    if (status != 0) return 0;


    //check decrement button    
    if (mouseXS >= xpos && mouseXS <= xpos+size && mouseYS >= ypos && mouseYS <= ypos+size)
    {     
      return 1;
    } 

    //check increment button    
    if (mouseXS >= xpos+(size*1.5f)+fieldWidth && mouseXS <= xpos+(size*2.5f)+fieldWidth && mouseYS >= ypos && mouseYS <= ypos+size)
    {     
      return 2;
    } 

    //check text field    
    if (mouseXS >= xpos+(size*1.25f) && mouseXS <= xpos+(size*1.25f)+fieldWidth && mouseYS >= ypos && mouseYS <= ypos+size)
    {     
      return 3;
    } 

    return 0;
  } //end 0ver() 
  
} //end class

//=====================================================================================================

class guiCheckBox
{
  int xpos; // rect xposition  
  int ypos; // rect yposition
  int size;
  int colBG;
  int colHL;
  int colOut;  
  boolean selected;
  int status = 0; //0=show, 1=grey out, 2=hide
  String callBack;

  boolean MouseOverFlag;

  guiCheckBox(int ixpos, int iypos, int isize, int iColBG, int iColHL, int iColOut, boolean iSelected, String icallBack)
  {
    xpos = ixpos;
    ypos = iypos;  
    size = isize;
    colBG = iColBG;
    colHL = iColHL;
    colOut = iColOut;
    selected = iSelected;
    callBack = icallBack;

    CheckBoxList.add(this);
    MouseOverFlag = false;
  }

  public void display()
  {
    strokeWeight(1);
    stroke(colOut);
    fill(colBG);

    if (selected==true) 
    {  
      fill(colBG);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colBG, 128);
      }
    } else  
    {  
      fill(colBG);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colBG, 64);
      }
    }    
    rect(xpos, ypos, size, size);  


    if (selected==true)
    {
      line(xpos, ypos, xpos+size, ypos+size);
      line(xpos+size, ypos, xpos, ypos+size);
    }

    handleStatus();
  }//end display()


  public void handleStatus()
  {
    noStroke();

    if (status == 1) 
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      rect(xpos, ypos, size, size);  
      noStroke();
    }
  }

  public boolean over() 
  {
    if ( mouseXS >= xpos && mouseXS <= xpos+size && mouseYS >= ypos && mouseYS <= ypos+size && status == 0)
    {     
      selected =! selected;  
      method(callBack);
      return true;
    } else 
    {
      return false;
    }
  }


  public boolean mouseOver() 
  {
    if ( mouseXS >= xpos && mouseXS <= xpos+size && mouseYS >= ypos && mouseYS <= ypos+size && status == 0)
    {     
      MouseOverFlag = true;  
      return true;
    } else {
      MouseOverFlag = false;
      return false;
    }
  }
}//end class

//======================================================================================

class guiDropDown
{
  //use one over() for detecting the button click and if it is open, then use a different over() with the full tab page size
  String[] labels;
  int numStrs;
  int selStr;
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int bWidth;
  int bHeight;
  int colBG;
  int colHL;
  int colOut;  
  boolean dirMode; //false = down, true = up
  String callBack;

  boolean selected;
  int status;

  boolean MouseOverFlag;

  //--------------------------------------------------------------------------

  guiDropDown(String[] iLabels, int iSelStr, int ixpos, int iypos, int ibWidth, int ibHeight, int iColBG, int iColHL, int iColOut, boolean iDirMode, String icallBack)
  {
    labels = iLabels; 
    numStrs = labels.length;
    selStr = iSelStr;
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = ibHeight;
    colBG = iColBG;
    colHL = iColHL;
    colOut = iColOut;
    dirMode = iDirMode;
    callBack = icallBack;

    selected = false;
    status = 0; //normal display

    DropDownList.add(this);
  }

  //--------------------------------------------------------------------------

  public int getValue()
  {
    return selStr;
  }

  //--------------------------------------------------------------------------

  public void setValue(int passedVal)
  {
    if (passedVal < 0) selStr = 0; //set min
    else if (passedVal > numStrs) selStr = numStrs; //set max
    else selStr = passedVal; //else set to passed
  }

  //--------------------------------------------------------------------------

  public void handleStatus()
  {
    noStroke();

    if (status == 1) 
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      rect(xpos, ypos, bWidth, bHeight);  
      noStroke();
    }
  }  

  //--------------------------------------------------------------------------

  public void displayNormal()
  {
    strokeWeight(1);
    stroke(colOut);

    if (selected==true) 
    {  
      fill(colHL);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colHL, 128);
      }
    } else  
    {  
      fill(colBG);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colBG, 64);
      }
    }

    rect(xpos, ypos, bWidth, bHeight);  
    //adjust stroke to on higlight and invert text color
    textAlign(LEFT);
    fill(colOut);
    textSize((bHeight/1.75f));
    text(labels[selStr], xpos+(bWidth/20), ypos+((bHeight-(textAscent()+textDescent()))/1.5f), bWidth, bHeight);

    fill(255);
    int tempSize = (bWidth/4);
    if (tempSize >  (bHeight/4)) tempSize = (bHeight);

    triangle(xpos+(bWidth-tempSize), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/2), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/4), ypos+(bHeight/4)+(bHeight/2));
    noStroke();
  }

  //--------------------------------------------------------------------------

  public void selectDD()
  {
    selected =! selected;  //toggle selected state

    if (selected == true)
    {  
      DropDownPointer = this;
      GlobalDDOpen = true;
      DropDownMouseOverID = selStr;
    } else
    {
      GlobalDDOpen = false; //clear in case
    }
  }

  //--------------------------------------------------------------------------

  public void display()
  {
    if (status == 2) return; //do not display if status = 2 = hide

    // pushStyle();

    if (selected == false)
    {
      displayNormal();
    } else
    {
      if (dirMode == false)  
      {  
        //down mode
        fill(colBG);
        displayNormal();
        strokeWeight(1);
        stroke(colOut);  

        for (int i = 0; i != numStrs; i++)
        {
          fill(colBG);
          rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);
          if (i == selStr) fill(colHL);
          else fill(colBG);
          //drop down drawn

          if (DropDownMouseOverID > 0 && DropDownMouseOverID == (i+1) && MouseOverFlag == true)
          {
            if (i == selStr) fill(colHL, 128);
            else fill(0, 0, 0, 64);//fill(ColBG,64);
          }

          rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);

          fill(colOut);
          textSize((bHeight/1.75f));
          text(labels[i], 5+xpos+((bWidth - textWidth(labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5f)+(bHeight*(i+1)), bWidth, bHeight);
        }   //end for 
        noStroke();
      } else
      {            
        //Menu upwards
        displayNormal(); //display the button part
        strokeWeight(1);
        stroke(colOut);  

        for (int i = 0; i != numStrs; i++)
        {
          fill(colBG);
          rect(5+xpos, ypos+(bHeight*(i-numStrs)), bWidth-5, bHeight); //draw background first
          if (((numStrs-1)-i) == selStr) fill(colHL);
          else fill(colBG);
          //drop down drawn

          if (DropDownMouseOverID > 0 && DropDownMouseOverID == (i+1) && MouseOverFlag == true)
          {
            if (((numStrs-1)-i) == selStr) fill(colHL, 128);
            else fill(0, 0, 0, 64);//fill(ColBG,64);
          }
          rect(5+xpos, ypos+(bHeight*(i-numStrs)), bWidth-5, bHeight);
        } //end for

        for (int i = 0; i != numStrs; i++)
        {
          fill(colOut);
          textSize((bHeight/1.75f));
          text(labels[i], 5+xpos+((bWidth - textWidth(labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5f)-(bHeight*(i+1)), bWidth, bHeight);
        }   //end for 
        noStroke();
      }
    }

    handleStatus();
    //   popStyle();
  } //end display()

  //--------------------------------------------------------------------------

  public boolean overOpen()
  {
    if (dirMode == false)  
    {    
      //Down Mode
      if (selected == false)
      {
        return false;
      } else
      {
        for (int i = 0; i != numStrs; i++)
        {
          if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos+(bHeight*(i+1)) && mouseYS <= ypos+bHeight+(bHeight*(i+1)) && status == 0)
          {    
            selected = false;
            selStr = i;  
            GlobalDDOpen = false;        
            return true;
          }
        }
        selected = false;
        return false;
      }
    } else
    {  
      if (selected == false)
      {
        return false;
      } else
      {
        for (int i = 0; i != numStrs; i++)
        {
          if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos-(bHeight*(i+1)) && mouseYS <= ypos+bHeight-(bHeight*(i-1)) && status == 0)
          {    
            selected = false;
            selStr = i;  
            GlobalDDOpen = false;        
            return true;
          }
        }
        selected = false;
        return false;
      }
    }
  } //end overNew()

  //--------------------------------------------------------------------------

  public boolean over() ///over the icon to open
  {
    if ( mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight  && status == 0) 
    {    
      if (GlobalDDOpen == true) return false; //if drop down already open, don't let it open another

      selectDD();
      return true;
    } else {
      return false;
    }
  }

  //--------------------------------------------------------------------------

  public boolean mouseOver() ///over the icon to open
  {
    if ( mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight  && status == 0)   
    {      
      MouseOverFlag = true;  
      return true;
    } else {
      MouseOverFlag = false;
      return false;
    }
  }

  //--------------------------------------------------------------------------

  public boolean mouseOverOpen()
  {
    if (dirMode == false)  
    {    
      //Down Mode
      if (selected == false)
      {
        return false;
      } else
      {
        for (int i = 0; i != numStrs; i++)
        {
          if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos+(bHeight*(i+1)) && mouseYS <= ypos+bHeight+(bHeight*(i+1)) && status == 0)
          {    
            MouseOverFlag = true;
            DropDownMouseOverID = i+1;        
            return true;
          }
        }
        DropDownMouseOverID = 0;
        MouseOverFlag = false;
        return false;
      }
    } else
    {  
      if (selected == false)
      {
        return false;
      } else
      {
        for (int i = 0; i != numStrs; i++)
        {
          if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos-(bHeight*(i+1)) && mouseYS <= ypos+bHeight-(bHeight*(i-1)) && status == 0)
          {    
            MouseOverFlag = true;
            DropDownMouseOverID = (numStrs-i);        
            return true;
          }
        }
        DropDownMouseOverID = 0;
        MouseOverFlag = false;
        return false;
      }
    }
  } //end overNew()
} //end dropdown class

//======================================================================================

class guiButton
{
  String Label;
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int bWidth;
  int bHeight;
  int colBG;
  int colHL;
  int colOut;  
  boolean toggle;
  boolean selected;
  int status; //0=show, 1=grey out, 2=hide

  boolean MouseOverFlag;

  //--------------------------------------------------------------------------

  guiButton(String iLabel, int ixpos, int iypos, int ibWidth, int ibHeight, int iColBG, int iColHL, int iColOut, boolean iToggle, boolean iSelected, boolean iAllowMouseOver)
  {
    Label = iLabel; 
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = ibHeight;
    colBG = iColBG;
    colHL = iColHL;
    colOut = iColOut;
    toggle = iToggle;
    selected = iSelected;
    status = 0; //init at 0

    //Setup mouse over list
    if (iAllowMouseOver == true) ButtonList.add(this);
    MouseOverFlag = false;
  }

  //--------------------------------------------------------------------------

  public void handleStatus()
  {
    noStroke();

    if (status == 1) 
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      rect(xpos, ypos, bWidth, bHeight);  
      noStroke();
    }
  }

  //--------------------------------------------------------------------------

  public void displayOutline()
  {
    strokeWeight(3);
    stroke(colOut);

    fill(255, 255, 255, 0); //transparent fill
    rect(xpos, ypos, bWidth, bHeight, bHeight/4);
  }

  //--------------------------------------------------------------------------

  public void display()
  {
    if (status == 2) return; //element is hidden, do not draw

    strokeWeight(1);
    stroke(colOut);

    if (selected==true) 
    {  
      fill(colHL);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colHL, 128);
      }
    } 
    else  
    {  
      fill(colBG);

      if (status == 0 && MouseOverFlag == true) 
      {
        fill(colBG, 64);
        //stroke(colHL);
      }
    }

    textAlign(LEFT);
    rect(xpos, ypos, bWidth, bHeight, bHeight/4);  
    //adjust stroke to on higlight and invert text color
    fill(colOut);
    textSize((bHeight/1.75f));
    text(Label, xpos+((bWidth - textWidth(Label))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5f), bWidth, bHeight);

    handleStatus();

    if (toggle==false) selected = false;
  }

  //--------------------------------------------------------------------------

  public boolean over() 
  {
    if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight && status == 0) 
    {      
      selected =! selected;  
      return true;
    } else {
      return false;
    }
  } //end over()

  //--------------------------------------------------------------------------

  public boolean mouseOver() //does not toggle selected
  {
    if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight && status == 0) 
    {      
      MouseOverFlag = true;  
      return true;
    } else {
      MouseOverFlag = false;
      return false;
    }
  } //end over()
} //end class

//======================================================================================

class guiSliderBar
{
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int slWidth; // single bar slWidthth
  int slHeight; // rect height
  int value; 
  int min; 
  int max;
  int bgColor;
  int slHandleColor;
  int slSelectColor; 
  int slStrokeColor;
  boolean circleHandle;  //ture circle, false rectangle
  boolean HorizVertToggle;
  boolean ScrollBarToggle;
  boolean GraphicDisplay;

  String callBackStr;

  int status;

  //gotta move the int variables into float variables
  float temp1=slWidth;
  float temp2=max;
  float temp3=value;
  float temp4=min;
  float MathVar=0;
  
  //--------------------------------------------------------------------------

  guiSliderBar(int ixp, int iyp, int iw, int ih, int iValue, int iMin, int iMax, int iBgColor, int islHandleColor, int islSelectColor, int islStrokeColor, boolean iCircleHandle, boolean iHorizVertToggle, boolean iScrollBarToggle, boolean iGraphicDisplay, String imethodStr) 
  {   
    xpos = ixp;
    ypos = iyp;  
    slWidth = iw;
    slHeight = ih;
    value=iValue;
    max=iMax;
    min=iMin;
    bgColor=iBgColor;
    slHandleColor=islHandleColor;
    slSelectColor=islSelectColor;
    slStrokeColor=islStrokeColor;
    circleHandle=iCircleHandle;      //false is rectangle, true is circle
    HorizVertToggle = iHorizVertToggle; //false is horiz, true is vert
    ScrollBarToggle= iScrollBarToggle; //false is normal, true is scrollbar style
    GraphicDisplay = iGraphicDisplay;
    callBackStr = imethodStr;

    status = 0;  //0=show, 1=grey out, 2=hide
  }
  
  //--------------------------------------------------------------------------

  public void setValue(int passedVal)
  {
    if (passedVal > max) { //check so it doesn't drag off the end
      passedVal = max;
    }
    if (passedVal < min) { //check so it doesn't drag off the end
      passedVal = min;
    }
    value = passedVal;
    method(callBackStr); //run callback to update any other variabes, since setValue over rides move
  }//end setValue() 

  //--------------------------------------------------------------------------

  public short getUnsignedChar() //no byte sized data type, use short(16-bit, signed)
  {
    //returns 8 bit unsigned number 0 - 255, based on float 
    return (short)(255*getFloat());
  }

  //--------------------------------------------------------------------------

  public float getFloat()
  {
    return ((float)value / (float)max);
  }
  
  //--------------------------------------------------------------------------

  public short getValue()
  {
    if (value > max) { //check so it doesn't drag off the end
      value = max;
    }
    if (value < min) { //check so it doesn't drag off the end
      value = min;
    }  

    return (short)value;
  }
  
  //--------------------------------------------------------------------------

  public void runCallBack()
  {
    move(mouseXS, mouseYS);
    try {
      method(callBackStr); //run function that is called everytime the slider is adjusted
    }
    catch(Exception e) { 
      println("Slider Method not found");
    }
  }

  //--------------------------------------------------------------------------

  public void move(int uX, int uY) 
  {
    //uX = round((float)uX / SF);
    //uY = round((float)uY / SF);    
    if (ScrollBarToggle == true)
    {
      //scrollbar mode
      if (HorizVertToggle == false) //horizontal
      {
        //this function handles position of the slider's handle for Verticle
        temp1 = slWidth - (slHeight * 2);   
        temp3 = value;

        MathVar = ((((uX-xpos)-slHeight) / temp1)-1) *-1; //creates a scale factor based on mouse position 
        MathVar = max - (MathVar*(max-min));   

        if (MathVar > max) { //check so it doesn't drag off the end
          MathVar=max;
        }
        if (MathVar < min) { //check so it doesn't drag off the end
          MathVar=min;
        }    

        value=PApplet.parseInt(MathVar); //set the value based on mouse movement
      }//end if
      else
      {
        //this function handles position of the slider's handle for Verticle
        temp1 = slHeight - (slWidth * 2);   
        temp3 = value;

        MathVar = ((((uY-ypos)-slWidth) / temp1)-1) *-1; //creates a scale factor based on mouse position  
        MathVar = max - (MathVar*(max-min));   

        if (MathVar > max) { //check so it doesn't drag off the end
          MathVar=max;
        }
        if (MathVar < min) { //check so it doesn't drag off the end
          MathVar=min;
        }  
        value=PApplet.parseInt(MathVar); //set the value based on mouse movement
      }
    } 
    else
    {
      //regular usage
      if (HorizVertToggle == false) //horizontal
      {
        //this function handles position of the slider's handle
        temp1 = slWidth;   
        temp3 = value;
        temp4 = min;

        MathVar = (((uX-xpos) / temp1)-1) *-1; //creates a scale factor based on mouse position 
        //relative to slider position
        MathVar = max - (MathVar*(max-min));   

        if (MathVar > max) { //check so it doesn't drag off the end
          MathVar=max;
        }

        if (MathVar < min) { //check so it doesn't drag off the end
          MathVar=min;
        }  

        value = PApplet.parseInt(MathVar); //set the value based on mouse movement
      }//end if
      else
      {
        println("NO SLIDER MOVE() HANDLER");
      }
    }
  } //end move()
  
  //--------------------------------------------------------------------------


  public void handleStatus()
  {
    noStroke();

    if (status == 1) 
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      rect(xpos, ypos, slWidth, slHeight);  
      noStroke();
    }
  }

  //--------------------------------------------------------------------------

  public void display() 
  {   
    if (status == 2) return; //element is hidden, do not draw

    if (GraphicDisplay == true)
    {
      if (HorizVertToggle == false) //horizontal
      { 
        temp1 = slWidth-(slWidth/20);
        temp2 = max;  //move variables into float
        temp3 = value;    //or do they don't compute correctly
        temp4 = min;
        MathVar = temp1 / (temp2 - temp4) * (temp3-temp4); 

        image(imgSliderBarHoriz, xpos-(slWidth/20), ypos, slWidth+(slWidth/10), slHeight);    
        image(imgMixerHandleHoriz, (xpos+MathVar), ypos-(slHeight/4), imgMixerHandleHoriz.width, slHeight*1.5f);  
        return; //if return is removed, the normal style of slider will overlay
      } 
      else
      {
        //HorizVertToggle = true, vertical
        temp1 = slHeight-(slHeight/20);
        temp2 = max;  //move variables into float
        temp3 = value;    //or do they don't compute correctly
        temp4 = min;
        MathVar = temp1 / (temp2 - temp4) * (temp3-temp4);         
        
         image(imgSliderBarVert, xpos, ypos-(slHeight/20), slWidth, slHeight+(slHeight/10)); 
         image(imgMixerHandleVert,xpos-(slWidth/4), (ypos+MathVar),  slWidth*1.5f, imgMixerHandleVert.height);  
         return;
      }
    } 

    if (ScrollBarToggle == true)//ScrollBarToggle places buttons at either end, if false it has no end space
    {
      if (HorizVertToggle == false) //horizontal
      { 
        stroke(slStrokeColor);
        strokeWeight(1);

        //circle handles are displayed differently so there are if/else statements to set it correctly
        fill(bgColor); //covers up the old slider with a blank rectangle, BgColor set in declaration
        rect(xpos, ypos, slWidth, slHeight);   //main rect

        fill(200); 
        rect(xpos+(slHeight/10)+slHeight, ypos+(slHeight/10), slWidth-(slHeight/5)-(slHeight*2), (slHeight-(slHeight/5)));   //middle    
        rect(xpos+(slHeight/10), ypos+(slHeight/10), slHeight-(slHeight/5), slHeight-(slHeight/5)); //left rect  
        rect(xpos+(slHeight/10)+(slWidth-slHeight), ypos+(slHeight/10), slHeight-(slHeight/5), slHeight-(slHeight/5));  //right rect 

        stroke(slStrokeColor);
        fill(slHandleColor);
        strokeWeight(1);
        //top
        triangle(xpos+slHeight-(slHeight/5), ypos+(slHeight/5), xpos+slHeight-(slHeight/5), ypos+(slHeight-(slHeight/5)), xpos+(slHeight/5), ypos+(slHeight/2));  
        //right
        triangle(xpos+slWidth-slHeight+(slHeight/5), ypos+(slHeight/5), xpos+slWidth-slHeight+(slHeight/5), ypos+(slHeight-(slHeight/5)), xpos+slWidth-slHeight+slHeight-(slHeight/5), ypos+(slHeight/2));
        //Handle

        temp1 = (slWidth - (slHeight*3));
        temp2 = max;  //move variables into float
        temp3 = value;    //or do they don't compute correctly
        temp4 = min;      
        float MathVar;

        MathVar = temp1 / (temp2 - temp4) * (temp3-temp4) + slHeight/2+slHeight;

        fill(slHandleColor); //places the slider handle
        ellipse(xpos+MathVar, ypos+(slHeight/2), slHeight-2, slHeight-2); //Circle instead of rectangle
      } 
      else
      {
        //HorizVertToggle = true
        stroke(slStrokeColor);
        strokeWeight(1);

        //circle handles are displayed differently so there are if/else statements to set it correctly
        fill(bgColor); //covers up the old slider with a blank rectangle, BgColor set in declaration
     //   rect(xpos, ypos, slWidth, slHeight);  

        fill(200); 
        rect(xpos+(slWidth/10), ypos+(slWidth/10)+slWidth, slWidth-(slWidth/5), (slHeight-(slWidth/5))-(slWidth*2));   
        rect(xpos+(slWidth/10), ypos+(slWidth/10), slWidth-(slWidth/5), slWidth-(slWidth/5));   
        rect(xpos+(slWidth/10), ypos+(slWidth/10)+slHeight-slWidth, slWidth-(slWidth/5), slWidth-(slWidth/5));   

        stroke(slStrokeColor);
        fill(slHandleColor);
        strokeWeight(1);
        //top
        triangle(xpos+(slWidth/5), ypos+(slWidth/4)+(slWidth/2), xpos+(slWidth/2), ypos+(slWidth/5), (xpos+slWidth) - (slWidth/5), ypos+(slWidth/4)+(slWidth/2));
        //bottom
        triangle(xpos+(slWidth/5), ypos+(slWidth/4)+slHeight-slWidth, xpos+(slWidth/2), ypos+slHeight-(slWidth/5), (xpos+slWidth) - (slWidth/5), ypos+(slWidth/4)+slHeight-slWidth);

        temp1 = (slHeight - (slWidth*3));
        temp2 = max;  //move variables into float
        temp3 = value;    //or do they don't compute correctly
        temp4 = min;

        //println(temp1+" : "+temp2+" : "+temp3+" : "+temp4);

        //Handle
        float MathVar;
        MathVar = temp1 / (temp2 - temp4) * (temp3-temp4) + slWidth/2+slWidth;
        fill(slHandleColor); //places the slider handle
        ellipse(xpos+(slWidth/2), ypos+MathVar, slWidth-2, slWidth-2); //Circle instead of rectangle

        // rect(xpos, ypos+MathVar, slWidth, slWidth); //and comment this line for circle
        noStroke();
      }
    } 
    else
    {
      //not a graphic or a scroll bar, draw a normal scroll bar with circle or square handle
      stroke(slStrokeColor);
      strokeWeight(1);

      //circle handles are displayed differently so there are if/else statements to set it correctly
      fill(bgColor); //covers up the old slider with a blank rectangle, BgColor set in declaration
      rect(xpos, ypos, slWidth, slHeight);   

      if (circleHandle==true) temp1 = slWidth - slHeight;
      else temp1=slWidth-(slWidth/20);

      temp2=max;  //move variables into float
      temp3=value;    //or do they don't compute correctly
      temp4 = min;

      float MathVar;

      if (circleHandle==true)
        MathVar = temp1 / (temp2 - temp4) * (temp3-temp4) + slHeight/2;
      else  
      MathVar = temp1 / (temp2 - temp4) * (temp3-temp4); 

      fill(color(slSelectColor)); //draws the Selcolor over the selected area
      rect(xpos, ypos, MathVar, slHeight);

      fill(slHandleColor); //places the slider handle
      if (circleHandle==true)   ellipse(xpos+MathVar, ypos+(slHeight/2), slHeight-1, slHeight-1); //Circle instead of rectangle
      else   rect((xpos+MathVar), ypos, slWidth/20, slHeight); //and comment this line for circle
    }
    
    handleStatus();
  }//end display()
  
  //--------------------------------------------------------------------------
  
  public void selectSlider()
  {
    SliderPointer = this;
    GlobalDragging = true;
  }//end selectSlider
  
  //--------------------------------------------------------------------------
  
  public boolean over() 
  {
    if (mouseXS >= xpos && mouseXS <= xpos+slWidth && mouseYS >= ypos && mouseYS <= ypos+slHeight && status == 0) 
    {  
      selectSlider();
      return true;
    } else {
      return false;
    }
  }
} //end class

//=====================================================================================

class guiTextField
{
  String label;
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int bWidth;
  int bHeight;
  int colBG;
  int colHL;

  int inputMethod; //0 = characters only, 1 = numbers only , 2 = numbers only with trailing %, 3 = floats
  int minValue; //either min number, or minim characters
  int maxValue; //either max number, or maximum characters
  boolean toggle;
  boolean selected;
  String callBack;

  int status; //0=show, 1=grey out, 2=hide

  boolean MouseOverFlag;

  guiTextField(String iLabel, int ixpos, int iypos, int ibWidth, int ibHeight, int iColBG, int iColHL, int iInputMethod, int iminValue, int imaxValue, boolean iToggle, boolean iSelected, String itxtfCallback)
  {
    label = iLabel; 
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = ibHeight;
    colBG = iColBG;
    colHL = iColHL;
    toggle = iToggle;
    selected = iSelected;
    inputMethod = iInputMethod;
    minValue = iminValue; 
    maxValue = imaxValue; 
    callBack = itxtfCallback;
    status = 0; //init at show

    //if(sEnableMouseOver == 1) 
    TextFieldList.add(this);
    MouseOverFlag = false;
  }

  public void handleStatus()
  {
    noStroke();

    if (status == 1) 
    {
      strokeWeight(2);
      stroke(gui.windowBackground, 200); //grey out button
      fill(gui.windowBackground, 200); //grey out button
      rect(xpos, ypos, bWidth, bHeight);  
      noStroke();
    }
  }  

  public void display()
  {
    if (status == 2) return; //element is hidden, do not draw

    //translate(0, 0);
    strokeWeight(2);
    fill(colBG);

    if (selected==true)
    {
      stroke(colHL);
    } else       
    {
      stroke(gui.buttonColor);
      if (status == 0 && MouseOverFlag == true) 
      {
        stroke(colHL, 128);
        fill(colBG, 192);
      }
    }
    rect(xpos, ypos, bWidth, bHeight);

    //adjust stroke to on higligh and invert text color
    noStroke();
    fill(0);
    textSize((bHeight/1.75f));

    textAlign(LEFT);
    if (inputMethod != 2) text(label, xpos+((bWidth - textWidth(label))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5f), bWidth, bHeight);
    else  text(label+"%", xpos+((bWidth - textWidth(label+1))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5f), bWidth, bHeight);

    handleStatus();

    if (toggle==false) selected = false;
  }


  public void selectField()
  {
    try {
      //textFieldPtr.selected = false; //was commented out
      deselectTextField(); //handle deselecting previous field if applicable
    }
    catch(Exception e) {
    }

    textFieldPtr = this;
    GlobalLabelStore = label;
    label = "";
    selected = true; 

    TextFieldActive = true;
  }


  public boolean over() 
  {
    if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight && status == 0) 
    {     
      selected =! selected;   
      
      if(selected == true) selectField(); 
      return true;
    } else {
      return false;
    }
  }

  public boolean mouseOver() //does not toggle selected
  {
    if (mouseXS >= xpos && mouseXS <= xpos+bWidth && mouseYS >= ypos && mouseYS <= ypos+bHeight && status == 0) 
    {     
      MouseOverFlag = true;    
      return true;
    } else {
      MouseOverFlag = false;
      return false;
    }
  } //end over()
} 

//============================================================================================

//===================================================================================================  

public void keyPressed()
{
  
 //------------------------------------------------------------------------------------------------

  if (key == ESC) 
  {
    key = 0; //have to alter it or if it gets to the end of the function(returns) with key=escape it closes the application

    deselectTextField(); 
    OverlayMenuID = 0; //close any menus

    return; //double up preventing escape for closing application
  }
  
  //------------------------------------------------------------------------------------------------

  if (NumberInputFieldActive == true)
  {

    if (key == ENTER || key == RETURN)
    {
      NumberInputFieldActive = false;
      method(numberInputFieldPtr.callBack);
      return;
    } //end enter/return if
    else if (key == BACKSPACE)
    {
      try {
        numberInputFieldPtr.label = numberInputFieldPtr.label.substring(0, numberInputFieldPtr.label.length()-1);
      }
      catch(Exception e) {
      }  
      return;
    }   


    if (numberInputFieldPtr.inputMethod == 0)
    {
      if (numberInputFieldPtr.label.length() > numberInputFieldPtr.maxValue) return;
      //Makes sure its a valid character otherwise it will be stored(but not seen) in .label as well
      if ((byte)key > 0 && (byte)key < 127) numberInputFieldPtr.label =  ""+numberInputFieldPtr.label + PApplet.parseChar(key);
      else println("KEY IGNORED"); //DEBUG
      return;
    } else
    {
      if (key=='1' || key== '2' || key=='3' || key=='4' || key=='5' || key=='6' || key=='7' || key=='8' || key=='9' || key=='0' || key=='-' || (key =='.' && numberInputFieldPtr.inputMethod == 3))
      {
        numberInputFieldPtr.label =  ""+numberInputFieldPtr.label + PApplet.parseChar(key);
        return;
      }
    }
    return;
  }  //end NumberInputFieldActive if()

  //------------------------------------------------------------------------------------------------


  if (TextFieldActive == true)
  {
    if (key == ENTER || key == RETURN)
    {
      TextFieldActive = false;
      method(textFieldPtr.callBack);
      return;
    } //end enter/return if
    else if (key == BACKSPACE)
    {
      try {
        textFieldPtr.label = textFieldPtr.label.substring(0, textFieldPtr.label.length()-1);
      }
      catch(Exception e) {
      }  
      return;
    }  


    if (textFieldPtr.inputMethod == 0)
    {
      if (textFieldPtr.label.length() > textFieldPtr.maxValue) return;
      //Makes sure its a valid character otherwise it will be stored(but not seen) in .label as well
      if ((byte)key > 0 && (byte)key < 127) textFieldPtr.label =  ""+textFieldPtr.label + PApplet.parseChar(key);
      else println("KEY IGNORED"); //DEBUG
      return;
    } else
    {
      if (key=='1' || key== '2' || key=='3' || key=='4' || key=='5' || key=='6' || key=='7' || key=='8' || key=='9' || key=='0' || key=='-' || (key =='.' && textFieldPtr.inputMethod == 3))
      {
        textFieldPtr.label =  ""+textFieldPtr.label + PApplet.parseChar(key);
        return;
      }
    }


    if (key == ENTER || key == RETURN)
    {
      TextFieldActive = false;
      method(textFieldPtr.callBack);
    } //end enter/return if
    return;
  }  //end NumberInputFieldActive if()


//------------------------------ Software Hot Keys ------------------------------------------------------------------

  if (key == '0') //forces a next sequence command
  {
    serialPort.write(0);
  }

  if (key == ' ') //SPACEBAR - toggles play/pause for the preview
  {
    //Spacebar is hotkey for play pause
    comPlayPauseButton.selected = !comPlayPauseButton.selected; //toggle
  }

  if (key == ',') //steps the preview frame back 1
  {
    if (PlayBackFrameNum > 0) PlayBackFrameNum--;
  }

  if (key == '.')  //steps the preview frame forward 1
  {
    if (PlayBackFrameNum < (sequence[SelectedSlot].totalFrames-1)) PlayBackFrameNum++;
  }


  if (key == '=') //forces a next sequence command
  {
    RequestNextSequenceCommand();
  }

  //--------------------------------------------------------------------------------------------------
  
  if (key == CODED) 
  {
    switch(keyCode)
    {
    case UP:
      if (DropDownPointer.selStr > 0) DropDownPointer.selStr--;
      break;

    case DOWN:
      if (DropDownPointer.selStr < (DropDownPointer.numStrs-1) ) DropDownPointer.selStr++;
      break;
    case RIGHT:

      break;

    case LEFT:

      break;
    }
  }
  //--------------------------------------------------------------------------------------------------
}//end keyPressed()

//======================================================================================================================

public void deselectTextField()
{
  if (TextFieldActive == true)
  {
    if (textFieldPtr.selected == true) 
    {
      textFieldPtr.selected = false; 
      textFieldPtr.label = GlobalLabelStore; //restore
    }
    TextFieldActive = false;
  }

  //---------------------------------------------------------

  if (NumberInputFieldActive == true)
  {
    if (numberInputFieldPtr.selected > 0) 
    {
      numberInputFieldPtr.selected = 0; 
      numberInputFieldPtr.label = GlobalLabelStore; //restore
    }

    NumberInputFieldActive = false;
  }
} //end mousePressedHandleTextFields

//======================================================================================================================

public void mousePressed() 
{
  mouseXS = round((float)mouseX / SF);
  mouseYS = round((float)mouseY / SF);

  if (mouseButton == RIGHT) 

  {
    println("X: "+mouseX+"   Y: "+mouseY);

    //---------------------------------------------------------

    //---------------------------------------------------------
  } //end right click
  else     
  {
    //regular left click
    //------------------------------------------------------------------------------------------------------------------

    if (ConfirmBoxIDNum > 0)
    {
      if (ConfirmButtonYes.over())
      {
        switch(ConfirmBoxIDNum)
        {
        case 0: //null
          break;
        case 1: //Upload Configurations      
          RequestConfigurationUpload(); //send configurations normally for all other devices.
          ShowNotification(4);
          break;
        case 2: //Remove Sequence from Index
          sequence[SelectedSlot].removeSequence();
          ShowNotification(9);
          break;        
        case 3: //Upload Indexed Sequences to Controller 
          StartSequenceUpload();
          break;
        } //end switch

        ConfirmBoxIDNum = 0; //close
      } //end confirm yes

      if (ConfirmButtonNo.over()) ConfirmBoxIDNum = 0; //close

      return;
    } //end confirm box if

    //------------------------------------------------------------------------------------------------------------------

    //If a drop down is open handle first, to prevent click-throughs to other elements
    if (GlobalDDOpen == true)   
    {
      //handle before anything
      if (DropDownPointer.overOpen())
      {
        println("Pressed Open DropDown "+DropDownPointer.selStr);
        method(DropDownPointer.callBack);
      } 
      else
      {
        //else a click was detected that wasn't on the open drop down, close it and leave
        DropDownPointer.selected = false;  
        GlobalDDOpen = false;
      }
      return; //whether it was a click on the open drop down, or somewhere not on the drop down, leave now, no other clicks should be detected
    }

    //------------------------------------------------------------------------------------------------------------------

    if (OverlayMenuID > 0)
    {
      if (OverlayMenus[OverlayMenuID].mousePressed()) return; //no need to check over()

      if (menuCloseButton.over()) OverlayMenuID = 0; //closes menu

      deselectTextField(); //handle deselecting previous field if applicable();

      if (OverlayMenus[OverlayMenuID].forceOverride == 0) return;
    }

    //------------------------------------------------------------------------------------------------------------------

    //only scans the buttons for the selected index slot
    try {
      //will catch if SelectedSlot with scrollbar offset is out of bounds, easy and quick. Fix later.
      if (SequenceSpeedField[SelectedSlot-indexScrollBar.getValue()].over()) return;

      if (SequenceRemoveButtons[SelectedSlot-indexScrollBar.getValue()].over())
      {
        if (IndexedSequences > 1)
        {
          //removed a sequence, move higher ones down into place 
          SetConfirmBox(2); //confirm remove sequence
        } 
        return;
      }

      if (SequenceLoadButtons[SelectedSlot-indexScrollBar.getValue()].over()) 
      {
        FileBrowserAction = 3;
        selectInput("Select a FilePlay data file for slot#"+(SelectedSlot+1), "fileSelected");  
        return;
      }

      if (SequencePlayModeDD[SelectedSlot-indexScrollBar.getValue()].over()) return;
      if (SequenceTransitionDD[SelectedSlot-indexScrollBar.getValue()].over()) return;

    }
    catch(Exception e) {
    }

    for (int i = 0; i < cMaxIndexViewed+1; i++)
    {
      if (mouseXS >= cSeqModuleXOffset && mouseXS <= cSeqModuleXOffset+cSeqModuleWidth && mouseYS >= (cSeqModuleYOffset+(i*cSeqModuleHeight)) && mouseYS <= (cSeqModuleYOffset+(i*cSeqModuleHeight))+cSeqModuleHeight)
      {
        int temp = i+indexScrollBar.getValue();

        println("Clicked sequence module "+sequence[temp].idNum+"   with "+i);
        //unselect all others first. For both selecting and adding sequences
        for (int x = 0; x < sequence.length; x++) sequence[x].selected = false;

        if (temp >= IndexedSequences)
        {
          //Its the add sequence button 
          println("Add Sequence To Index");

          if (sequence[temp-1].fileName.equals(""))
          {
            println("Previous sequence not defined");
            ShowNotification(8);
            return;
          }
          ShowNotification(10);
          IndexedSequences++; //increase index counter
          UpdateIndexMemoryUsage();
          return;
        }

        //select current sequence, and reset variables
        sequence[temp].selected = true;
        SelectedSlot = temp;
        comPlayPauseButton.selected = false;
        PlayBackFrameNum = 0;
        UpdateIndexGUIElements();
        return;
      } //end mouse locaiton if
    } //end for()

    //------------------------------------------------------------------------------------------------------------------

    if (comPlayPauseButton.over()) return; //only toggles state to indicate play or stop

      if (comGlobalIntensityField.over()) return;

    if (comLoadPatchFile.over()) 
    {
      FileBrowserAction = 4;
      selectInput("Select a FilePlay data file:", "fileSelected");  
      return;
    }


    if (comSaveSequenceFile.over())
    {
      FileBrowserAction = 2;
      selectOutput("Select a file to save the sequences to:", "fileSelected");  
      return;
    }

    if (comLoadSequenceFile.over())
    {
      FileBrowserAction = 1;
      selectInput("Select a Aurora Matrix sequence file to load(.auroramatrix)", "fileSelected");  
      return;
    }

    if (comUploadSequences.over())
    {
      SetConfirmBox(3); //confirm remove sequence
      println("Attempting Sequence Upload");
      return;
    }

    if (indexScrollBar.over()) return;

    if (comSerialPortDD.over()) return;

    if (comSerialBaudRateDD.over()) return;

    if(comHardwareColorReOrderDD.over()) return;

    if (comControlCommands.over())
    {
      println("Not Yet added");
      return;
    }

    if (UploadProgressCancel.over() && UploadInProgress == true)
    {
      UploadInProgress = false;
      return;
    }

    if (comConnectButton.over()) 
    {
      //setup baud rate based on user selections

      if (comSerialBaudRateDD.selStr == 0) ProgramBaudRate = 0;//USB/None selected, use any baud rate, doesn't matter
      else 
      {
        //A baud rate is selected
        ProgramBaudRate = PApplet.parseInt(comSerialBaudRateDD.labels[comSerialBaudRateDD.selStr]);
      }

      if (devConnected == false) OpenCOMPort(serialPort.list()[comSerialPortDD.selStr]); 
      else CloseCOMPort();
      return;
    }

    if (comOpenConfigurationsMenu.over())
    {
      SetOverlayMenuID(1); //Configurations Overlay
    }
  } //end left click else

  deselectTextField(); //no click on a gui object,  if a text field is selected, de-select
} //end mousePressed

//======================================================================================================================

public void mouseReleased()
{
  //Update scaled mouse everytime before the values are used
  mouseXS = round((float)mouseX / SF);
  mouseYS = round((float)mouseY / SF);

  GlobalDragging = false; //incase a slider is being dragged
} //end mouseReleased

//======================================================================================================================

public void mouseDragged()
{
  //The mouseDragged() function is called once every time the mouse moves while a mouse button is pressed. (If a button is not being pressed, mouseMoved() is called instead.) 
  //println("MouseDragging");
  //if(GlobalDragging == true) redraw();
}

//======================================================================================================================

public void mouseMoved()
{
  mouseXS = round((float)mouseX / SF);
  mouseYS = round((float)mouseY / SF);

  //---------------------------------------------------------

  if (software.mouseOverEnabled == true)
  {
    //Mouse Over Functions
    // Fairly ineffiecent, it will detect any object, even those not shown, so it will always redraw on mouse moved
    for (int i = 0; i != DropDownList.size(); i++) 
    { 
      if (DropDownList.get(i).mouseOver()) return; //no other way to highlight, have to redraw
      if (DropDownList.get(i).mouseOverOpen()) return; //no other way to highlight, have to redraw
    } //end for

    //---------------------------------------------------------

    if (GlobalDDOpen == true)   return; //if a drop down is open, don't bother mousing over any other elements, just leave

    //---------------------------------------------------------

    for (int i = 0; i != ButtonList.size(); i++) 
    { 
      // An ArrayList doesn't know what it is storing so we have to cast the object coming out
      PointerButton = ButtonList.get(i);  
      if (PointerButton.mouseOver()) return;
    } //end for

    //---------------------------------------------------------
    
   for (int i = 0; i != NumberInputList.size(); i++) 
    { 
      // An ArrayList doesn't know what it is storing so we have to cast the object coming out
     // numberInputFieldPtr = NumberInputList.get(i);  
      if (NumberInputList.get(i).mouseOver() > 0) return;
    } //end for   
    
    
        //---------------------------------------------------------
  } //end sEnableMouseOver if()
} //end mouseMoved

//======================================================================================================================

public void mouseWheel(MouseEvent event) 
{
  println("mouseWheel()");
  mouseXS = round((float)mouseX / SF);
  mouseYS = round((float)mouseY / SF);

  float e = event.getCount();
  if (mouseXS > 0 && mouseXS < 274 && mouseYS > 165)
  {
    indexScrollBar.setValue(PApplet.parseInt(indexScrollBar.value+e));
  }
}

//======================================================================================================================

//============================================================================================

public class DeviceObj
{
  int HardwareID; //unique hardware ID number
  String Name;  //Stores the readable name for the controller as string
  int HardwareVersion;
  int FirmwareVersion;
  int FirmwareRevision;
  int BootloaderVersion;

  //Sequence defines
  int Channels; //maximum amount of channels
  int MaxSpeed;  
  int MaxIndexedSequences;
  int IndexMemoryModel;
  int MaxFrames;
  int MaxFramesGlobal;

  //Memory defines
  int DataSpace;
  int HWPVSpace;
  int HPVBlockSize;
  int SeqMemoryBlockSize; 
  int IndexBlockSize; 
  int CurrentBlockSize;
  int EraseBlockSize;

  boolean LockChannelAmt; //disallow per sequence data size, only device.channels
  int GammaCorrection;
  boolean LinkedSupport;  
  boolean Bit16Mode;  //flag to set 16-bit data mode
  boolean FillUpperData;
  boolean Bit16Processor;
  boolean Bit16LiveMode;

  int BasicVersion;
  int BasicMaxSequences; 
  int BasicSeqLength;
  int AccelerometerMode;

  int ArrayCastSize; //can't always use device channels, because of double duty for linked variables

  //How are these loaded differently???????????????  
  int DMXModeAmt;
  String[] DMXModeLabels;
  int ListedChipsets;

  //Not loaded from Device Files
  int UserConfiguredIDNum; //set by user saved in firwmare  
  int UserIdleSequence;  //Held is save file, sent with index

  //scratch
  String ConfigFlagsStr;
  String ConfigBytesStr;

  String WebpageURL;
  String MemorySpaceString; //stores byte to kb/mb conversion string

  DeviceObj(String iName)
  {
    HardwareID = 0;
    Name = iName;
    HardwareVersion = 0;
    FirmwareVersion = 0;
    FirmwareRevision = 0;  
    Channels = 0;
    DataSpace = 0;
    BootloaderVersion = 0;
    UserConfiguredIDNum = 0;
    UserIdleSequence = 0;

    MaxFrames = 0;
    MaxFramesGlobal = 0;

    MaxIndexedSequences = 8; //default, will get set when device file is loaded

    EraseBlockSize = 0;
    HWPVSpace = 0;
    DMXModeAmt = 0;
    GammaCorrection = 0;
    LinkedSupport = false;
    Bit16Mode = false;
    DMXModeLabels = new String[1];
  }
} //end object

//============================================================================

public class SoftwareObj
{
  int frameRate;

  int GUIWidth;
  int GUIHeight;

  boolean mouseOverEnabled;

  //--------------------------------------------------------------------------

  SoftwareObj()
  {
    mouseOverEnabled = true;
  }
  //--------------------------------------------------------------------------
} //end object

//=================================================================================

public class GUIObj
{
  int windowBackground;
  int windowStroke;
  int layerBackground;
  int textColor;
  int textMenuColor;

  int buttonColor;
  int buttonHighlightColor;
  int menuBackground;

  int textFieldHighlight;
  int textFieldBG;


  //--------------------------------------------------------------------------

  GUIObj()
  {
    windowBackground = color(65, 65, 65);
    windowStroke  = color(255);
    layerBackground = color(100, 100, 100);
    textColor = color(255);
    buttonColor = color(0, 0, 100);
    buttonHighlightColor = color(100, 0, 100);

    menuBackground  = color(200);
    textMenuColor = color(0);

    textFieldBG = color(255); //white
    textFieldHighlight = color(200, 40, 40);
  }
} //end gui class

//======================================================================================

public class MatrixObj
{
  int width;
  int height;
  String patchFileName; //actual file name and location
  String patchFileShort; //for GUI display

  int patchedPixels;  //base 1
  int patchedChannels; //base 1

  //---------------------------------------------------------------------------------------

  MatrixObj()
  {
    patchFileName = "";
  }

  //--------------------------------------------------------------------------

  public void setPatchFile(String passedStr)
  {
    patchFileName = passedStr;

    if (patchFileName.length() > 35) patchFileShort = "..."+patchFileName.substring(patchFileName.length()-35, patchFileName.length());
    else patchFileShort = passedStr;

    loadPatchFile();
  }
  //--------------------------------------------------------------------------

  private void loadPatchFile()
  {
    //loads a coordinate patch file, find min and mix point in both directions and scales the overall size to fit within the preview areas
    // creates the Pixel objects to display during draw and fills the Coordinate Arrays for Pixel Patching
    println("BuildCustom()");

    int Xdifference = 0;
    int Ydifference = 0;

    String[] lines = loadStrings(matrix.patchFileName); //divides the lines
    String[] WorkString = new String[3]; //used to divide the lines into tab

    WorkString = split(lines[0], '\t');
    matrix.patchedPixels = PApplet.parseInt(WorkString[0]);

    //---------------------------------------------------------------------------------------------------------

    patchedChannels = matrix.patchedPixels * 3;

    println("PatchedChannels: "+matrix.patchedChannels);
    println("TotalPixels: "+matrix.patchedPixels); 

    int MinX = 10000; //set to a large value
    int MinY = 10000;
    int MaxX = 0;
    int MaxY = 0;

    for (int i=1; i != lines.length; i++)
    {
      WorkString = split(lines[i], '\t');
      if (PApplet.parseInt(WorkString[0]) > MaxX) MaxX = PApplet.parseInt(WorkString[0]);
      if (PApplet.parseInt(WorkString[0]) < MinX) MinX = PApplet.parseInt(WorkString[0]);

      if (PApplet.parseInt(WorkString[1]) > MaxY) MaxY = PApplet.parseInt(WorkString[1]);
      if (PApplet.parseInt(WorkString[1]) < MinY) MinY = PApplet.parseInt(WorkString[1]);
    }
    //println("X: "+MinX+" : "+MaxX);
    //println("Y: "+MinY+" : "+MaxY);

    Xdifference = MaxX - MinX;
    Ydifference = MaxY - MinY;
    //println(Xdifference+" : "+Ydifference);

    PatchCoordX = new short[patchedPixels]; //resize the patch arrays
    PatchCoordY = new short[patchedPixels];

    //file created was incremented method channel numbers
    for (int i=0; i != matrix.patchedPixels; i++)
    {
      WorkString = split(lines[i+1], '\t');      
      PatchCoordX[i] = (short)(PApplet.parseInt(WorkString[0]) - MinX);
      PatchCoordY[i] = (short)(PApplet.parseInt(WorkString[1]) - MinY);
    }

    //set final matrix size
    matrix.width = Xdifference+1;//not base 0
    matrix.height = Ydifference+1;// not base 0
    //println("maxX: "+MaxX+"   maxY: "+MaxY);

    //set pixSize that draw() uses to draw the matrix preview
    if (MaxX > MaxY)
    {
      displayPixSize = ((cPreviewContentWidth-20) / MaxX);
    } else
    {
      displayPixSize = ((cPreviewContentHeight-20)  / MaxY);
    }
  }//end method
} //end object

//============================================================================

public class SequenceObj
{
  int idNum;
  String fileName;
  int totalFrames; //stored as base 1
  int speed;
  float memorySize; //in bytes

  int playMode;
  int transition;

  //GUI variables
  boolean selected;

  //Color Data
  int[][] dataFrames; //Data Storage array

  //Non-Saved Function Only - For Index Building
  int mappedROMAdr;

  //--------------------------------------------------------------------------

  SequenceObj(int pidNum)
  {
    idNum = pidNum;
    fileName = "";
    totalFrames = 0;
    speed = cDefaultSeqSpeed;
  }

  //--------------------------------------------------------------------------

  public void display(int yOffset)
  {
    if (idNum >= IndexedSequences)
    {
      //sequence not defined, it should show the add sequence button
      stroke(0, 0, 255); //blue
      fill(100);
      rect(cSeqModuleXOffset, cSeqModuleYOffset+((yOffset)*cSeqModuleHeight)+10, cSeqModuleWidth-10, cSeqModuleHeight);
      fill(255); //white
      textAlign(CENTER);
      text("Click to\nAdd Sequence to slot "+(idNum+1)+":", cSeqModuleXOffset+(cSeqModuleWidth/2), cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+60);
      textAlign(LEFT);
    } 
    else
    {
      //display normally
      fill(255); //white
      text("Slot "+(idNum+1)+":", cSeqModuleXOffset, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+20);

      if (fileName.length() > 28) text("File: "+"..."+fileName.substring(fileName.length()-28, fileName.length()), cSeqModuleXOffset, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+55);
      else text("File: "+fileName, cSeqModuleXOffset, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+55);

      text("Frames: "+totalFrames, cSeqModuleXOffset, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+75);
      text("Memory: "+((float)memorySize)+"Kb", cSeqModuleXOffset+105, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+75); 
      text("Speed(mSec): ", cSeqModuleXOffset, cSeqModuleYOffset+(yOffset*cSeqModuleHeight)+105);

      strokeWeight(2);
      stroke(gui.windowStroke);
      line(cSeqModuleXOffset, cSeqModuleYOffset+((yOffset+1)*cSeqModuleHeight), 274, cSeqModuleYOffset+((yOffset+1)*cSeqModuleHeight));
    }

    if (selected == true)
    {
      stroke(255, 0, 0); //red
      noFill();
      rect(cSeqModuleXOffset-5, cSeqModuleYOffset+((yOffset)*cSeqModuleHeight), cSeqModuleWidth, cSeqModuleHeight);
    }
  }

  //--------------------------------------------------------------------------

  public void loadSequence(String passedFilePath)
  {
    //Speed is not available, manually
    speed = cDefaultSeqSpeed; //default 30 FPS
    fileName = passedFilePath;

    //SelectedSlot = idNum;
    LoadFilePlayModeFile(passedFilePath, idNum);
    //totalFrames is updated in file load
    memorySize = (totalFrames*matrix.patchedChannels)+cHeaderByteLen; //answer in bytes
    UpdateIndexMemoryUsage();
  }

  //--------------------------------------------------------------------------

  public void removeSequence()
  {
    //have to move all other ones down and adjust ID number 
    for (int i = SelectedSlot; i < IndexedSequences; i++) 
    {
      try 
      {
        //move down the sequences
        //sequence[i] = sequence[i+1]; //don't think it works like that
        sequence[i].idNum = sequence[i+1].idNum-1;
        sequence[i].fileName = sequence[i+1].fileName;
        sequence[i].totalFrames = sequence[i+1].totalFrames;
        sequence[i].speed = sequence[i+1].speed;
        sequence[i].memorySize = sequence[i+1].memorySize;
        //mappedROMAdr only updated at upload
        sequence[i].selected  = false;   
        //sequence[i].dataFrames = sequence[i+1].dataFrames; 

        for (int x= 0; x < sequence[i].dataFrames.length; x++)
          for (int j=0; j < sequence[i].dataFrames[x].length; j++)
            sequence[i].dataFrames[x][j]= sequence[i+1].dataFrames[x][j];
      }
      catch(Exception e) {
        println("Exception attempting to shift sequence objects down");
      }
    } //end q for() 

    //update variabels and elements
    IndexedSequences--; 

    UpdateIndexMemoryUsage();
    
    sequence[SelectedSlot].selected  = false;
    SelectedSlot = 0;
    sequence[SelectedSlot].selected  = true;
  } //end removeSequence()

  //--------------------------------------------------------------------------

  public String getSaveString()
  {
    String[] qpath = splitTokens(fileName, System.getProperty("file.separator"));

    return  qpath[qpath.length-1]+"\t"+speed+"\t"+playMode+"\t"+transition;
  }

  //--------------------------------------------------------------------------

  public void setLoadString(String passedStr)
  {
    String[] WorkString = new String[5]; //more than required, not sure how many are listed yet

    WorkString = split(passedStr, '\t');

    fileName = WorkString[0];
    loadSequence(FileZipper.localTempFolder+fileName);
    speed = PApplet.parseInt(WorkString[1]);   
    playMode = PApplet.parseInt(WorkString[2]);   
    transition = PApplet.parseInt(WorkString[3]);
  }

} //end class


//========================================================================================================

//Sample code taken from the Processing forums and probably another website. Credit is owed to someone.
public class zipper
{
  String localTempFolder =""; //static for now

  //----------------------------------------------------------------------------------------------------

  public void clearTempFolder()
  {
    try {
      File fp = new File(localTempFolder); 
      String[] fileNames = fp.list(); //can't use .list() directly since it updates as items are deleted
      printArray(fileNames);

      for (int i = 0; i <= fileNames.length+1; i++)
      {
        File x = new File(localTempFolder+fileNames[i]); 
        x.delete(); // It is supposed to be still there...
        println("Deleted "+i+"  "+x);
      }
    }
    catch(Exception e) { 
      println("Error clearing temp folder");
    }
  } //end method

  //----------------------------------------------------------------------------------------------------

  public void unzipToTemp(String passedZipLoc)
  {
    println("Starting unzipToTemp("+passedZipLoc+")");

    byte[] buff = new byte[cBufferByteSize];

    try {
      FileInputStream fis = new FileInputStream(passedZipLoc);

      // this is where you start, with an InputStream containing the bytes from the zip file
      ZipInputStream zis = new ZipInputStream(fis);
      ZipEntry entry;

      // while there are entries I process them
      while ((entry = zis.getNextEntry()) != null)
      {
        System.out.println("entry: " + entry.getName() + ", " + entry.getSize());
        // consume all the data from this entry
        while (zis.available() > 0)
        try {
          FileOutputStream fos = new FileOutputStream(localTempFolder+entry.getName());
          int l=0;
          // write buffer to file
          while ((l=zis.read(buff))>0) {
            fos.write(buff, 0, l);
          }
          fos.close(); //crucial for it to work or won't release the files so they can not be deleted
        } 
        catch (Exception e) {
          println("Exception writing files to disk");
          break;
        }
      }
      zis.close();
      fis.close();
    }
    catch (Exception e) {
      println("Exception caught within Unzip Class");
    }
  } //end method

  //----------------------------------------------------------------------------------------------------

  public void zipFiles(String[] srcFiles, String zipDirStr) 
  {
    println("Saving "+zipDirStr);
    printArray(srcFiles);

    try {
      // create byte buffer
      byte[] buffer = new byte[1024];

      FileOutputStream fos = new FileOutputStream(zipDirStr);
      ZipOutputStream zos = new ZipOutputStream(fos);

      for (int i=0; i < srcFiles.length; i++) 
      {
        //println("Running with "+srcFiles[i]);
        File srcFile = new File(srcFiles[i]);

        FileInputStream fis = new FileInputStream(srcFile);

        // begin writing a new ZIP entry, positions the stream to the start of the entry data
        zos.putNextEntry(new ZipEntry(srcFile.getName()));

        int length;

        while ((length = fis.read(buffer)) > 0) {
          zos.write(buffer, 0, length);
        }

        zos.closeEntry();
        fis.close();           // close the InputStream
      }

      zos.close();         // close the ZipOutputStream
    }
    catch (IOException ioe) {
      System.out.println("Error creating zip file: " + ioe);
    }
  }//end method
} //end class

//========================================================================================================

//=======================================================================================================================  

class guiOverlayMenu
{
  int ObjID;
  int xpos;
  int ypos;
  int Height;
  int Width;

  int autoPosition; //0 = use xpos & ypos, 1 = center X & Y, 2 = center Y alight right, 3 = center Y align left
  int forceOverride; //0 = only menu elements can be clicked, 1 = any exposed elements can be clicked

  int xOffset, yOffset;

  //constructor
  guiOverlayMenu(int iObjID, int ixpos, int iypos, int ibWidth, int ibHeight)
  {
    ObjID = iObjID;
    xpos = ixpos;
    ypos = iypos;  
    Width = ibWidth;
    Height = ibHeight;

    autoPosition = 1; //set as default
    forceOverride = 0;
  } //end constructor

  //-----------------------------------------------------------------------------------------------

  public void display()
  {
    pushStyle();
    fill(gui.menuBackground);
    stroke(0);
    strokeWeight(4);

    //draws background of menu
    switch(autoPosition) 
    {
    case 0:
    default:
      rect(xpos, ypos, Width, Height, 20);
      break;
    case 1:
      rect(xOffset, yOffset, Width, Height, 20); //background
      textSize(24);
      fill(0);
      textAlign(LEFT);
      text("Configuration Menu:", xOffset+20, yOffset+30);
      textSize(14);
      text("Use the configuration modules to select your hardware options. Once selected, upload them to the controller.", xOffset+20, yOffset+60); 
      
      menuUploadConfigurations.display();
      menuRequestConfigurations.display();
      break;
    }//end switch

    //========================================

    switch(ObjID)
    {
    case 1: //Configurations Menu 
    //scan through all enabled configuration modules, and display them
     for (int i =0; i != InputModule.length; i++) InputModule[i].display();
      break;  
    case 2:  //Stand-Alone commands menu
    
      break;    
    } //end ObjID switch

    menuCloseButton.display();

    popStyle();
  } //end display()

  //-----------------------------------------------------------------------------------------------

  public void initMenu()
  {
    //----------------------------------------------------
    
    switch(autoPosition) 
    {
    case 0:
    default:
      xOffset = xpos;
      yOffset = ypos;
      break;
    case 1: //= center X&Y
      xOffset = (1024-Width)/2;
      yOffset = (768-Height)/2;
      break;
    }//end switch   
    
    //----------------------------------------------------

    switch(ObjID)
    {
    case 1: //Configurations Menu 
    menuUploadConfigurations.xpos = xOffset+300;
    menuUploadConfigurations.ypos  = yOffset+10; 
    menuRequestConfigurations.xpos = xOffset+500;
    menuRequestConfigurations.ypos = yOffset+10; 
      break;  
    case 2:  //Stand-alone commands menu
    
    break;    
    } //end ObjID switch
    
    //----------------------------------------------------
    
    menuCloseButton.xpos = xOffset+(Width-100);
    menuCloseButton.ypos = yOffset+20;
    
  //----------------------------------------------------  
  } //end display()

  //-----------------------------------------------------------------------------------------------

  public boolean over() 
  {
    if (mouseXS >= xpos && mouseXS <= xpos+Width && mouseYS >= ypos && mouseYS <= ypos+Height) 
    {      
      return true;
    } else {
      return false;
    }
  } //end over()

  //-----------------------------------------------------------------------------------------------

  public boolean overEllipse() 
  {
    if (mouseXS >= xpos-(Width/2) && mouseXS <= xpos+(Width/2) && mouseYS >= ypos-(Height/2) && mouseYS <= ypos+(Height/2)) 
    {      
      return true;
    } 
    else {
      return false;
    }
  } //end over()  

  //-----------------------------------------------------------------------------------------------

  public boolean mousePressed()
  {

    switch(ObjID)
    {
    case 1: //Configurations Menu    
     if (CheckModuleElements() == true) return true; 
     if(menuUploadConfigurations.over()) { SetConfirmBox(1); return true; }
     if(menuRequestConfigurations.over()) { println("NO FUNCTION"); return true; }
      break;  
    case 2: //Stand-alone commands menu
    
    break;    
    } //end switch
    return false;
  } //end mouse pressed
} //end overlayMenu class

//================================================================================================

final int mStall = 0;
final int mFade = 1;
final int mGradient = 2;
final int mInstant = 3;
final int mTech = 4;
final int mPOV = 5;
final int mFilePlay = 6; //only sequence mode supported
final int mLinked = 7;

//================================================================================================

public void BuildUSBPacket(int dataValue)
{
  //THIS DO ANYTHING? ALL DEVICES ERASE AT FIRST PACKET....  
  SendCounter++; //used for 
  SendCounterB++; //counts all bytes
  if (SendCounter >= device.EraseBlockSize)
  {
    delay(5); //delay to allow the device to erase the next block
    SendCounter = 0;
  }
  //end this  

  USBPacket[PacketPointer] =  PApplet.parseByte(dataValue);
  PacketPointer++;

  if (PacketPointer == 64) //always sends in blocks of 64, that is built into 64/128/192/256 byte sized packets
  {  
    //packet full, send it  
    PacketPointer = 0;
    try {    

      if (CommunicationMode == 0) serialPort.write(USBPacket); //send a full 64 byte packet all at once
      else if (CommunicationMode == 1) TCPClient.write(USBPacket);    

      USBpacketCount++;

      //this runs the routine for waiting for "zAck"
      if (SendCounter == device.CurrentBlockSize)
      {
        println("Waiting for ACK, USBPacket: "+USBpacketCount+"    SendCounter: "+SendCounter);
        SendCounter = 0;
        while (WaitForAckFlag == false) 
        {
          delay(1); //runs infinitly til ack flag is set, needs delay to not error, but not locking
          if (TerminateUpload == true) { 
            break;
          } //once set will force the function to finish and thread to end
        }
        WaitForAckFlag = false; //set back to false
        redraw();
      }
    }
    catch(Exception e)
    {
      println("Unable to Send Packet");  
      redraw();
    }
  }  //end if()
}  //end BuildUSBPacket()

//===================================================================================================

public void BreakValueForUSBPacket(int passedValue, int passedType)
{
  //breaks address value into properly sized bytes, either 16-bit or 32-bit  
  //println("Passed:" +passedValue);
  switch(passedType)
  {
  case 0: //16-bit 
    BuildUSBPacket((passedValue>>8)); //MSB  
    BuildUSBPacket(passedValue); //LSB, correct order?
    break;

  case 1: //32-bit
    BuildUSBPacket((passedValue>>24));
    //println((passedValue>>24));    
    BuildUSBPacket((passedValue>>16)); //MSB   
    //println((passedValue>>16)); //MSB   
    BuildUSBPacket((passedValue>>8)); //MSB      
    //println((passedValue>>8)); //MSB      
    BuildUSBPacket(passedValue); 
    //println(passedValue& 0xFF); 
    break;
  } //end switch
} //end BreakUSBValue()  

//=====================================================================================================

public void BreakUSBValue(int passedValue)
{
  //breaks address value into properly sized bytes, either 16-bit or 32-bit    
  switch(device.IndexMemoryModel)
  {
  case 0: //standard 16-bit     
    BuildUSBPacket(passedValue); //LSB, correct order?
    BuildUSBPacket((passedValue>>8)); //MSB  
    if (device.FillUpperData == true)  BuildUSBPacket(0); //fill upper with 0   
    break;

  case 1: //32-bit  
    BuildUSBPacket(passedValue); 
    BuildUSBPacket((passedValue>>8)); //MSB  
    if (device.FillUpperData == true)  BuildUSBPacket(0); //fill upper with 0     
    BuildUSBPacket((passedValue>>16)); //MSB 
    BuildUSBPacket((passedValue>>24));  
    if (device.FillUpperData == true)  BuildUSBPacket(0); //fill upper with 0     
    break;
  } //end switch
} //end BreakUSBValue()  

//================================================================================================

public void StartSequenceUpload()
{
  println("StartSequenceUpload()    IndexedMemoryUsage: "+IndexedMemoryUsage);
  ShowNotification(11);

  UpdateIndexMemoryUsage();
  if (IndexedMemoryUsage > device.DataSpace) return; //SHOULD NEVER GET HERE BUT LEAVE IF IT DOES

  int adrCount = 0;
  int i = 0; //declare it here so the result can be used outside 

  sequence[0].mappedROMAdr = adrCount; 

  for (i = 1; i < IndexedSequences; i++) //was not equals to
  {
    adrCount += sequence[i-1].memorySize;
    sequence[i].mappedROMAdr = adrCount; //store for the sequence header
  } //end for()

  println("Total Sequences Length in Bytes: "+IndexedMemoryUsage);

  //--------------------------------------------------------------------------------------------------------------

  int tempExpectedPacketsIndex = 0;
  //IndexMemoryModel: 0 = 16-bit, 1 = 32-bit
  if (device.IndexMemoryModel == 0)
  {
    tempExpectedPacketsIndex +=  device.MaxIndexedSequences * 2;//length of Index Map
    if (device.FillUpperData == true) tempExpectedPacketsIndex +=  device.MaxIndexedSequences;
  } else if (device.IndexMemoryModel == 1)
  {
    tempExpectedPacketsIndex +=  device.MaxIndexedSequences * 4;//length of Index Map
    if (device.FillUpperData == true) tempExpectedPacketsIndex +=  device.MaxIndexedSequences * 2;
  }

  tempExpectedPacketsIndex =  ceil((float)tempExpectedPacketsIndex/device.IndexBlockSize);    
  int tempExpectedPacketsSequence = ceil((float)IndexedMemoryUsage/device.SeqMemoryBlockSize); 

  println("Index Space Length, in packets: "+tempExpectedPacketsIndex);
  int totalExpectedPackets = tempExpectedPacketsIndex + tempExpectedPacketsSequence;

  device.CurrentBlockSize = device.IndexBlockSize;  //start with IndexBlockSize since that is first

  println("Index Pkts("+device.IndexBlockSize+"): "+tempExpectedPacketsIndex+"     Seq Pkts("+device.SeqMemoryBlockSize+"): "+tempExpectedPacketsSequence);
  println("Total Pkts(variable size): "+totalExpectedPackets);

  println("RequestFullUpload()   Sequence Bytes: "+(IndexedMemoryUsage)+" : IndexedSequences: "+IndexedSequences+"    CurrentBlockSize: "+device.CurrentBlockSize);
  cmdByte = cmdFullSeqUpload; //Upload Sequences
  cmdData[0] = ((totalExpectedPackets) >> 8) & 0xFF; //MSB
  cmdData[1] = (totalExpectedPackets) & 0xFF; //LSB
  cmdData[2] = IndexedSequences-1;  //Max Sequences
  cmdData[3] = 0;   //send idle sequence ID number
  RequestCommand();
} //end func

//===================================================================================================

public void SendSequence(int passedSeqID)
{
  println("SendSequence() "+(passedSeqID)+"    Start SendCounterB="+SendCounterB+"    Total Bytes to Send: "+(sequence[passedSeqID].memorySize));

  //Set Default Message
  DisplayMessageStr = "Attempting to Upload Sequence Data. Slot# "+(passedSeqID+1);

  //----------------------------------------- Pack & Send Header  -----------------------------------------

  SendSequenceHeader(passedSeqID); //send header using passedSeqID

  //----------------------------------------- Header Packed  --------------------------------------------

  float GlobalIntensity = (float)PApplet.parseInt(comGlobalIntensityField.value) / 100;  
  println("GlobalIntensity: "+GlobalIntensity);
  println(sequence[passedSeqID].totalFrames+"   "+matrix.patchedChannels);

  ColorOrderCounter = 0; //reset variable

  for (int q = 0; q < sequence[passedSeqID].totalFrames; q++)
  {
    //for (int i = 0; i != matrix.patchedChannels; i++)  BuildUSBPacket(int(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));

    for (int i = 0; i < UtilizedChannels; i+=3)
    {  
      switch(comHardwareColorReOrderDD.selStr)
      {
      case 0://RGB
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        break;
      case 1: //BRG
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));          
        break;
      case 2: //GBR
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));         
        break;
      case 3: //RBG
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));  
        break;
      case 4: //BGR
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));
        break;
      case 5: //GRB - WS2812B
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+1] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i] * GlobalIntensity));
        BuildUSBPacket(PApplet.parseInt(sequence[passedSeqID].dataFrames[q][i+2] * GlobalIntensity));
        break;
      } //end switch
    } //end for()
  }//end for()
} //end func

//================================================================================================

public void SendSequenceHeader(int passedSeqID)
{
  /*
  32-bit(?) First bytes should be length/or address, and that should be double checked against the Index address, make sure they match or bad data
   8-bit Mode, the mode alters the header and data storing methods - Stall - Fade - Gradient - Instant - Technical(Combo)- POV - Video - Linked
   16-bit Max Frames - SeqFrameAmount
   32-bit speed - for long instants
   16-bit Sequence Channels - MSB - LSB - SeqDataSize
   8-bit Sequence Flags (T2-T1-T0-P2-P1-P0-D1-D0)
   - (3-bits) Transistion In/Start: Fade Up/Down, Instant, 6 spots for others
   - (3-bits) One shot and stop at end values, one shot and stop at start values, one shot and blank, one shot and go to next sequence, standard loop  
   - (2-bits) Data Type Single/RGB/RGBW/EXTRA
   8-bit POV Flags OR Seqeunce Transition Speed Value
   - (4-Bits) POV Flag bits 
   - (4-Bits) Enable accmeter option
   8-bit Packet Clone value or hardware specific?
   = 16 bytes   
   */

  BreakValueForUSBPacket(sequence[passedSeqID].mappedROMAdr, 1);//break 32-bit number into 4 bytes
  //BuildUSBPacket(0); //Index ID MSB3
  //BuildUSBPacket(0); //MSB2
  //BuildUSBPacket(0); //LSB1
  //BuildUSBPacket(0); //LSB0

  BuildUSBPacket(mFilePlay); //no other sequence modes available on the Aurora Matrix Firmware

  BreakValueForUSBPacket((sequence[passedSeqID].totalFrames-1), 0); //break into 16-bit number

  BreakValueForUSBPacket(sequence[passedSeqID].speed, 1); //break into 32-bit number

  //BreakValueForUSBPacket(matrix.patchedChannels, 0);   //break into 16-bit number
  BreakValueForUSBPacket(UtilizedChannels, 0);   //break into 16-bit number

  //(TF2-TF1-TF0-PM2-PM1-PM0-DT1-DT0)
  int tempStorage = 0; //start with all 0s
  tempStorage = 1; //RGB - only color data type available - No RGBW or single color
  tempStorage = (sequence[passedSeqID].playMode << 2) | tempStorage;
  tempStorage = (sequence[passedSeqID].transition << 5) | tempStorage;
  BuildUSBPacket(tempStorage); //Send packed SeqFlags

  BuildUSBPacket(0); 
  BuildUSBPacket(0); //if true send 0 for now...  //Packet Cloning or Hardware Specific, seems to work on hardware
} //end SendSequenceHeader()

//================================================================================================

public void FullIndexUpload()
{
  println("FullUpload()");

  for (int q = 0; q < IndexedSequences; q++) BreakUSBValue(sequence[q].mappedROMAdr);//IndexMapArray[q]);    

  //Finish off index slots with 0000s
  for (int i = IndexedSequences; i != device.MaxIndexedSequences; i++)   BreakUSBValue(0);  

  println("Sent "+SendCounterB+" index bytes");

  //--------------------------------------- End Map Built --------------------------------------------------

  //Switch from IndexBlockSize to Seqblocksize
  device.CurrentBlockSize = device.SeqMemoryBlockSize;

  for (int i = 0; i < IndexedSequences; i++)
  {
    println("Running with "+i);
    SendSequence(i);
  }  //end for()

  println("TotalSent: "+SendCounterB);     
  println("Left to Send: "+(device.CurrentBlockSize - (SendCounter%device.CurrentBlockSize)));

  while ((float)(SendCounterB%device.CurrentBlockSize) != 0)
  {  
    BuildUSBPacket(33); //send dummy
  } 

  println("Indexed & Sequences Sent");
  UploadInProgress = false;  //do it here not from switch()  
  redraw();
}//end FullUpload()

//===================================================================================================  

public void SendConfigurations()
{
  println("SendConfigurations()"); 

  String[] ModuleString = new String[16];  
  int[] TransmitByteArray = new int[16];  
  DevStrArray = split(device.ConfigFlagsStr, ',');
  //Now single delimenated string is an array, the pointer is the bit number in ConfigFlags  

  //Module0 with matching string ex:   "fDisModeMSBa,fDisModeLSBa"
  //Module1 with matching string ex:   "fAutoDetectID,fAutoDetect"
  //Loop goes through enabled modules, checks the string to the DevStr array  
  //  If a module string matches the list build ConfigFlags

  ConfigFlags = 0; //reset flags here.....

  for (int y = 0; y != InputModule.length; y++)
  {  
    if (InputModule[y].Enabled == true)
    {
      ModuleString = split(InputModule[y].ModuleFlagStr, ',');

      for (int q = 0; q != ModuleString.length; q++)
      {  
        for (int i = 0; i != DevStrArray.length; i++)
        {  
          if (DevStrArray[i].equals(ModuleString[q]) == true) //if string matches
          {
            //println("String Matches "+ModuleString[q]+" on Bit ID# "+i+"   StrID: "+q);

            if (InputModule[y].GetBitValues(q) == 1) bitSet(i); //this pretty much does it gotta setup modules
          }
        } //end i for()
      } //end q for()
    } //end if
  } //end y for()

  println("ConfigFlags: "+binary(ConfigFlags, 16));

  for (int i = 0; i != TransmitByteArray.length; i++) TransmitByteArray[i] = 0; //clear array first
  TransmitByteArray[0] = (ConfigFlags >> 8); //MSB first
  TransmitByteArray[1] = ConfigFlags & 0xFF;  //LSB second
  ///NOW RUN CONFIGURATION BYTES  

  DevStrArray = split(device.ConfigBytesStr, ',');  //load device config byte string

  for (int y = 0; y != InputModule.length; y++)
  {    
    if (InputModule[y].Enabled == true)
    {
      ModuleString = split(InputModule[y].ModuleByteStr, ',');

      for (int q = 0; q != ModuleString.length; q++)
      {  
        for (int i = 0; i != DevStrArray.length; i++)
        {  
          if (DevStrArray[i].equals(ModuleString[q]) == true) //if string matches
          {
            //println("Set Byte - String Matches "+ModuleString[q]+" on BYTE# "+(i+2));
            TransmitByteArray[i+2] = InputModule[y].GetByteValues(q);
          }
        } //end i for()
      } //end q for()
    } //end if
  } //end y for()

  printArray(TransmitByteArray);

  if (device.BasicVersion == 0)
  {
    //Now Send TransmitByteArray
    for (int i = 0; i != (DevStrArray.length+2); i++)  SendSingleDataByte(TransmitByteArray[i]); //+2 for 2x config bytes
  } 
  println("SendConfigurations() finished");
} //end SendConfigurations()  

//=====================================================================================================
  public void settings() {  size(1024, 768); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "NLEDAuroraMatrix" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
